@@ -6,11 +6,7 @@
  *
  * released under GNU GPL v2 licence
  *
- * $Id: AddrClient.h,v 1.3 2004/09/08 21:22:45 thomson Exp $
- *
- * $Log: AddrClient.h,v $
- * Revision 1.3  2004/09/08 21:22:45  thomson
- * Parser improvements, signed/unsigned issues addressed.
+ * $Id: AddrClient.h,v 1.7 2007-01-27 17:09:31 thomson Exp $
  *
  */
 
@@ -39,14 +35,32 @@ public:
     bool delIA(unsigned long IAID);
     int countIA();
 
+    //--- PD list ---
+    void firstPD();
+    SmartPtr<TAddrIA> getPD();
+    SmartPtr<TAddrIA> getPD(unsigned long IAID);
+    void addPD(SmartPtr<TAddrIA> ia);
+    bool delPD(unsigned long IAID);
+    int countPD();
+
+    //--- TA list ---
+    void firstTA();
+    SmartPtr<TAddrIA> getTA();
+    SmartPtr<TAddrIA> getTA(unsigned long iaid);
+    void addTA(SmartPtr<TAddrIA> ia);
+    bool delTA(unsigned long iaid);
+    int countTA();
+
     // time related
     unsigned long getT1Timeout();
     unsigned long getT2Timeout();
     unsigned long getPrefTimeout();
     unsigned long getValidTimeout();
-
+    
 private:
-    TContainer< SmartPtr<TAddrIA> > IAsLst;
+    List(TAddrIA) IAsLst;
+    List(TAddrIA) TALst;
+    List(TAddrIA) PDLst;
     SmartPtr<TDUID> DUID;
 };
 
