@@ -3,38 +3,11 @@
  *                                                                           
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>                           
  *          Marek Senderski <msend@o2.pl>                                    
- *                                                                           
+ *  changes: Krzysztof Wnuk <keczi@poczta.onet.pl>                                                                         
  * released under GNU GPL v2 or later licence                                
  *                                                                           
- * $Id: ClntCfgIface.cpp,v 1.13 2005/01/03 21:53:01 thomson Exp $
+ * $Id: ClntCfgIface.cpp,v 1.26 2007-04-10 19:41:10 thomson Exp $
  *
- * $Log: ClntCfgIface.cpp,v $
- * Revision 1.13  2005/01/03 21:53:01  thomson
- * id is now called ifindex
- *
- * Revision 1.12  2004/11/30 00:42:50  thomson
- * Client no longer sends RapidCommit, unless told to do so (bug #55)
- *
- * Revision 1.11  2004/11/29 21:21:56  thomson
- * Client parser now supports 'option lifetime' directive (bug #75)
- *
- * Revision 1.10  2004/11/01 23:31:24  thomson
- * New options,option handling mechanism and option renewal implemented.
- *
- * Revision 1.9  2004/10/27 22:07:55  thomson
- * Signed/unsigned issues fixed, Lifetime option implemented, INFORMATION-REQUEST
- * message is now sent properly. Valid lifetime granted by server fixed.
- *
- * Revision 1.8  2004/10/25 20:45:52  thomson
- * Option support, parsers rewritten. ClntIfaceMgr now handles options.
- *
- * Revision 1.7  2004/10/02 13:11:24  thomson
- * Boolean options in config file now can be specified with YES/NO/TRUE/FALSE.
- * Unicast communication now can be enable on client side (disabled by default).
- *
- * Revision 1.6  2004/07/05 00:53:03  thomson
- * Various changes.
- *                                                                           
  */
 
 #include <iostream>
@@ -42,42 +15,47 @@
 #include "ClntCfgIface.h"
 #include "Logger.h"
 #include "Portable.h"
+#include "ClntOptVendorSpec.h"
 using namespace std;
 
 TClntCfgIface::TClntCfgIface(string ifaceName) {
     NoConfig=false;
     IfaceName=ifaceName;
     ID=-1;
-    this->DNSServerState  = DISABLED;
-    this->DomainState     = DISABLED;
-    this->NTPServerState  = DISABLED;
-    this->TimezoneState   = DISABLED;
-    this->SIPServerState  = DISABLED;
-    this->SIPDomainState  = DISABLED;
-    this->FQDNState       = DISABLED;
-    this->NISServerState  = DISABLED;
-    this->NISPServerState = DISABLED;
-    this->NISDomainState  = DISABLED;
-    this->NISPDomainState = DISABLED;
-    this->LifetimeState   = DISABLED;
+    this->DNSServerState  = STATE_DISABLED;
+    this->DomainState     = STATE_DISABLED;
+    this->NTPServerState  = STATE_DISABLED;
+    this->TimezoneState   = STATE_DISABLED;
+    this->SIPServerState  = STATE_DISABLED;
+    this->SIPDomainState  = STATE_DISABLED;
+    this->FQDNState       = STATE_DISABLED;
+    this->NISServerState  = STATE_DISABLED;
+    this->NISPServerState = STATE_DISABLED;
+    this->NISDomainState  = STATE_DISABLED;
+    this->NISPDomainState = STATE_DISABLED;
+    this->LifetimeState   = STATE_DISABLED;
+    this->PrefixDelegationState = STATE_DISABLED;
+    this->VendorSpecState = STATE_DISABLED;
 }
 
 TClntCfgIface::TClntCfgIface(int ifaceNr) {
     NoConfig=false;
     ID=ifaceNr;
     IfaceName="[unknown]";
-    this->DNSServerState  = DISABLED;
-    this->DomainState     = DISABLED;
-    this->NTPServerState  = DISABLED;
-    this->TimezoneState   = DISABLED;
-    this->SIPServerState  = DISABLED;
-    this->SIPDomainState  = DISABLED;
-    this->FQDNState       = DISABLED;
-    this->NISServerState  = DISABLED;
-    this->NISPServerState = DISABLED;
-    this->NISDomainState  = DISABLED;
-    this->NISPDomainState = DISABLED;
-    this->LifetimeState   = DISABLED;
+    this->DNSServerState  = STATE_DISABLED;
+    this->DomainState     = STATE_DISABLED;
+    this->NTPServerState  = STATE_DISABLED;
+    this->TimezoneState   = STATE_DISABLED;
+    this->SIPServerState  = STATE_DISABLED;
+    this->SIPDomainState  = STATE_DISABLED;
+    this->FQDNState       = STATE_DISABLED;
+    this->NISServerState  = STATE_DISABLED;
+    this->NISPServerState = STATE_DISABLED;
+    this->NISDomainState  = STATE_DISABLED;
+    this->NISPDomainState = STATE_DISABLED;
+    this->LifetimeState   = STATE_DISABLED;
+    this->PrefixDelegationState = STATE_DISABLED;
+    this->VendorSpecState = STATE_DISABLED;
 }
 
 void TClntCfgIface::setOptions(SmartPtr<TClntParsGlobalOpt> opt) {
@@ -98,6 +76,8 @@ void TClntCfgIface::setOptions(SmartPtr<TClntParsGlobalOpt> opt) {
     ReqNISPServer= opt->getReqNISPServer();
     ReqNISPDomain= opt->getReqNISPDomain();
     ReqLifetime  = opt->getReqLifetime();
+    ReqPrefixDelegation = opt->getReqPrefixDelegation();
+    ReqVendorSpec= opt->getReqVendorSpec();
 
     // copy parameters
     this->DNSServerLst = *opt->getDNSServerLst();
@@ -111,67 +91,163 @@ void TClntCfgIface::setOptions(SmartPtr<TClntParsGlobalOpt> opt) {
     this->NISDomain    = opt->getNISDomain();
     this->NISPServerLst= *opt->getNISPServerLst();
     this->NISPDomain   = opt->getNISPDomain();
+    this->VendorSpec   = opt->getVendorSpec();
 
-    if (ReqDNSServer)  this->setDNSServerState(NOTCONFIGURED);
-    if (ReqDomain)     this->setDomainState(NOTCONFIGURED);
-    if (ReqNTPServer)  this->setNTPServerState(NOTCONFIGURED);
-    if (ReqTimezone)   this->setTimezoneState(NOTCONFIGURED);
-    if (ReqSIPServer)  this->setSIPServerState(NOTCONFIGURED);
-    if (ReqSIPDomain)  this->setSIPDomainState(NOTCONFIGURED);
-    if (ReqFQDN)       this->setFQDNState(NOTCONFIGURED);
-    if (ReqNISServer)  this->setNISServerState(NOTCONFIGURED);
-    if (ReqNISDomain)  this->setNISDomainState(NOTCONFIGURED);
-    if (ReqNISPServer) this->setNISPServerState(NOTCONFIGURED);
-    if (ReqNISPDomain) this->setNISPDomainState(NOTCONFIGURED);
-    if (ReqLifetime)   this->setLifetimeState(NOTCONFIGURED);
+    if (ReqDNSServer)  this->setDNSServerState(STATE_NOTCONFIGURED);
+    if (ReqDomain)     this->setDomainState(STATE_NOTCONFIGURED);
+    if (ReqNTPServer)  this->setNTPServerState(STATE_NOTCONFIGURED);
+    if (ReqTimezone)   this->setTimezoneState(STATE_NOTCONFIGURED);
+    if (ReqSIPServer)  this->setSIPServerState(STATE_NOTCONFIGURED);
+    if (ReqSIPDomain)  this->setSIPDomainState(STATE_NOTCONFIGURED);
+    if (ReqFQDN)       this->setFQDNState(STATE_NOTCONFIGURED);
+    if (ReqNISServer)  this->setNISServerState(STATE_NOTCONFIGURED);
+    if (ReqNISDomain)  this->setNISDomainState(STATE_NOTCONFIGURED);
+    if (ReqNISPServer) this->setNISPServerState(STATE_NOTCONFIGURED);
+    if (ReqNISPDomain) this->setNISPDomainState(STATE_NOTCONFIGURED);
+    if (ReqLifetime)   this->setLifetimeState(STATE_NOTCONFIGURED);
+    if (ReqVendorSpec) this->setVendorSpecState(STATE_NOTCONFIGURED);
+    // copy preferred-server list
+    SmartPtr<TStationID> station;
+    opt->firstPrefSrv();
+    while (station = opt->getPrefSrv())
+	this->PrefSrvLst.append(station);
+
+    // copy rejected-server list
+    opt->firstRejedSrv();
+    while (station = opt->getRejedSrv())
+	this->RejectedSrvLst.append(station);
 }
 
- void TClntCfgIface::firstGroup()
+bool TClntCfgIface::isServerRejected(SmartPtr<TIPv6Addr> addr,SmartPtr<TDUID> duid)
 {
-    ClntCfgGroupLst.first();
+    this->RejectedSrvLst.first();
+    SmartPtr<TStationID> RejectedSrv;
+    while(RejectedSrv=RejectedSrvLst.get())
+    {
+        if (((*RejectedSrv)==addr)||((*RejectedSrv)==duid))
+            return true;
+    }
+    return false;
 }
 
- int TClntCfgIface::countGroup()
-{
-    return ClntCfgGroupLst.count();
+void TClntCfgIface::firstTA() {
+    this->ClntCfgTALst.first();
 }
 
- SmartPtr<TClntCfgGroup> TClntCfgIface::getGroup()
-{
-    return ClntCfgGroupLst.get();
+SmartPtr<TClntCfgTA> TClntCfgIface::getTA() {
+    return this->ClntCfgTALst.get();
 }
 
- void TClntCfgIface::addGroup(SmartPtr<TClntCfgGroup> ptr)
-{
-    ClntCfgGroupLst.append(ptr);
+void  TClntCfgIface::addTA(SmartPtr<TClntCfgTA> ta) {
+    this->ClntCfgTALst.append(ta);
 }
 
- SmartPtr<TClntCfgGroup> TClntCfgIface::getLastGroup()
+int TClntCfgIface::countTA()
 {
-    return ClntCfgGroupLst.getLast();
+    return ClntCfgTALst.count();
 }
 
+void TClntCfgIface::firstIA()
+{
+    IALst.first();
+}
+
+int TClntCfgIface::countIA()
+{
+    return IALst.count();
+}
+
+ SmartPtr<TClntCfgIA> TClntCfgIface::getIA()
+{
+    return IALst.get();
+}
+
+SPtr<TClntCfgIA> TClntCfgIface::getIA(int iaid)
+{
+    SPtr<TClntCfgIA> ia;
+    IALst.first();
+    while (ia = IALst.get() ) {
+	if (ia->getIAID() == iaid)
+	    return ia;
+    }
+    return 0;
+}
+
+
+ void TClntCfgIface::addIA(SmartPtr<TClntCfgIA> ptr)
+{
+    IALst.append(ptr);
+}
+
+void TClntCfgIface::firstPD()
+{
+    PDLst.first();
+}
+
+int TClntCfgIface::countPD()
+{
+    return PDLst.count();
+}
+
+ SmartPtr<TClntCfgPD> TClntCfgIface::getPD()
+{
+    return PDLst.get();
+}
+
+SPtr<TClntCfgPD> TClntCfgIface::getPD(int iaid)
+{
+    SPtr<TClntCfgPD> ia;
+    PDLst.first();
+    while (ia = PDLst.get() ) {
+	if (ia->getIAID() == iaid)
+	    return ia;
+    }
+    return 0;
+}
+
+void TClntCfgIface::addPD(SmartPtr<TClntCfgPD> ptr)
+{
+    PDLst.append(ptr);
+}
+
+
+#if 0
+SmartPtr<TClntCfgIA> TClntCfgIface::getLastIA()
+{
+    return IALst.getLast();
+}
+#endif
+
  string TClntCfgIface::getName(void)
 {
     return IfaceName;    
 }
 
- int	TClntCfgIface::getID(void)
+string TClntCfgIface::getFullName() {
+    ostringstream oss;
+    oss << this->ID;
+    return string(this->IfaceName)
+	+"/"
+	+oss.str();
+}
+
+
+int TClntCfgIface::getID(void)
 {
     return ID;
 }
 
- void TClntCfgIface::setIfaceID(int ifaceID)
+void TClntCfgIface::setIfaceID(int ifaceID)
 {
     ID=ifaceID;
 }
 
- void TClntCfgIface::setIfaceName(string ifaceName)
+void TClntCfgIface::setIfaceName(string ifaceName)
 {
     this->IfaceName=ifaceName;
 }
 
- void TClntCfgIface::setNoConfig()
+void TClntCfgIface::setNoConfig()
 {
     NoConfig=true;
 }
@@ -193,12 +269,21 @@ bool TClntCfgIface::getRapidCommit() {
     return this->RapidCommit;
 }
 
+void TClntCfgIface::vendorSpecSupported(bool support)
+{
+    this->ReqVendorSpec   = support;
+    this->VendorSpecState = STATE_NOTCONFIGURED;
+}
+
 // --------------------------------------------------------------------------------
 // --- options below --------------------------------------------------------------
 // --------------------------------------------------------------------------------
 bool TClntCfgIface::isReqDNSServer() {
     return this->ReqDNSServer;
 }
+bool TClntCfgIface::isReqPrefixDelegation() {
+    return this->ReqPrefixDelegation;
+}
 bool TClntCfgIface::isReqDomain() {
     return this->ReqDomain;
 }
@@ -232,6 +317,9 @@ bool TClntCfgIface::isReqNISPDomain() {
 bool TClntCfgIface::isReqLifetime() {
     return this->ReqLifetime;
 }
+bool TClntCfgIface::isReqVendorSpec() {
+    return this->ReqVendorSpec;
+}
 
 // --------------------------------------------------------------------------------
 // --- options: state -------------------------------------------------------------
@@ -273,6 +361,9 @@ EState TClntCfgIface::getLifetimeState() {
     return LifetimeState;
 }
 
+EState TClntCfgIface::getVendorSpecState() {
+    return VendorSpecState;
+}
 // --------------------------------------------------------------------
 // --- options: get option --------------------------------------------
 // --------------------------------------------------------------------
@@ -310,6 +401,14 @@ string TClntCfgIface::getProposedNISPDomain() {
     return this->NISPDomain;
 }
 
+void TClntCfgIface::firstVendorSpec() {
+    VendorSpec.first();
+}
+
+SPtr<TClntOptVendorSpec> TClntCfgIface::getVendorSpec() {
+    return this->VendorSpec.get();
+}
+
 // --------------------------------------------------------------------
 // --- options: setState ----------------------------------------------
 // --------------------------------------------------------------------
@@ -350,6 +449,20 @@ void TClntCfgIface::setLifetimeState(EState state) {
     this->LifetimeState=state;
 }
 
+void TClntCfgIface::setVendorSpecState(EState state) {
+    this->VendorSpecState = state;
+}
+
+void TClntCfgIface::setPrefixLength(int len) {
+    this->PrefixLength = len;
+}
+
+int  TClntCfgIface::getPrefixLength() {
+    return this->PrefixLength;
+}
+
+
+
 // --------------------------------------------------------------------
 // --- operators ------------------------------------------------------
 // --------------------------------------------------------------------
@@ -376,22 +489,37 @@ ostream& operator<<(ostream& out,TClntCfgIface& iface)
     }
 
     out << "    <!-- addresses -->" << endl;
-    out << "    <groups count=\"" << iface.ClntCfgGroupLst.count() << "\">" << endl;
-    SmartPtr<TClntCfgGroup>	groupPtr;
-    iface.ClntCfgGroupLst.first();
-    while(groupPtr=iface.ClntCfgGroupLst.get())
+    out << "    <iaLst count=\"" << iface.IALst.count() << "\">" << endl;
+    SmartPtr<TClntCfgIA> ia;
+    iface.IALst.first();
+    while(ia=iface.IALst.get())
     {	
-        out << *groupPtr;
+        out << *ia;
     }
-    out << "    </groups>" << endl;
+    out << "    </iaLst>" << endl;
 
+    out << "    <!-- temporary addresses -->" << endl;
+    SmartPtr<TClntCfgTA> ta;
+    iface.firstTA();
+    while (ta = iface.getTA() ) {
+	out << *ta;
+    }
 
     out << "    <!-- options -->" << endl;
 
+    out << "    <!-- prefix delegation -->" << endl;
+    out << "    <pdLst count=\"" << iface.countPD() << "\">" << endl;
+    SmartPtr<TClntCfgPD> pd;
+    iface.firstPD();
+    while (pd = iface.getPD()) {
+	out << *pd;
+    }
+    out << "    </pdLst>" << endl;
+
     // --- option: DNS-servers ---
     if (iface.isReqDNSServer()) {
 	out << "    <dns-servers state=\"" << StateToString(iface.getDNSServerState())
-	    << "\" state=\"" << iface.DNSServerLst.count() << "\" />" << endl;
+	    << "\" hints=\"" << iface.DNSServerLst.count() << "\" />" << endl;
 
 	iface.DNSServerLst.first();
 	while(addr=iface.DNSServerLst.get())
@@ -506,6 +634,20 @@ ostream& operator<<(ostream& out,TClntCfgIface& iface)
 	out << "    <!-- <lifetime/> -->" << endl;
     }
 
+    // --- option: Vendor-spec ---
+    if (iface.isReqVendorSpec()) {
+	SPtr<TClntOptVendorSpec> opt;
+	out << "    <vendorSpecLst count=\"" << iface.VendorSpec.count() << "\">" << endl;
+	iface.VendorSpec.first();
+	while (opt = iface.VendorSpec.get()) {
+	    out << "      <vendor enterprise=\"" << opt->getVendor() << "\" length=\"" << opt->getVendorDataLen() << "\">"
+		<< opt->getVendorDataPlain() << "</vendor>" << endl;
+	}
+	out << "    <vendorSpecLst/>" << endl;
+    } else {
+	out << "    <!-- <vendorSpecLst/> -->" << endl;
+    }
+
     out << "  </ClntCfgIface>" << endl;
 
     return out;
