@@ -3,53 +3,32 @@
  *                                                                           
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>                           
  *          Marek Senderski <msend@o2.pl>                                    
- *                                                                           
+ * changes: Krzysztof Wnuk <keczi@poczta.onet.pl>                                                                         
  * released under GNU GPL v2 or later licence                                
  *                                                                           
- * $Id: ClntCfgIface.h,v 1.8 2004/11/30 00:42:50 thomson Exp $
- *
- * $Log: ClntCfgIface.h,v $
- * Revision 1.8  2004/11/30 00:42:50  thomson
- * Client no longer sends RapidCommit, unless told to do so (bug #55)
- *
- * Revision 1.7  2004/11/29 21:21:56  thomson
- * Client parser now supports 'option lifetime' directive (bug #75)
- *
- * Revision 1.6  2004/10/27 22:07:55  thomson
- * Signed/unsigned issues fixed, Lifetime option implemented, INFORMATION-REQUEST
- * message is now sent properly. Valid lifetime granted by server fixed.
- *
- * Revision 1.5  2004/10/25 20:45:52  thomson
- * Option support, parsers rewritten. ClntIfaceMgr now handles options.
- *
- * Revision 1.4  2004/10/02 13:11:24  thomson
- * Boolean options in config file now can be specified with YES/NO/TRUE/FALSE.
- * Unicast communication now can be enable on client side (disabled by default).
- *
- * Revision 1.3  2004/05/23 20:13:12  thomson
- * *** empty log message ***
- *
- *                                                                           
+ * $Id: ClntCfgIface.h,v 1.18 2007-01-27 17:11:10 thomson Exp $
  */
 
+class TClntCfgIface;
 #ifndef CLNTCFGIFACE_H
 #define CLNTCFGIFACE_H
 
 #include <iostream>
 #include <iomanip>
+#include <sstream>
 
 #include "Container.h"
 #include "StationID.h"
 #include "SmartPtr.h"
 #include "DHCPConst.h"
 #include "IPv6Addr.h"
-#include "ClntCfgGroup.h"
+#include "ClntCfgTA.h"
 #include "ClntParsGlobalOpt.h"
 #include "SmartPtr.h"
 #include "DUID.h"
-#include "IPv6Addr.h"
-
-using namespace std;
+#include "ClntCfgIA.h"
+#include "ClntCfgPD.h"
+#include "ClntOptVendorSpec.h"
 
 class TClntCfgIface
 {
@@ -58,14 +37,31 @@ public:
     TClntCfgIface(string ifaceName);
     TClntCfgIface(int ifaceNr);
 
-    TContainer< SmartPtr<TClntCfgGroup> > ClntCfgGroupLst;
-    void firstGroup();
-    int countGroup();
-    SmartPtr<TClntCfgGroup> getGroup();
-    void addGroup(SmartPtr<TClntCfgGroup> ptr);
+    bool isServerRejected(SmartPtr<TIPv6Addr> addr,SmartPtr<TDUID> duid);
+
+    // IA
+    void firstIA();
+    int countIA();
+    SPtr<TClntCfgIA> getIA();
+    SPtr<TClntCfgIA> getIA(int iaid);
+    void addIA(SmartPtr<TClntCfgIA> ptr);
+
+    // PD
+    void firstPD();
+    int countPD();
+    SPtr<TClntCfgPD> getPD();
+    SPtr<TClntCfgPD> getPD(int iaid);
+    void addPD(SmartPtr<TClntCfgPD> ptr);
+    
+    // TA
+    void firstTA();
+    void addTA(SmartPtr<TClntCfgTA> ta);
+    SmartPtr<TClntCfgTA> getTA();
+    int countTA();
+
     string getName(void);
+    string getFullName(void);
     void setOptions(SmartPtr<TClntParsGlobalOpt> opt);
-    SmartPtr<TClntCfgGroup> getLastGroup();
     int	getID(void);
     void setNoConfig();
     void setIfaceID(int ifaceID);
@@ -160,6 +156,20 @@ public:
     EState getLifetimeState();
     void setLifetimeState(EState state);
 
+    // --- option: Prefix Delegation ---
+    bool isReqPrefixDelegation();
+    void setPrefixLength(int len);
+    int  getPrefixLength();
+
+    // --- option: VendorSpec ---
+    bool isReqVendorSpec();
+    void vendorSpecSupported(bool support);
+    EState getVendorSpecState();
+    void setVendorSpecState(EState state);
+    void firstVendorSpec();
+    SPtr<TClntOptVendorSpec> getVendorSpec();
+    int getVendorSpecCount();
+
 private:
     string IfaceName;
     int ID;
@@ -167,6 +177,14 @@ private:
     bool isIA;
     bool Unicast;
     bool RapidCommit;
+    int  PrefixLength; // default prefix length of the configured addresses
+
+    List(TClntCfgIA) IALst;
+    List(TClntCfgPD) PDLst;
+    List(TStationID) PrefSrvLst;
+    List(TStationID) RejectedSrvLst;
+
+    List(TClntCfgTA) ClntCfgTALst;
     
     List(TIPv6Addr) DNSServerLst;
     List(string) DomainLst;
@@ -179,6 +197,7 @@ private:
     List(TIPv6Addr) NISPServerLst;
     string NISDomain;
     string NISPDomain;
+    List(TClntOptVendorSpec) VendorSpec;
     
     EState DNSServerState;
     EState DomainState;
@@ -192,6 +211,8 @@ private:
     EState NISDomainState;
     EState NISPDomainState;
     EState LifetimeState;
+    EState PrefixDelegationState;
+    EState VendorSpecState;
 
     bool ReqDNSServer;
     bool ReqDomain;
@@ -205,6 +226,8 @@ private:
     bool ReqNISDomain;
     bool ReqNISPDomain;
     bool ReqLifetime;
+    bool ReqPrefixDelegation;
+    bool ReqVendorSpec;
 };
 
 #endif 
