@@ -6,35 +6,51 @@
  *                                                                           
  * released under GNU GPL v2 or later licence                                
  *                                                                           
- * $Id: ClntParsGlobalOpt.h,v 1.4 2004/10/25 20:45:52 thomson Exp $
+ * $Id: ClntParsGlobalOpt.h,v 1.11 2007-04-22 21:19:27 thomson Exp $
  *
- * $Log: ClntParsGlobalOpt.h,v $
- * Revision 1.4  2004/10/25 20:45:52  thomson
- * Option support, parsers rewritten. ClntIfaceMgr now handles options.
- *
- * Revision 1.3  2004/05/23 22:37:54  thomson
- * *** empty log message ***
- *
- *                                                                           
  */
 
+class TClntParsGlobalOpt;
 #ifndef PARSGLOBALOPT_H
 #define PARSGLOBALOPT_H
 
 #include "ClntParsIfaceOpt.h"
+#include "DHCPConst.h"
 
 class TClntParsGlobalOpt : public TClntParsIfaceOpt
 {
 public:
     TClntParsGlobalOpt();
-    TClntParsGlobalOpt(TClntParsGlobalOpt *old);
     ~TClntParsGlobalOpt();
 
     void setWorkDir(string dir);
     string getWorkDir();
+    void setPrefixLength(int len);
+    int getPrefixLength();
+    void setDigest(DigestTypes digest);
+    DigestTypes getDigest();
+    void setScriptsDir(string dir);
+    string getScriptsDir();
+    void setAnonInfRequest(bool anonymous);
+    bool getAnonInfRequest();
+    void setInsistMode(bool insist);
+    bool getInsistMode();
+    void setInactiveMode(bool flex);
+    bool getInactiveMode();
+
+    void setExperimental();
+    bool getExperimental();
 
 private:
     string WorkDir;
+    int PrefixLength;
+    DigestTypes Digest;
+    string ScriptsDir;
+    bool   AnonInfRequest;
+    bool   InactiveMode;
+    bool   InsistMode;
+
+    bool Experimental;
 };
 
 #endif
