@@ -3,7 +3,7 @@
  *
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>
  *          Marek Senderski <msend@o2.pl>
- *
+ *changes : Krzysztof Wnuk <keczi@poczta.onet.pl>
  * released under GNU GPL v2 or later licence
  */
 
@@ -32,7 +32,6 @@ class TClntParsIAOpt : public TClntParsAddrOpt
     
     long getIAIDCnt();
     void setIAIDCnt(long cnt);
-    long getIncedIAIDCnt();
     
     bool getAddrHint();
     void setAddrHint(bool addrHint);
@@ -48,14 +47,19 @@ class TClntParsIAOpt : public TClntParsAddrOpt
     SmartPtr<TStationID> getRejedSrv();
     void clearRejedSrv();
     void setRejedSrvLst(TContainer<SmartPtr<TStationID> > *lst);
+
+    void setAddrParams(bool useAddrParams);
+    bool getAddrParams();
 private:
     long T1;
     long T2;
     
     long IAIDCnt;
     bool AddrHint;
-    TContainer<SmartPtr<TStationID> > PrefSrv;
-    TContainer<SmartPtr<TStationID> > RejedSrv;
+    List(TStationID) PrefSrv;
+    List(TStationID) RejedSrv;
+
+    bool AddrParams;
 };
 
 #endif
