@@ -6,31 +6,7 @@
  *
  * released under GNU GPL v2 or later licence
  *
- * $Id: ClntIfaceIface.cpp,v 1.7 2005/07/21 20:42:42 thomson Exp $
- *
- * $Log: ClntIfaceIface.cpp,v $
- * Revision 1.7  2005/07/21 20:42:42  thomson
- * Minor change for wingw compatibility in win2k port,
- * provided by sob@hisoftware.cz.
- *
- * Revision 1.6  2005/07/17 21:09:52  thomson
- * Minor improvements for 0.4.1 release.
- *
- * Revision 1.5  2005/01/25 00:32:26  thomson
- * Global addrs support added.
- *
- * Revision 1.4  2005/01/03 21:55:47  thomson
- * Minor print change.
- *
- * Revision 1.3  2004/11/01 23:31:24  thomson
- * New options,option handling mechanism and option renewal implemented.
- *
- * Revision 1.2  2004/10/27 22:07:55  thomson
- * Signed/unsigned issues fixed, Lifetime option implemented, INFORMATION-REQUEST
- * message is now sent properly. Valid lifetime granted by server fixed.
- *
- * Revision 1.1  2004/10/25 20:45:53  thomson
- * Option support, parsers rewritten. ClntIfaceMgr now handles options.
+ * $Id: ClntIfaceIface.cpp,v 1.12 2007-04-22 21:28:27 thomson Exp $
  *
  */
 
@@ -372,8 +348,9 @@ bool TClntIfaceIface::setSIPDomainLst(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr>
 }
 
 bool TClntIfaceIface::setFQDN(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr> srv, string fqdn) {
-    // FIXME: Implement FQDN
-    Log(Error) << "FQDN option is not fully supported yet [" << fqdn << "]" << LogEnd;
+    this->FQDN = fqdn;
+    this->FQDNDUID = duid;
+    this->FQDNAddr = srv;
     return true;
 }
 
@@ -382,7 +359,7 @@ bool TClntIfaceIface::setNISServerLst(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr>
     SmartPtr<TIPv6Addr> old, addr;
     this->NISServerLst.first();
     while (old = this->NISServerLst.get()) {
-	// for each already set server...
+        // for each already set server...
 	addrs.first();
 	bool found = false;
 	while (addr = addrs.get()) {
@@ -621,7 +598,7 @@ void TClntIfaceIface::removeAllOpts() {
 	Log(Notice) << "NTP server " << *addr << " removed from the "
 		    << this->getName() << "/" << this->getID() <<" interface." << LogEnd;
 	this->NTPServerLst.del();
-	dns_del(this->getName(), this->getID(), addr->getPlain());
+	ntp_del(this->getName(), this->getID(), addr->getPlain());
     }
 
     // --- option: TIMEZONE ---
@@ -637,7 +614,7 @@ void TClntIfaceIface::removeAllOpts() {
 	Log(Notice) << "SIP server " << *addr << " removed from the "
 		    << this->getName() << "/" << this->getID() <<" interface." << LogEnd;
 	this->SIPServerLst.del();
-	dns_del(this->getName(), this->getID(), addr->getPlain());
+	sipserver_del(this->getName(), this->getID(), addr->getPlain());
     }
 
     // --- option: SIP-DOMAIN ---
@@ -655,7 +632,7 @@ void TClntIfaceIface::removeAllOpts() {
 	Log(Notice) << "NIS server " << *addr << " removed from the "
 		    << this->getName() << "/" << this->getID() <<" interface." << LogEnd;
 	this->NISServerLst.del();
-	dns_del(this->getName(), this->getID(), addr->getPlain());
+	nisserver_del(this->getName(), this->getID(), addr->getPlain());
     }
 
     // --- option: NIS-DOMAIN ---
@@ -683,6 +660,14 @@ void TClntIfaceIface::removeAllOpts() {
     }
 }
 
+string TClntIfaceIface::getFQDN() {
+	return FQDN;
+}
+
+List(TIPv6Addr) TClntIfaceIface::getDNSServerLst() {
+	return DNSServerLst;
+}
+
 // --------------------------------------------------------------------
 // --- operators ------------------------------------------------------
 // --------------------------------------------------------------------
@@ -698,6 +683,7 @@ ostream & operator <<(ostream & strum, TClntIfaceIface &x) {
     strum << "  <ClntIfaceIface";
     strum << " name=\"" << x.Name << "\"";
     strum << " ifindex=\"" << x.ID << "\"";
+    strum << " hwType=\"" << x.getHardwareType() << "\"";
     strum << " flags=\"" << x.Flags << "\">" << endl;
     strum << "    <!-- " << x.LLAddrCnt << " link scoped addrs -->" << endl;
 
