@@ -6,30 +6,13 @@
  *
  * released under GNU GPL v2 or later licence
  *
- * $Id: ClntIfaceMgr.h,v 1.6 2005/01/08 16:52:03 thomson Exp $
- *
- * $Log: ClntIfaceMgr.h,v $
- * Revision 1.6  2005/01/08 16:52:03  thomson
- * Relay support implemented.
- *
- * Revision 1.5  2004/12/07 00:45:41  thomson
- * Clnt managers creation unified and cleaned up.
- *
- * Revision 1.4  2004/12/01 20:55:17  thomson
- * Obsolete definitions removed.
- *
- * Revision 1.3  2004/10/27 22:07:55  thomson
- * Signed/unsigned issues fixed, Lifetime option implemented, INFORMATION-REQUEST
- * message is now sent properly. Valid lifetime granted by server fixed.
- *
- * Revision 1.2  2004/10/25 20:45:53  thomson
- * Option support, parsers rewritten. ClntIfaceMgr now handles options.
- *
+ * $Id: ClntIfaceMgr.h,v 1.12 2007-04-01 04:53:19 thomson Exp $
  *
  */
 
 class TClntIfaceMgr;
 class TClntMsg;
+class TClntIfaceIface;
 #ifndef CLNTIFACEMGR_H
 #define CLNTIFACEMGR_H
 
@@ -42,6 +25,12 @@ class TClntMsg;
 #include "IPv6Addr.h"
 #include "ClntMsg.h"
 
+typedef enum {
+    PREFIX_MODIFY_ADD,
+    PREFIX_MODIFY_UPDATE,
+    PREFIX_MODIFY_DEL
+} PrefixModifyMode;
+
 class TClntIfaceMgr : public TIfaceMgr
 {
  public:
@@ -56,17 +45,30 @@ class TClntIfaceMgr : public TIfaceMgr
     
     SmartPtr<TClntMsg> select(unsigned int timeout);
 
-    void setThats(SmartPtr<TClntIfaceMgr> clntIfaceMgr,
-		  SmartPtr<TClntTransMgr> clntTransMgr,
-		  SmartPtr<TClntCfgMgr>   clntCfgMgr,
-		  SmartPtr<TClntAddrMgr>  clntAddrMgr);
+    void setContext(SmartPtr<TClntIfaceMgr> clntIfaceMgr,
+		    SmartPtr<TClntTransMgr> clntTransMgr,
+		    SmartPtr<TClntCfgMgr>   clntCfgMgr,
+		    SmartPtr<TClntAddrMgr>  clntAddrMgr);
+
+    bool fqdnAdd(SmartPtr<TClntIfaceIface> iface, string domainname);
+    bool fqdnDel(SmartPtr<TClntIfaceIface> iface, SmartPtr<TAddrIA> ia, string domainname);
+
+    bool addPrefix   (int iface, SPtr<TIPv6Addr> prefix, int prefixLen, unsigned int pref, unsigned int valid);
+    bool updatePrefix(int iface, SPtr<TIPv6Addr> prefix, int prefixLen, unsigned int pref, unsigned int valid);
+    bool delPrefix   (int iface, SPtr<TIPv6Addr> prefix, int prefixLen);
 
     // --- option related ---
     void removeAllOpts();
 
     unsigned int getTimeout();
+    
+    bool doDuties();
+
+    void redetectIfaces();
 
   private:
+    bool modifyPrefix(int iface, SPtr<TIPv6Addr> prefix, int prefixLen, unsigned int pref, unsigned int valid, PrefixModifyMode mode);
+
     string XmlFile;
     SmartPtr<TClntCfgMgr> ClntCfgMgr;
     SmartPtr<TClntAddrMgr> ClntAddrMgr;
