@@ -18,6 +18,7 @@
 #include "SmartPtr.h"
 #include "Opt.h"
 
+
 class TClntMsg : public TMsg
 {
 public:
@@ -36,20 +37,25 @@ public:
     void send();
 
     //answer for a specific message
-    virtual void answer(SmartPtr<TClntMsg> Rep) = 0;
     virtual void doDuties() = 0;
     virtual bool check() = 0;
+    void setIface(int iface); // used to override when we have received msg via loopback interface.
 
+    void appendTAOptions(bool switchToInProcess); // append all TAs, which are currently in the NOTCONFIGURED state
+//    void appendPDOptions(bool switchToInProcess); // append all PDs, which are currently in the NOTCONFIGURED state
     void appendRequestedOptions();
-    
     SmartPtr<TClntTransMgr>  getClntTransMgr();
     SmartPtr<TClntAddrMgr>   getClntAddrMgr();
     SmartPtr<TClntCfgMgr>    getClntCfgMgr();
     SmartPtr<TClntIfaceMgr>  getClntIfaceMgr();
 
     virtual string getName() = 0;
+    virtual void answer(SPtr<TClntMsg> reply);
 
  protected:
+    bool check(bool clntIDmandatory, bool srvIDmandatory);
+    bool appendClientID();
+
     long IRT;           // Initial Retransmission Time
     long MRT;           // Maximum Retransmission Time
     long MRC;           // Maximum Retransmission Count
@@ -62,6 +68,7 @@ public:
     SmartPtr<TClntAddrMgr>   ClntAddrMgr;
     SmartPtr<TClntCfgMgr>    ClntCfgMgr;
     SmartPtr<TClntIfaceMgr>  ClntIfaceMgr;
+
  private:
     void setAttribs(SmartPtr<TClntIfaceMgr> IfaceMgr, 
 		    SmartPtr<TClntTransMgr> TransMgr, 
