@@ -6,18 +6,7 @@
  *
  * released under GNU GPL v2 or later licence
  *
- * $Id: ClntMsgConfirm.cpp,v 1.4 2005/01/08 16:52:03 thomson Exp $
- *
- * $Log: ClntMsgConfirm.cpp,v $
- * Revision 1.4  2005/01/08 16:52:03  thomson
- * Relay support implemented.
- *
- * Revision 1.3  2004/12/02 00:51:04  thomson
- * Log files are now always created (bugs #34, #36)
- *
- * Revision 1.2  2004/06/20 17:51:48  thomson
- * getName() method implemented, comment cleanup
- *
+ * $Id: ClntMsgConfirm.cpp,v 1.8 2007-04-22 21:19:28 thomson Exp $
  *
  */
 
@@ -95,7 +84,7 @@ void TClntMsgConfirm::addrsAccepted() {
     SmartPtr<TOpt> ptrOpt;
     this->firstOption();
     while ( ptrOpt = this->getOption() ) {
-	if (ptrOpt->getOptType()!=OPTION_IA)
+	if (ptrOpt->getOptType()!=OPTION_IA_NA)
 	    continue;
 
 	SmartPtr<TClntOptIA_NA> ptrOptIA = (Ptr*) ptrOpt;
@@ -106,7 +95,7 @@ void TClntMsgConfirm::addrsAccepted() {
 	// set them to RENEW timeout
 	unsigned long ts =now()-ptrIA->getT1();
 	ptrIA->setTimestamp(ts);
-	ptrIA->setState(CONFIGURED);
+	ptrIA->setState(STATE_CONFIGURED);
     }
     ClntAddrMgr->firstIA();
 }
@@ -116,7 +105,7 @@ void TClntMsgConfirm::addrsRejected() {
     SmartPtr<TOpt> ptrOpt;
     this->firstOption();
     while ( ptrOpt = this->getOption() ) {
-	if (ptrOpt->getOptType()!=OPTION_IA)
+	if (ptrOpt->getOptType()!=OPTION_IA_NA)
 	    continue;
 
 	SmartPtr<TClntOptIA_NA> ptrOptIA = (Ptr*) ptrOpt;
@@ -130,18 +119,18 @@ void TClntMsgConfirm::addrsRejected() {
 	if (!ptrIface) {
 	    Log(Crit) << "We have addresses assigned to non-existing interface."
 		"Help! Somebody stole an interface!" << LogEnd;
-	    ptrIA->setState(NOTCONFIGURED);
+	    ptrIA->setState(STATE_NOTCONFIGURED);
 	    return;
 	}
 	SmartPtr<TAddrAddr> ptrAddr;
 	ptrIA->firstAddr();
 	while (ptrAddr = ptrIA->getAddr() ) {
 	    // remove addr from ...
-	    ptrIface->delAddr( ptrAddr->get() );
+	    ptrIface->delAddr( ptrAddr->get(), ptrIface->getPrefixLength() );
 	    // ... and from DB
 	    ptrIA->delAddr( ptrAddr->get() );
 	}
-	ptrIA->setState(NOTCONFIGURED);
+	ptrIA->setState(STATE_NOTCONFIGURED);
     }
     ClntAddrMgr->firstIA();
 }
