@@ -6,24 +6,7 @@
  *
  * released under GNU GPL v2 or later licence
  *
- * $Id: ClntMsgRebind.cpp,v 1.6 2005/01/08 16:52:03 thomson Exp $
- *
- * $Log: ClntMsgRebind.cpp,v $
- * Revision 1.6  2005/01/08 16:52:03  thomson
- * Relay support implemented.
- *
- * Revision 1.5  2004/12/08 00:15:49  thomson
- * Issues with denied RENEW (bug #53), code clean up
- *
- * Revision 1.4  2004/12/02 00:51:04  thomson
- * Log files are now always created (bugs #34, #36)
- *
- * Revision 1.3  2004/09/07 22:02:32  thomson
- * pref/valid/IAID is not unsigned, RAPID-COMMIT now works ok.
- *
- * Revision 1.2  2004/06/20 17:51:48  thomson
- * getName() method implemented, comment cleanup
- *
+ * $Id: ClntMsgRebind.cpp,v 1.13 2007-05-04 17:22:49 thomson Exp $
  *
  */
 
@@ -31,6 +14,7 @@
 #include "ClntMsg.h"
 #include "ClntMsgRebind.h"
 #include "ClntOptIA_NA.h"
+#include "ClntOptIA_PD.h"
 #include "ClntOptClientIdentifier.h"
 #include "ClntOptServerIdentifier.h"
 #include "ClntOptServerUnicast.h"
@@ -38,14 +22,14 @@
 #include "Logger.h"
 #include "ClntOptOptionRequest.h"
 #include <cmath>
+#include <iostream>
 
 TClntMsgRebind::TClntMsgRebind(SmartPtr<TClntIfaceMgr> IfaceMgr, 
-			       SmartPtr<TClntTransMgr> TransMgr, 
-			       SmartPtr<TClntCfgMgr> CfgMgr, 
-			       SmartPtr<TClntAddrMgr> AddrMgr,
-			       TContainer<SmartPtr<TOpt> > ptrOpts, int iface)
-    :TClntMsg(IfaceMgr,TransMgr,CfgMgr,AddrMgr,iface, 
-	      SmartPtr<TIPv6Addr>() /*NULL*/, REBIND_MSG)
+                               SmartPtr<TClntTransMgr> TransMgr, 
+                               SmartPtr<TClntCfgMgr> CfgMgr, 
+                               SmartPtr<TClntAddrMgr> AddrMgr,
+                               List(TOpt) ptrOpts, int iface)
+  :TClntMsg(IfaceMgr,TransMgr,CfgMgr,AddrMgr, iface, 0, REBIND_MSG)
 {
     Options=ptrOpts;
     IRT=REB_TIMEOUT;
@@ -53,29 +37,40 @@ TClntMsgRebind::TClntMsgRebind(SmartPtr<TClntIfaceMgr> IfaceMgr,
     MRC=0;
     RT=0;
 
-    SmartPtr<TOpt> ptrOpt;
+    // there are options copied from RENEW. Get rid of some of them
+    SmartPtr<TOpt> opt;
     firstOption();
-    while(ptrOpt=getOption())
+    while(opt=getOption())
     {
-        if (ptrOpt->getOptType()==OPTION_ELAPSED_TIME)
+        if (opt->getOptType()==OPTION_ELAPSED_TIME)
             Options.del();
-        if (ptrOpt->getOptType()==OPTION_SERVERID)
+        if (opt->getOptType()==OPTION_SERVERID)
             Options.del();
     };
-    
+
+    // calculate timeout (how long should be the REBIND message transmitted)
     unsigned long maxMRD=0;    
     firstOption();
-    while(ptrOpt=getOption())
+    while(opt=getOption())
     {
-        if (ptrOpt->getOptType()==OPTION_IA)
-        {
-            SmartPtr<TClntOptIA_NA> ptrIA=(Ptr*) ptrOpt;
-            SmartPtr<TAddrIA> ptrAddrIA=
-                this->ClntAddrMgr->getIA(ptrIA->getIAID());
-
-            if (maxMRD<ptrAddrIA->getMaxValidTimeout())
-                maxMRD=ptrAddrIA->getMaxValidTimeout();
-        }
+      switch (opt->getOptType()) {
+        case OPTION_IA_NA:
+          {
+            SmartPtr<TClntOptIA_NA> ptrIA=(Ptr*) opt;
+            SmartPtr<TAddrIA> ptrAddrIA= ClntAddrMgr->getIA(ptrIA->getIAID());
+            if (ptrAddrIA && maxMRD<ptrAddrIA->getMaxValidTimeout())
+              maxMRD=ptrAddrIA->getMaxValidTimeout();
+            break;
+          }
+        case OPTION_IA_PD:
+          {
+            SPtr<TClntOptIA_PD> pd = (Ptr*) opt;
+            SPtr<TAddrIA> addrPd = ClntAddrMgr->getPD(pd->getIAID());
+            if (addrPd && maxMRD<addrPd->getMaxValidTimeout())
+              maxMRD=addrPd->getMaxValidTimeout();
+            break;
+          }
+      }
     }
     MRD= maxMRD;
     pkt = new char[getSize()];
@@ -84,6 +79,10 @@ TClntMsgRebind::TClntMsgRebind(SmartPtr<TClntIfaceMgr> IfaceMgr,
 
 void TClntMsgRebind::answer(SmartPtr<TClntMsg> Reply)
 {
+  TClntMsg::answer(Reply);
+  return;
+
+#if 0
         SmartPtr<TOpt> opt;
     
     // get DUID
@@ -99,11 +98,11 @@ void TClntMsgRebind::answer(SmartPtr<TClntMsg> Reply)
     {
         switch (opt->getOptType()) 
         {
-            case OPTION_IA:
+            case OPTION_IA_NA:
             {
                 SmartPtr<TClntOptIA_NA> ptrOptIA = (Ptr*)opt;
-                ptrOptIA->setThats(ClntIfaceMgr, ClntTransMgr, ClntCfgMgr, ClntAddrMgr,
-                    ptrDUID->getDUID(), SmartPtr<TIPv6Addr>() /*NULL*/,Reply->getIface());
+                ptrOptIA->setContext(ClntIfaceMgr, ClntTransMgr, ClntCfgMgr, ClntAddrMgr,
+                                     ptrDUID->getDUID(), SmartPtr<TIPv6Addr>() /*NULL*/,Reply->getIface());
                 //FIXME: replace this NULL with something meaningfull
                 //       (required for Unicast to work correctly)
                 ptrOptIA->doDuties();
@@ -115,7 +114,7 @@ void TClntMsgRebind::answer(SmartPtr<TClntMsg> Reply)
                     Options.first();
                     while (requestOpt = Options.get())
                     {
-                        if (requestOpt->getOptType()==OPTION_IA)
+                        if (requestOpt->getOptType()==OPTION_IA_NA)
                         {
                             SmartPtr<TClntOptIA_NA> ptrIA = (Ptr*) requestOpt;
                             if ((ptrIA->getIAID() == ptrOptIA->getIAID() ) &&
@@ -131,6 +130,10 @@ void TClntMsgRebind::answer(SmartPtr<TClntMsg> Reply)
                 }
                 break;
             }
+            case OPTION_IA_PD:
+            {
+              break;
+            }
             case OPTION_ORO:
             case OPTION_RELAY_MSG:
             case OPTION_INTERFACE_ID:
@@ -162,6 +165,7 @@ void TClntMsgRebind::answer(SmartPtr<TClntMsg> Reply)
         }
     }
     IsDone = true;
+#endif
 }
 
 void TClntMsgRebind::updateIA(SmartPtr <TClntOptIA_NA> ptrOptIA,
@@ -212,7 +216,7 @@ void TClntMsgRebind::updateIA(SmartPtr <TClntOptIA_NA> ptrOptIA,
 	    } else {
 		if ( (ptrOptAddr->getPref() == 0) || (ptrOptAddr->getValid() == 0) ) {
 		    releaseIA( ptrOptIA->getIAID() );
-		    break; // analyze next option OPTION_IA
+		    break; // analyze next option OPTION_IA_NA
 		}
 		// set up new options
 		if (ptrOptAddr->getPref() != ptrAddrAddr->getPref() )
@@ -231,7 +235,7 @@ void TClntMsgRebind::updateIA(SmartPtr <TClntOptIA_NA> ptrOptIA,
 	ptrAddrIA->setT1( ptrOptIA->getT1() );
 	ptrAddrIA->setT2( ptrOptIA->getT2() );
 	ptrAddrIA->setTimestamp();
-	ptrAddrIA->setState(CONFIGURED);
+	ptrAddrIA->setState(STATE_CONFIGURED);
     } else {
 	// unknown IAID, ignore it
 	Log(Warning) << "Received message contains unknown IA (IAID="
@@ -241,25 +245,38 @@ void TClntMsgRebind::updateIA(SmartPtr <TClntOptIA_NA> ptrOptIA,
 
 void TClntMsgRebind::doDuties()
 {
-    SmartPtr<TClntOptIA_NA> ptrIA=(Ptr*)getOption(OPTION_IA);
-    SmartPtr<TAddrIA> ptrAddrIA = ClntAddrMgr->getIA(ptrIA->getIAID());
     SmartPtr<TIfaceIface> iface = ClntIfaceMgr->getIfaceByID(this->Iface);
 
     if (!MRD)
     {
+	stringstream iaLst;
+	stringstream pdLst;
         SmartPtr<TOpt> ptrOpt;
         firstOption();
-	Log(Warning) << "REBIND for the IA:";
         while(ptrOpt=getOption())
         {
-            if (ptrOpt->getOptType()!=OPTION_IA)
-                continue;
-            SmartPtr<TClntOptIA_NA> ptrIA=(Ptr*)ptrOpt;
-	    Log(Cont) << ptrIA->getIAID() << " ";
-            releaseIA(ptrIA->getIAID());
+	    switch( ptrOpt->getOptType()) {
+	    case OPTION_IA_NA:
+	    {
+		SmartPtr<TClntOptIA_NA> ptrIA=(Ptr*)ptrOpt;
+		iaLst << ptrIA->getIAID() << " ";
+		releaseIA(ptrIA->getIAID());
+		break;
+	    }
+	    case OPTION_IA_PD:
+	    {
+		SPtr<TClntOptIA_PD> ptrPD = (Ptr*)ptrOpt;
+		ptrPD->setContext(ClntIfaceMgr, ClntTransMgr, ClntCfgMgr, ClntAddrMgr, 0, 0, this);
+		ptrPD->delPrefixes();
+		break;
+	    }
+	    };
         }
-	Log(Cont) << " failed on the " << iface->getName() << "/" 
-		  << iface->getID() << " interface." << LogEnd;
+	Log(Warning) << "REBIND for the IA(s):" << iaLst.str()
+		     << ", PD(s):" << pdLst.str()
+		     << " failed on the " << iface->getFullName() 
+		     << " interface." << LogEnd;
+	Log(Warning) << "Restarting server discovery process." << LogEnd;
         IsDone=true;
     }
     else
@@ -280,11 +297,21 @@ void TClntMsgRebind::releaseIA(int IAID)
     while(ptrAddr=ptrAddrIA->getAddr())
     {
         //remove outdated address from interface
-        ClntIfaceMgr->getIfaceByID(ptrAddrIA->getIface())->delAddr(ptrAddr->get());
+	SPtr<TIfaceIface> ptrIface = ClntIfaceMgr->getIfaceByID(ptrAddrIA->getIface());
+	if (!ptrIface) {
+	    Log(Error) << "Unable to find interface with ifindex " << ptrAddrIA->getIface() << LogEnd;
+	    continue;
+	}
+	ptrIface->delAddr(ptrAddr->get(), ptrIface->getPrefixLength());
         //and from db
         ptrAddrIA->delAddr(ptrAddr->get());
     }
-    ptrAddrIA->setState(NOTCONFIGURED);
+    ptrAddrIA->setState(STATE_NOTCONFIGURED);
+
+    SPtr<TClntCfgIA> cfgIA = ClntCfgMgr->getIA(IAID);
+    if (!cfgIA)
+	return;
+    cfgIA->setState(STATE_NOTCONFIGURED);
 }
 
 bool TClntMsgRebind::check() {
