@@ -3,77 +3,11 @@
  *
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>
  *          Marek Senderski <msend@o2.pl>
- *
+ * changes: Krzysztof Wnuk <keczi@poczta.onet.pl>
  * released under GNU GPL v2 or later licence
  *
- * $Id: ClntTransMgr.cpp,v 1.35 2005/09/15 19:46:08 thomson Exp $
+ * $Id: ClntTransMgr.cpp,v 1.56 2007-04-22 23:03:18 thomson Exp $
  *
- * $Log: ClntTransMgr.cpp,v $
- * Revision 1.35  2005/09/15 19:46:08  thomson
- * *** empty log message ***
- *
- * Revision 1.34  2005/01/12 00:10:05  thomson
- * Compilation fixes.
- *
- * Revision 1.33  2005/01/08 16:52:03  thomson
- * Relay support implemented.
- *
- * Revision 1.32  2004/12/08 00:18:35  thomson
- * Log message clarified.
- *
- * Revision 1.31  2004/12/07 22:57:51  thomson
- * removeExpired() method rewritten.
- *
- * Revision 1.30  2004/12/07 00:45:41  thomson
- * Clnt managers creation unified and cleaned up.
- *
- * Revision 1.29  2004/12/04 23:45:40  thomson
- * Problem with client and server on the same Linux host fixed (bug #56)
- *
- * Revision 1.28  2004/12/03 20:51:42  thomson
- * Logging issues fixed.
- *
- * Revision 1.27  2004/12/02 00:51:04  thomson
- * Log files are now always created (bugs #34, #36)
- *
- * Revision 1.26  2004/11/30 00:53:56  thomson
- * RapidCommit is now property of the interface, not the group.
- *
- * Revision 1.25  2004/11/05 09:01:29  thomson
- * Win32 socket initialization error corrected.
- * Thanks to Michal Balcerkiewicz.
- *
- * Revision 1.24  2004/11/01 23:31:25  thomson
- * New options,option handling mechanism and option renewal implemented.
- *
- * Revision 1.23  2004/10/27 22:07:56  thomson
- * Signed/unsigned issues fixed, Lifetime option implemented, INFORMATION-REQUEST
- * message is now sent properly. Valid lifetime granted by server fixed.
- *
- * Revision 1.22  2004/10/25 20:45:53  thomson
- * Option support, parsers rewritten. ClntIfaceMgr now handles options.
- *
- * Revision 1.21  2004/09/07 22:02:33  thomson
- * pref/valid/IAID is not unsigned, RAPID-COMMIT now works ok.
- *
- * Revision 1.20  2004/09/07 17:42:31  thomson
- * Server Unicast implemented.
- *
- * Revision 1.19  2004/09/07 15:37:44  thomson
- * Socket handling changes.
- *
- * Revision 1.16  2004/07/11 14:08:01  thomson
- * Opening additonal socket on loopback is not necessary in WIN32 systems.
- *
- * Revision 1.8  2004/04/10 12:18:01  thomson
- * Numerous fixes: LogName, LogMode options added, dns-servers changed to
- * dns-server, '' around strings are no longer needed.
- *
- * Revision 1.7  2004/04/09 22:33:11  thomson
- * dns-servers changed to option dns-server
- *
- * Revision 1.6  2004/03/29 18:53:08  thomson
- * Author/Licence/cvs log/cvs version headers added.
  */
 
 #ifdef WIN32
@@ -89,7 +23,9 @@
 #include "ClntTransMgr.h"
 #include "ClntAddrMgr.h"
 #include "ClntCfgMgr.h"
+#include "ClntCfgPD.h"
 #include "Msg.h"
+#include "ClntMsgAdvertise.h"
 #include "ClntMsgRequest.h"
 #include "ClntMsgRenew.h"
 #include "ClntMsgRebind.h"
@@ -105,14 +41,18 @@
 
 using namespace std;
 
-
 TClntTransMgr::TClntTransMgr(SmartPtr<TClntIfaceMgr> ifaceMgr, 
 			     SmartPtr<TClntAddrMgr> addrMgr,
 			     SmartPtr<TClntCfgMgr> cfgMgr,
                              string config)
 {
     // should we set REUSE option during binding sockets?
-    this->BindReuse = CLIENT_BIND_REUSE;
+#ifdef MOD_CLNT_BIND_REUSE
+    this->BindReuse = true;
+#else
+    this->BindReuse = false;
+#endif
+
     this->IsDone = true;
 
     this->IfaceMgr = ifaceMgr;
@@ -120,9 +60,9 @@ TClntTransMgr::TClntTransMgr(SmartPtr<TClntIfaceMgr> ifaceMgr,
     this->CfgMgr   = cfgMgr;
 
     if (this->BindReuse)
-	Log(Info) << "Bind reuse enabled." << LogEnd;
+	Log(Debug) << "Bind reuse enabled (multiple instances allowed)." << LogEnd;
     else
-	Log(Info) << "Bind reuse disabled." << LogEnd;
+	Log(Debug) << "Bind reuse disabled (multiple instances not allowed)." << LogEnd;
 
     if (!this->openLoopbackSocket()) {
 	return;
@@ -150,34 +90,15 @@ bool TClntTransMgr::openSocket(SmartPtr<TClntCfgIface> iface) {
     if (iface->noConfig())       
 	return true;
 
-    SmartPtr<TIfaceIface> loopback;
-    SmartPtr<TIfaceIface> ptrIface;
-    IfaceMgr->firstIface();
-    while (ptrIface=IfaceMgr->getIface()) {
-        if (!ptrIface->flagLoopback()) {
-            continue;
-	}
-	loopback = ptrIface;
-	break;
-    }
-    if (!loopback) {
-	Log(Error) << "Loopback interface not found!" << LogEnd;
-	return false;
-    }
-
     // create IAs in AddrMgr corresponding to those specified in CfgMgr.
-    SmartPtr<TClntCfgGroup> group;
-    iface->firstGroup();
-    while(group=iface->getGroup()) {
-	SmartPtr<TClntCfgIA> ia;
-	group->firstIA();
-	while(ia=group->getIA()) {
-	    AddrMgr->addIA(new TAddrIA(iface->getID(),SmartPtr<TIPv6Addr>(), 
-				       SmartPtr<TDUID>(),CLIENT_DEFAULT_T1,CLIENT_DEFAULT_T2,
-				       ia->getIAID()));
-	}
+    SmartPtr<TClntCfgIA> ia;
+    iface->firstIA();
+    while(ia=iface->getIA()) {
+	AddrMgr->addIA(new TAddrIA(iface->getID(),SmartPtr<TIPv6Addr>(), 
+				   SmartPtr<TDUID>(),CLIENT_DEFAULT_T1,CLIENT_DEFAULT_T2,
+				   ia->getIAID()));
     }
-
+    
     // open socket
     SmartPtr<TIfaceIface> realIface = IfaceMgr->getIfaceByID(iface->getID());
     if (!realIface) {
@@ -210,6 +131,21 @@ bool TClntTransMgr::openSocket(SmartPtr<TClntCfgIface> iface) {
 
 #if 0
 #ifndef WIN32
+    SmartPtr<TIfaceIface> loopback;
+    SmartPtr<TIfaceIface> ptrIface;
+    IfaceMgr->firstIface();
+    while (ptrIface=IfaceMgr->getIface()) {
+        if (!ptrIface->flagLoopback()) {
+            continue;
+	}
+	loopback = ptrIface;
+	break;
+    }
+    if (!loopback) {
+	Log(Error) << "Loopback interface not found!" << LogEnd;
+	return false;
+    }
+
     // required to be able to receive data from server on the same machine
     // (data is sent via the lo interface)
     Log(Notice) << "Creating socket (addr=" << *addr << ") on the " << loopback->getName() 
@@ -267,7 +203,7 @@ void TClntTransMgr::removeExpired() {
 			 << " interface (in IA " << ptrIA->getIAID() <<") has expired." << LogEnd;
 
 	    // remove that address from the physical interace
-	    ptrIface->delAddr(ptrAddr->get());
+	    ptrIface->delAddr(ptrAddr->get(), ptrIface->getPrefixLength());
 	}
     }
 }
@@ -286,7 +222,7 @@ void TClntTransMgr::checkDB()
             ptrIA->firstAddr();
             while (ptrAddr = ptrIA->getAddr())
                 ptrIA->delAddr( ptrAddr->get() );
-            ptrIA->setState(NOTCONFIGURED);
+            ptrIA->setState(STATE_NOTCONFIGURED);
         }
     }
 }
@@ -355,6 +291,14 @@ void TClntTransMgr::doDuties()
             Transactions.del();
     }
 
+    if (CfgMgr->inactiveMode())
+    {
+	SPtr<TClntCfgIface> x;
+	x = CfgMgr->checkInactiveIfaces();
+	if (x)
+	    openSocket(x);
+    }
+
     this->removeExpired();
     this->AddrMgr->doDuties();
 
@@ -366,14 +310,14 @@ void TClntTransMgr::doDuties()
         // are there any tentative addrs?
         checkDecline();
 
-        // are there any IAs to configure?
-        checkSolicit();    
+        // are there any IAs or TAs to configure?
+        checkSolicit();
 
         //is there any IA in Address manager, which has not sufficient number 
         //of addresses
         checkRequest();
 
-        // are there any aging IAs?
+        // are there any aging IAs or PDs?
         checkRenew();
 
         // did we switched links lately?
@@ -384,6 +328,9 @@ void TClntTransMgr::doDuties()
         //Maybe we require only infromations concernig link
         checkInfRequest();
     } 
+    
+    // This method launch the DNS update, so the checkDecline has to be done before to ensure the ip address is valid
+    this->IfaceMgr->doDuties();
 
     if (this->Shutdown && !Transactions.count())
         this->IsDone = true;
@@ -391,22 +338,22 @@ void TClntTransMgr::doDuties()
 
 void TClntTransMgr::shutdown()
 {
-    this->Shutdown = true;
-
-    Log(Notice) << "Shutting down entire client." << LogEnd;
-
-    // delete all transactions
-    Transactions.clear();
-
-    List(TAddrIA) releasedIAs;
-
     SmartPtr<TAddrIA> ptrFirstIA;
     SmartPtr<TAddrIA> ptrNextIA;
+    SmartPtr<TAddrIA> ta;
+    SPtr<TAddrIA> pd;
+    SmartPtr<TClntCfgIface> iface;
+    List(TAddrIA) releasedIAs;
+    List(TAddrIA) releasedPDs;
 
+    Transactions.clear(); // delete all transactions
+    this->Shutdown = true;
+    Log(Notice) << "Shutting down entire client." << LogEnd;
+	
     // delete all weird-state and address-free IAs 
     AddrMgr->firstIA();
     while (ptrFirstIA = AddrMgr->getIA()) {
-        if ( (ptrFirstIA->getState() != CONFIGURED && ptrFirstIA->getState() != INPROCESS) ||
+        if ( (ptrFirstIA->getState() != STATE_CONFIGURED && ptrFirstIA->getState() != STATE_INPROCESS) ||
             !ptrFirstIA->countAddr() )
             AddrMgr->delIA(ptrFirstIA->getIAID()); 
     }
@@ -449,10 +396,68 @@ void TClntTransMgr::shutdown()
                     AddrMgr->delIA( ptrNextIA->getIAID() );
                 }
         }
-        if (releasedIAs.count()) 
-            this->sendRelease(releasedIAs);
+
+	ta = 0;
+        if (releasedIAs.count()) { 
+	    // check if there are TA to release
+	    releasedIAs.first();
+	    iface = CfgMgr->getIface(releasedIAs.get()->getIface());
+	    if (iface && iface->countTA()) {
+		iface->firstTA();
+		SmartPtr<TClntCfgTA> cfgTA = iface->getTA();
+		ta = AddrMgr->getTA(cfgTA->getIAID());
+		cfgTA->setState(STATE_DISABLED);
+	    }
+
+	}
+	pd = 0;
+	AddrMgr->firstPD();
+	while (pd = AddrMgr->getPD()) {
+	    releasedPDs.append(pd);
+	    SPtr<TClntCfgPD> cfgPD = CfgMgr->getPD(pd->getIAID());
+	    if (cfgPD)
+		cfgPD->setState(STATE_DISABLED);
+	}
+	this->sendRelease(releasedIAs,ta, releasedPDs);
     }
 
+    // now check if there are any TA left
+    CfgMgr->firstIface();
+    while (iface = CfgMgr->getIface()) {
+	if (iface->countTA()) {
+	    iface->firstTA();
+	    SmartPtr<TClntCfgTA> cfgTA = iface->getTA();
+	    ta = AddrMgr->getTA(cfgTA->getIAID());
+	    releasedIAs.clear();
+	    if (!ta) {
+		Log(Warning) << "Unable to find TA(taid=" << cfgTA->getIAID() <<"). " << LogEnd;
+		continue;
+	    }
+	    if (cfgTA->getState()==STATE_CONFIGURED) {
+		this->sendRelease(releasedIAs, ta, releasedPDs);
+		cfgTA->setState(STATE_DISABLED);
+	    }
+	}
+
+    }
+
+    // are there any PDs left to release?
+    pd = 0;
+    AddrMgr->firstPD();
+    releasedIAs.clear();
+    releasedPDs.clear();
+    while (pd = AddrMgr->getPD()) {
+	releasedPDs.append(pd);
+	SPtr<TClntCfgPD> cfgPD = CfgMgr->getPD(pd->getIAID());
+	if (cfgPD)
+	    cfgPD->setState(STATE_DISABLED);
+    }
+    if (releasedPDs.count())
+	this->sendRelease(releasedIAs, 0, releasedPDs);
+    
+    //doDuties(); // just to send RELEASE msg
+    //Transactions.clear(); // delete all transactions
+
     // clean up options
     this->IfaceMgr->removeAllOpts();
 }
@@ -478,8 +483,9 @@ void TClntTransMgr::relayMsg(SmartPtr<TClntMsg>  msgAnswer)
 		ifaceAnswer   = this->IfaceMgr->getIfaceByID(msgAnswer->getIface());
 		Log(Warning) << "Reply for transaction 0x" << hex << msgQuestion->getTransID() << dec
 			     << " sent on " << ifaceQuestion->getFullName() << " was received on interface " 
-			     << ifaceAnswer->getFullName() << ". Ignored." << LogEnd;
-		return;
+			     << ifaceAnswer->getFullName() << "." << LogEnd;
+		// return; // don't return, just fix interface ID
+		msgAnswer->setIface(msgQuestion->getIface());
 	    }
 
             msgQuestion->answer(msgAnswer);
@@ -490,6 +496,7 @@ void TClntTransMgr::relayMsg(SmartPtr<TClntMsg>  msgAnswer)
     if (!found) 
         Log(Warning) << "Message with wrong transID (" << hex << msgAnswer->getTransID() << dec
 		     << ") received. Ignoring." << LogEnd;
+    CfgMgr->dump();
     AddrMgr->dump();
 }
 
@@ -524,6 +531,11 @@ unsigned long TClntTransMgr::getTimeout()
     if (timeout > tmp)
 	timeout = tmp;
 
+    if (CfgMgr->inactiveIfacesCnt()) {
+	if (timeout>INACTIVE_MODE_INTERVAL)
+	    timeout=INACTIVE_MODE_INTERVAL;
+    }
+
     return timeout;
 }
 
@@ -534,34 +546,63 @@ void TClntTransMgr::stop()
 /**
  * Note: requestOptions list MUST NOT contain server DUID.
  */
-void TClntTransMgr::sendRequest(List(TOpt) requestOptions, 
-                                List(TMsg) srvlist,int iface)
+void TClntTransMgr::sendRequest(List(TOpt) requestOptions, int iface)
 {
+    sortAdvertiseLst();
+
     SmartPtr<TOpt> opt;
     requestOptions.first();
     while (opt = requestOptions.get()) {
 	if (!allowOptInMsg(REQUEST_MSG, opt->getOptType()))
 	    requestOptions.del();
     }
-    SmartPtr<TClntMsg> ptr = new TClntMsgRequest(IfaceMgr,That,CfgMgr, AddrMgr, requestOptions,srvlist,iface);
-    Transactions.append( ptr );
+    SmartPtr<TClntMsg> ptr = new TClntMsgRequest(IfaceMgr,That,CfgMgr, AddrMgr, requestOptions, iface);
+    Transactions.append( (Ptr*)ptr );
 }
 
 // Send RELEASE message
-void TClntTransMgr::sendRelease( List(TAddrIA) IALst)
+void TClntTransMgr::sendRelease( List(TAddrIA) IALst, SmartPtr<TAddrIA> ta, List(TAddrIA) pdLst)
 {
-    if (!IALst.count()) {
-        Log(Error) << "Unable to send RELEASE with empty IAs list." << LogEnd;
+    if ( !IALst.count() && !ta && !pdLst.count() ) {
+        Log(Error) << "Unable to send RELEASE with empty IA list, PD list and no TA." << LogEnd;
         return;
     }
 
+    // find interface and srv address
+    int iface;
+    SmartPtr<TIPv6Addr> addr;
     SmartPtr<TAddrIA> ptrIA;
-    IALst.first();
-    ptrIA = IALst.get();
-    Log(Notice) << "Creating RELEASE for " << IALst.count() << " IA(s)." << LogEnd;
+    if (IALst.count()) {
+	IALst.first();
+	ptrIA = IALst.get();
+	iface = ptrIA->getIface();
+	addr  = ptrIA->getSrvAddr();
+    } else if (pdLst.count()) {
+	pdLst.first();
+	ptrIA = pdLst.get();
+	iface = ptrIA->getIface();
+	addr  = ptrIA->getSrvAddr();
+    } else if (ta) {
+	iface = ta->getIface();
+	addr  = ta->getSrvAddr();
+    } else {
+	Log(Error) << "Unable to send RELEASE message. No IA, PD or TA defined." << LogEnd;
+	return;
+    }
 
-    SmartPtr<TClntMsg> ptr = new TClntMsgRelease(IfaceMgr,That,CfgMgr, AddrMgr, ptrIA->getIface(), 
-        ptrIA->getSrvAddr(), IALst);
+    SmartPtr<TClntCfgIface> ptrIface = CfgMgr->getIface(iface);
+    if (!iface) {
+	Log(Error) << "Unable to find interface with ifindex=" << iface << LogEnd;
+	return;
+    }
+	
+    Log(Notice) << "Creating RELEASE for " << IALst.count() << " IA(s), " 
+		<< pdLst.count() << " PD(s), "
+		<< (ta?" and TA":" (no TA)") << " on the " << ptrIface->getFullName() 
+		<< " interface." << LogEnd;
+
+    SmartPtr<TClntMsg> ptr = new TClntMsgRelease(IfaceMgr,That,CfgMgr, AddrMgr, iface, 
+						 addr, IALst, ta, pdLst);
     Transactions.append( ptr );
 }
 
@@ -594,6 +635,7 @@ void TClntTransMgr::sendInfRequest(List(TOpt) requestOptions, int iface) {
 
 // should we send SOLICIT ?
 void TClntTransMgr::checkSolicit() {
+
     //For every iface, every group in iface in ClntCfgMgr 
     //Enumerate IA's from this group
     SmartPtr<TClntCfgIface> iface;
@@ -602,37 +644,56 @@ void TClntTransMgr::checkSolicit() {
     {
         if (iface->noConfig())
             continue;
-        SmartPtr<TClntCfgGroup> group;
-        iface->firstGroup();
-        while(group=iface->getGroup())
-        {
-            List(TClntCfgIA) IALstToConfig;
-            SmartPtr<TClntCfgIA> ia;
-            group->firstIA();
-            while(ia=group->getIA())
-            {
-                //These not assigned in AddrMgr and configurable and not in trasaction
-                //group and pass to constructor of Solicit message
-                SmartPtr<TAddrIA> iaAddrMgr=AddrMgr->getIA(ia->getIAID());
-                if(iaAddrMgr->getState()==NOTCONFIGURED)
-                {
-                    IALstToConfig.append(ia);
-                    iaAddrMgr->setState(INPROCESS);
-                }
-            };
-            if (IALstToConfig.count()) {//Are there any IA, which should be configured?
-		
-		Log(Info) << "Creating SOLICIT message ";
-		if (iface->getRapidCommit()) {
-		    Log(Cont) << "(with rapid-commit)";
-		} 
-		Log(Cont) << " on " << iface->getName() <<" interface." << LogEnd;
-                Transactions.append(
-		    new TClntMsgSolicit(IfaceMgr,That,CfgMgr,AddrMgr,
-					iface->getID(), SmartPtr<TIPv6Addr>()/*NULL*/, 
-					IALstToConfig, iface->getRapidCommit()));
+
+	// step 1: check if there are any IA to be configured
+	List(TClntCfgIA) iaLst; // list of IA requiring configuration
+	SmartPtr<TClntCfgIA> cfgIA;
+    
+	iface->firstIA();
+	while(cfgIA=iface->getIA())
+	{
+	    // These not assigned in AddrMgr and configurable and not in 
+	    // trasaction group and pass to constructor of Solicit message
+	    if(cfgIA->getState()==STATE_NOTCONFIGURED)
+		iaLst.append(cfgIA);
+	}
+
+	// step 2: check if TA has to be configured
+	SmartPtr<TClntCfgTA> ta;
+	if (iface->countTA() ) {
+	    iface->firstTA();
+	    ta = iface->getTA();
+	    if (ta->getState()==STATE_NOTCONFIGURED)
+		ta->setState(STATE_INPROCESS);
+	    else
+		ta = 0;
+	}
+
+	// step 3: check if there are any PD to be configured
+	List(TClntCfgPD) pdLst;
+	SPtr<TClntCfgPD> pd;
+
+	iface->firstPD();
+	while ( pd = iface->getPD() ) {
+	    if (pd->getState()==STATE_NOTCONFIGURED) {
+		pdLst.append(pd);
 	    }
-        }//for every group
+	}
+
+	//Are there any IA, TA or PD which should be configured?
+	if (iaLst.count() || ta || pdLst.count()) {
+	    Log(Info) << "Creating SOLICIT message with " << iaLst.count()
+		      << " IA(s), " << (ta?"1":"no") << " TA and " << pdLst.count()
+		      << " PD(s)";
+	    if (iface->getRapidCommit()) {
+		Log(Cont) << " (with rapid-commit)";
+	    } 
+	    Log(Cont) << " on " << iface->getFullName() <<" interface." << LogEnd;
+	    Transactions.append(new TClntMsgSolicit(IfaceMgr,That,CfgMgr,AddrMgr,
+						    iface->getID(), 0, iaLst, ta, pdLst, 
+						    iface->getRapidCommit()));
+        }
+
     }//for every iface
 }
 
@@ -654,7 +715,7 @@ void TClntTransMgr::checkConfirm()
             if(ptrIA->getIface()==ptrIface->getID())
             {
                 IALst.append(ptrIA);
-                ptrIA->setState(INPROCESS);
+                ptrIA->setState(STATE_INPROCESS);
             }
         }
         if (IALst.count())
@@ -677,41 +738,41 @@ void TClntTransMgr::checkInfRequest()
 	    continue;
 	}
 	if (!ifaceIface->getTimeout()) {
-	    if (iface->getDNSServerState()  == CONFIGURED) 
-		iface->setDNSServerState(NOTCONFIGURED);
-	    if (iface->getDomainState()     == CONFIGURED) 
-		iface->setDomainState(NOTCONFIGURED);
-	    if (iface->getNTPServerState()  == CONFIGURED) 
-		iface->setNTPServerState(NOTCONFIGURED);
-            if (iface->getTimezoneState()   == CONFIGURED) 
-		iface->setTimezoneState (NOTCONFIGURED);
-	    if (iface->getSIPServerState()  == CONFIGURED) 
-		iface->setSIPServerState(NOTCONFIGURED);
-	    if (iface->getSIPDomainState()  == CONFIGURED) 
-		iface->setSIPDomainState(NOTCONFIGURED);
-	    if (iface->getFQDNState()       == CONFIGURED) 
-		iface->setFQDNState(NOTCONFIGURED);
-	    if (iface->getNISServerState()  == CONFIGURED) 
-		iface->setNISServerState(NOTCONFIGURED);
-	    if (iface->getNISDomainState()  == CONFIGURED) 
-		iface->setNISDomainState(NOTCONFIGURED);
-	    if (iface->getNISPServerState() == CONFIGURED) 
-		iface->setNISPServerState(NOTCONFIGURED);
-	    if (iface->getNISPDomainState() == CONFIGURED) 
-		iface->setNISPDomainState(NOTCONFIGURED);
+	    if (iface->getDNSServerState()  == STATE_CONFIGURED) 
+		iface->setDNSServerState(STATE_NOTCONFIGURED);
+	    if (iface->getDomainState()     == STATE_CONFIGURED) 
+		iface->setDomainState(STATE_NOTCONFIGURED);
+	    if (iface->getNTPServerState()  == STATE_CONFIGURED) 
+		iface->setNTPServerState(STATE_NOTCONFIGURED);
+            if (iface->getTimezoneState()   == STATE_CONFIGURED) 
+		iface->setTimezoneState (STATE_NOTCONFIGURED);
+	    if (iface->getSIPServerState()  == STATE_CONFIGURED) 
+		iface->setSIPServerState(STATE_NOTCONFIGURED);
+	    if (iface->getSIPDomainState()  == STATE_CONFIGURED) 
+		iface->setSIPDomainState(STATE_NOTCONFIGURED);
+	    if (iface->getFQDNState()       == STATE_CONFIGURED) 
+		iface->setFQDNState(STATE_NOTCONFIGURED);
+	    if (iface->getNISServerState()  == STATE_CONFIGURED) 
+		iface->setNISServerState(STATE_NOTCONFIGURED);
+	    if (iface->getNISDomainState()  == STATE_CONFIGURED) 
+		iface->setNISDomainState(STATE_NOTCONFIGURED);
+	    if (iface->getNISPServerState() == STATE_CONFIGURED) 
+		iface->setNISPServerState(STATE_NOTCONFIGURED);
+	    if (iface->getNISPDomainState() == STATE_CONFIGURED) 
+		iface->setNISPDomainState(STATE_NOTCONFIGURED);
 	}
 
-	if ( (iface->getDNSServerState()  == NOTCONFIGURED) ||
-	     (iface->getDomainState()     == NOTCONFIGURED) ||
-	     (iface->getNTPServerState()  == NOTCONFIGURED) ||
-	     (iface->getTimezoneState()   == NOTCONFIGURED) ||
-	     (iface->getSIPServerState()  == NOTCONFIGURED) ||
-	     (iface->getSIPDomainState()  == NOTCONFIGURED) ||
-	     (iface->getFQDNState()       == NOTCONFIGURED) ||
-	     (iface->getNISServerState()  == NOTCONFIGURED) ||
-	     (iface->getNISDomainState()  == NOTCONFIGURED) ||
-	     (iface->getNISPServerState() == NOTCONFIGURED) ||
-	     (iface->getNISPDomainState() == NOTCONFIGURED) ) {
+	if ( (iface->getDNSServerState()  == STATE_NOTCONFIGURED) ||
+	     (iface->getDomainState()     == STATE_NOTCONFIGURED) ||
+	     (iface->getNTPServerState()  == STATE_NOTCONFIGURED) ||
+	     (iface->getTimezoneState()   == STATE_NOTCONFIGURED) ||
+	     (iface->getSIPServerState()  == STATE_NOTCONFIGURED) ||
+	     (iface->getSIPDomainState()  == STATE_NOTCONFIGURED) ||
+	     (iface->getFQDNState()       == STATE_NOTCONFIGURED) ||
+	     (iface->getNISServerState()  == STATE_NOTCONFIGURED) ||
+	     (iface->getNISDomainState()  == STATE_NOTCONFIGURED) ||
+	     (iface->getNISPServerState() == STATE_NOTCONFIGURED) ||
+	     (iface->getNISPDomainState() == STATE_NOTCONFIGURED) ) {
 	    Log(Info) << "Creating INFORMATION-REQUEST message on "
 		      << iface->getName() << "/" << iface->getID() << " interface." << LogEnd;
             Transactions.append(new TClntMsgInfRequest(IfaceMgr,That,CfgMgr,AddrMgr,iface));
@@ -725,48 +786,67 @@ void TClntTransMgr::checkRenew()
     if (AddrMgr->getT1Timeout() > 0 ) 
 	return;
 
-    // yes, there are. Find them!
+    // TENTATIVE_YES, there are. Find them!
+    
+    // Find all IAs
+    List(TAddrIA) iaLst;
+    SmartPtr<TAddrIA> ia;
+    SmartPtr<TAddrIA> iaPattern;
     AddrMgr->firstIA();
-    SmartPtr < TAddrIA> ptrIA;
-    while (ptrIA = AddrMgr->getIA() ) 
+    while (ia = AddrMgr->getIA() ) 
     {
-        if ( (ptrIA->getT1Timeout() == 0) && (ptrIA->getState()==CONFIGURED) ) 
-        {
-            // to avoid race conditions (RENEW vs DECLINE)
-            if (ptrIA->getTentative()==DONTKNOWYET)
-                continue;
+        if ( (ia->getT1Timeout()!=0) || 
+	     (ia->getState()!=STATE_CONFIGURED) ||
+	     (ia->getTentative()==TENTATIVE_UNKNOWN) )
+	    continue;
+	
+	if (!iaPattern) {
+	    iaPattern = ia;
+	    iaLst.append(ia);
+	    ia->setState(STATE_INPROCESS);
+	    continue;
+	}
 
-            TContainer<SmartPtr<TAddrIA> > iaLst;
-            Log(Notice) << "IA (IAID=" << ptrIA->getIAID() 
-                << ") needs RENEW. Trying to group with other IA(s) requiring renewal:";
-            SmartPtr<TAddrIA> iaPattern=ptrIA;
-            iaLst.append(ptrIA);
-            ptrIA->setState(INPROCESS);
-            while(ptrIA = AddrMgr->getIA())
-            {
-                //the same diffrence between T1(which has just elapsed
-                //for both IAs) and T2
-                if (((ptrIA->getT2()-ptrIA->getT1())==
-                    (iaPattern->getT2()-iaPattern->getT1()))&&
-                    (ptrIA->getIface()==iaPattern->getIface())&&
-                    (ptrIA->getDUID()==iaPattern->getDUID()))
-                {
-		    Log(Cont) << ptrIA->getIAID() << " ";
-                    iaLst.append(ptrIA);
-                    ptrIA->setState(INPROCESS);
-                }
-            }
-	    if (iaLst.count()==1) {
-		Log(Cont) << "none found." << LogEnd;
-	    } else {
-		Log(Cont) << "." << LogEnd;
-	    }
-	    
-            SmartPtr <TClntMsg> ptrRenew = new TClntMsgRenew(IfaceMgr, That, CfgMgr, AddrMgr, iaLst);
-            Transactions.append(ptrRenew);
-            AddrMgr->firstIA();
-        }
+	if ( (ia->getIface() == iaPattern->getIface()) &&
+	     (ia->getDUID()  == iaPattern->getDUID()) )
+	{
+		iaLst.append(ia);
+		ia->setState(STATE_INPROCESS);
+	}
     }
+
+    // Find all PDs
+    List(TAddrIA) pdLst;
+    AddrMgr->firstPD();
+    while (ia = AddrMgr->getPD()) {
+        if ( (ia->getT1Timeout()!=0) || 
+	     (ia->getState()!=STATE_CONFIGURED) )
+	    continue;
+	
+	if (!iaPattern) {
+	    iaPattern = ia;
+	    pdLst.append(ia);
+	    ia->setState(STATE_INPROCESS);
+	    continue;
+	}
+
+	if ( (ia->getIface() == iaPattern->getIface()) &&
+	     (ia->getDUID()  == iaPattern->getDUID()) )
+	{
+		pdLst.append(ia);
+		ia->setState(STATE_INPROCESS);
+	}
+    }
+
+    if (iaLst.count() + pdLst.count() == 0) {
+	// FIXME: something is wrong
+	Log(Error) << "RENEW send attempted, but there are no IA(s) or PD(s) to send." << LogEnd;
+	return;
+    }
+	 
+    Log(Info) << "Generating RENEW for " << iaLst.count() << " IA(s) and " << pdLst.count() << " PD(s). " << LogEnd;
+    SmartPtr <TClntMsg> ptrRenew = new TClntMsgRenew(IfaceMgr, That, CfgMgr, AddrMgr, iaLst, pdLst);
+    Transactions.append(ptrRenew);
 }
 
 void TClntTransMgr::checkDecline()
@@ -785,7 +865,7 @@ void TClntTransMgr::checkDecline()
         AddrMgr->firstIA();
         while((ptrIA=AddrMgr->getIA())&&(!firstIA))
         {
-            if (ptrIA->getTentative()==YES)
+            if (ptrIA->getTentative()==TENTATIVE_YES)
                 firstIA=ptrIA;
         }
         if (firstIA)
@@ -793,7 +873,7 @@ void TClntTransMgr::checkDecline()
             declineIALst.append(firstIA);
             while(ptrIA=AddrMgr->getIA())
             {
-                if ((ptrIA->getTentative()==YES)&&
+                if ((ptrIA->getTentative()==TENTATIVE_YES)&&
                     (*ptrIA->getDUID()==*firstIA->getDUID()))
                 {
                     declineIALst.append(ptrIA);
@@ -809,90 +889,97 @@ void TClntTransMgr::checkDecline()
 
             // decline sent, now remove those addrs from IfaceMgr
             SmartPtr<TIfaceIface> ptrIface = IfaceMgr->getIfaceByID(firstIA->getIface());
+	    SPtr<TDUID> duid;
 
             declineIALst.first();
             while (ptrIA = declineIALst.get() ) {
                 SmartPtr<TAddrAddr> ptrAddr;
                 ptrIA->firstAddr();
-		Log(Info) << "Sending DECLINE for IA(IAID=" << ptrIA->getIAID() << "): ";
+		Log(Info) << "Sending DECLINE for IA(IAID=" << ptrIA->getIAID() << ")" << LogEnd;
 
                 while ( ptrAddr= ptrIA->getAddr() ) {
-                    if (ptrAddr->getTentative() == YES) {
+                    if (ptrAddr->getTentative() == TENTATIVE_YES) {
+			Log(Cont) << ptrAddr->get()->getPlain() << " ";
                         // remove this address from interface
-                        Log(Cont) << "(" << ptrAddr->get()->getPlain();
-                        result = ptrIface->delAddr(ptrAddr->get());
-                        Log(Cont) << " Iface removal=" << result;
+                        result = ptrIface->delAddr(ptrAddr->get(), ptrAddr->getPrefix());
 
                         // remove this address from addrDB
                         result = ptrIA->delAddr(ptrAddr->get());
-                        Log(Cont) << " AddrDB removal=" << result << ")";
                     }
                 }
 		Log(Cont) << LogEnd;
-		
+		duid = ptrIA->getDUID();
                 ptrIA->setTentative();
             }
+
+	    // create REQUEST message
+	    SmartPtr<TClntMsgRequest> request;
+	    request = new TClntMsgRequest(IfaceMgr, That, CfgMgr, AddrMgr,
+					  declineIALst, duid, firstIA->getIface() );
+	    Transactions.append( (Ptr*) request);
+
         }
     } while(firstIA);
 }
 
 void TClntTransMgr::checkRequest()
 {
-    SmartPtr<TAddrIA> firstIA;
-    SmartPtr<TAddrIA> ptrIA;
-    SmartPtr<TClntCfgIA> ptrCfgIA;
-    SmartPtr<TClntCfgGroup> firstIAGroup;
-//    int result;
-    do
-    {
-        firstIA= SmartPtr<TAddrIA>();
+    // FIXME: Reimplement check request support.
+    return;
 
-        TContainer<SmartPtr<TAddrIA> > requestIALst;
+    SmartPtr<TAddrIA> ia;
+    SmartPtr<TClntCfgIA> cfgIA;
+    SmartPtr<TDUID> duid = 0;
+    List(TAddrIA) requestIALst;
+    int ifaceID = 0;
 
-        AddrMgr->firstIA();
-        while((ptrIA=AddrMgr->getIA())&&(!firstIA))
-        {
-            //find first IA which is not in process and all addresses
-            //were checked against duplication and dosen't have assigned
-            //all addresses
-            ptrCfgIA=CfgMgr->getIA(ptrIA->getIAID());
-            if ((ptrIA->getTentative()==NO)&&
-                (ptrIA->getState()!=INPROCESS)&&
-                (ptrIA->countAddr()<ptrCfgIA->countAddr()))
-            {
-                firstIA=ptrIA;
-                firstIAGroup=CfgMgr->getGroupForIA(ptrIA->getIAID());
-            }
-        }
-        if (firstIA)
-        {
-            requestIALst.append(firstIA);
-            firstIA->setState(INPROCESS);
-            while(ptrIA=AddrMgr->getIA())
-            {
-                //find other IA's, which is not in process and all addresses
-                //were checked against DAD, and belong to the same group
-                //were received from one server
-                if(((ptrIA->getTentative()==NO)&&
-                    (ptrIA->getState()!=INPROCESS)&&
-                    (ptrIA->countAddr()<ptrCfgIA->countAddr()))&&
-                    (*ptrIA->getDUID()==*firstIA->getDUID())&&
-                    (&(*firstIAGroup)==&(*CfgMgr->getGroupForIA(ptrIA->getIAID())))
-                  )
-                {
-                    ptrIA->setState(INPROCESS);
-                    requestIALst.append(ptrIA);
-                }
+    requestIALst.clear();
+    AddrMgr->firstIA();
+    while( ia=AddrMgr->getIA() ) {
 
-            }
+	cfgIA = CfgMgr->getIA(ia->getIAID());
+	if (!cfgIA) {
+	    Log(Error) << "Internal sanity check failed. Unable to find IA (iaid=" << ia->getIAID()
+		       << ") in the CfgMgr." << LogEnd;
+	    continue;
+	}
 
-            //Here should be send decline for all tentative addresses in IAs
-            SmartPtr<TClntMsgRequest> request = 
-                new TClntMsgRequest(IfaceMgr, That, CfgMgr, AddrMgr,
-				    requestIALst,firstIA->getDUID(), firstIA->getIface());
-            Transactions.append( (Ptr*) request);
-        }
-    } while(firstIA);
+	// find all IAs which should be configured:
+	if ( (cfgIA->getState()!=STATE_CONFIGURED) )
+	    continue;
+
+	// this IA is being serviced by different server
+	if ( (duid) && ia->getDUID() && !((*ia->getDUID()) == (*duid)) ) {
+	    continue;
+	}
+
+	// if we have found one IA, following ones must be on the same interface
+	if ( ifaceID && (ia->getIface()!=ifaceID) ) {
+	    continue;
+	}
+
+	// do we have enough addresses?
+	// cfgIA - specified in conf file, ia - actually asigned by the server
+	if ( ia->countAddr() >= cfgIA->countAddr() ) 
+	    continue;
+	
+	Log(Info) << "IA (iaid=" << ia->getIAID() << ") is configured to get " 
+		  << cfgIA->countAddr() << " addr, but server provided " << ia->countAddr() 
+		  << ", REQUEST will be sent." << LogEnd;
+
+	requestIALst.append(ia);
+	ia->setState(STATE_INPROCESS);
+	duid = ia->getDUID();
+	ifaceID = ia->getIface();
+    }
+
+    if (requestIALst.count()) {
+	// create REQUEST message
+	SmartPtr<TClntMsgRequest> request;
+	request = new TClntMsgRequest(IfaceMgr, That, CfgMgr, AddrMgr,
+				      requestIALst, duid, ifaceID );
+	Transactions.append( (Ptr*) request);
+    } 
 }
 
 bool TClntTransMgr::isDone()
@@ -900,10 +987,10 @@ bool TClntTransMgr::isDone()
     return IsDone;
 }
 
-void TClntTransMgr::setThat(SmartPtr<TClntTransMgr> that)
+void TClntTransMgr::setContext(SmartPtr<TClntTransMgr> that)
 {
     this->That=that;
-    IfaceMgr->setThats(IfaceMgr,That,CfgMgr,AddrMgr);
+    IfaceMgr->setContext(IfaceMgr,That,CfgMgr,AddrMgr);
 }
 
 char* TClntTransMgr::getCtrlAddr() {
@@ -915,4 +1002,102 @@ int  TClntTransMgr::getCtrlIface() {
 
 TClntTransMgr::~TClntTransMgr() {
     Log(Debug) << "ClntTransMgr cleanup." << LogEnd;
+    this->IfaceMgr->setContext(0,0,0,0);
+    this->IfaceMgr  = 0;
+    this->That      = 0;
+    this->CfgMgr    = 0;
+    this->AddrMgr   = 0;
+}
+
+void TClntTransMgr::addAdvertise(SPtr<TMsg> advertise)
+{
+    AdvertiseLst.append( advertise );
+}
+
+void TClntTransMgr::firstAdvertise()
+{
+    AdvertiseLst.first();
+}
+
+SPtr<TMsg> TClntTransMgr::getAdvertise()
+{
+    return AdvertiseLst.get();
+}
+
+SPtr<TOpt> TClntTransMgr::getAdvertiseDUID()
+{
+    if (!AdvertiseLst.count())
+	return 0;
+    AdvertiseLst.first();
+    SPtr<TMsg> msg = AdvertiseLst.get();
+    return msg->getOption(OPTION_SERVERID);
+}
+
+void TClntTransMgr::delFirstAdvertise()
+{
+    AdvertiseLst.first();
+    AdvertiseLst.delFirst();
+} 
+
+int TClntTransMgr::getAdvertiseLstCount()
+{
+    return AdvertiseLst.count();
+}
+
+int TClntTransMgr::getMaxPreference()
+{
+    if (AdvertiseLst.count() == 0)
+	return -1;
+    int max = 0;
+    
+    SmartPtr<TClntMsgAdvertise> ptr;
+    AdvertiseLst.first();
+    while ( ptr = (Ptr*) AdvertiseLst.get() ) {
+	if ( max < ptr->getPreference() )
+	    max = ptr->getPreference();
+    }
+    return max;
+}
+
+
+void TClntTransMgr::printLst(List(TMsg) lst)
+{
+    SPtr<TMsg> x;
+    SPtr<TClntMsgAdvertise> adv;
+    lst.first();
+    while (x = lst.get()) {
+	adv = (Ptr*) x;
+	Log(Debug) << "Advertise from " << adv->getInfo() << "." << LogEnd;
+    }
+}
+
+void TClntTransMgr::sortAdvertiseLst()
+{
+    // we'll store all ADVERTISE here 
+    List(TMsg) sorted;
+
+    // sort ADVERTISE by the PREFERENCE value
+    SmartPtr<TClntMsgAdvertise> ptr;
+    while (AdvertiseLst.count()) {
+	int max = getMaxPreference();
+	AdvertiseLst.first();
+	while ( ptr = (Ptr*) AdvertiseLst.get() ) {
+	    if (ptr->getPreference() == max) 
+		break;
+	}
+	
+	// did we find it? Then append it on the end of sorted list, and delete from this new.
+	if (ptr) {
+	    sorted.append( (Ptr*) ptr );
+	    AdvertiseLst.del();
+	}
+    }
+
+    // now copy sorted list to AnswersLst
+    AdvertiseLst = sorted;
+
+}
+
+void TClntTransMgr::printAdvertiseLst() {
+    printLst(AdvertiseLst);
 }
