@@ -3,58 +3,14 @@
  *
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>
  *          Marek Senderski <msend@o2.pl>
+ * changes: Michal Kowalczuk <michal@kowalczuk.eu>
  *
  * Released under GNU GPL v2 licence
  *
- * $Id: DHCPConst.h,v 1.16 2005/08/03 22:47:34 thomson Exp $
- *
- * $Log: DHCPConst.h,v $
- * Revision 1.16  2005/08/03 22:47:34  thomson
- * Support for 'share' parameter added,
- * Support for randomization between classes added.
- *
- * Revision 1.15  2005/01/08 16:52:04  thomson
- * Relay support implemented.
- *
- * Revision 1.14  2004/11/30 00:43:24  thomson
- * SIP option value mismatch fixed (bug #68)
- *
- * Revision 1.13  2004/11/29 17:42:33  thomson
- * Update option values to RFC3898
- *
- * Revision 1.12  2004/10/27 22:07:56  thomson
- * Signed/unsigned issues fixed, Lifetime option implemented, INFORMATION-REQUEST
- * message is now sent properly. Valid lifetime granted by server fixed.
- *
- * Revision 1.11  2004/10/25 20:45:54  thomson
- * Option support, parsers rewritten. ClntIfaceMgr now handles options.
- *
- * Revision 1.10  2004/10/03 21:52:12  thomson
- * FALSE changed to false
- *
- * Revision 1.9  2004/10/03 21:21:32  thomson
- * CLIENT_DEFAULT_UNICAST is set to false.
- *
- * Revision 1.8  2004/09/28 21:49:32  thomson
- * no message
- *
- * Revision 1.7  2004/09/07 22:02:34  thomson
- * pref/valid/IAID is not unsigned, RAPID-COMMIT now works ok.
- *
- * Revision 1.6  2004/09/07 17:42:31  thomson
- * Server Unicast implemented.
- *
- * Revision 1.5  2004/09/03 23:20:23  thomson
- * RAPID-COMMIT support fixed. (bugs #50, #51, #52)
- *
- * Revision 1.4  2004/06/04 19:03:47  thomson
- * Resolved warnings with signed/unisigned
- *
- * Revision 1.3  2004/03/29 22:06:49  thomson
- * 0.1.1 version
- *
  */
 
+#include "limits.h"
+
 #ifndef DHCPCONST_H
 #define DHCPCONST_H
 
@@ -78,6 +34,10 @@
 #define INFORMATION_REQUEST_MSG 11
 #define RELAY_FORW_MSG 12
 #define RELAY_REPL_MSG 13
+// unimplemented messages
+#define LEASEQUERY     14
+#define LEASEQUERY_REPLY 15
+// implementation specific
 #define CONTROL_MSG    255
 
 // timers, timeouts
@@ -114,7 +74,8 @@
 // RFC3315: supported options
 #define OPTION_CLIENTID         1
 #define OPTION_SERVERID         2
-#define OPTION_IA               3
+#define OPTION_IA_NA            3
+#define OPTION_IA_TA            4
 #define OPTION_IAADDR           5
 #define OPTION_ORO              6
 #define OPTION_PREFERENCE       7
@@ -123,10 +84,9 @@
 #define OPTION_STATUS_CODE      13
 #define OPTION_RAPID_COMMIT     14
 
-// RFC: options not supported yet
-#define OPTION_IA_TA            4
+// RFC3315: options not supported yet
 #define OPTION_RELAY_MSG        9
-#define OPTION_AUTH_MSG         11
+#define OPTION_AUTH             11
 #define OPTION_USER_CLASS       15
 #define OPTION_VENDOR_CLASS     16
 #define OPTION_VENDOR_OPTS      17
@@ -137,14 +97,14 @@
 // additional options
 
 // RFC3319: SIP servers and domains
-#define OPTION_SIP_DOMAINS      21
-#define OPTION_SIP_SERVERS      22
+#define OPTION_SIP_SERVER_D      21
+#define OPTION_SIP_SERVER_A      22
 
 // RFC3646: DNS servers and domains
-#define OPTION_DNS_RESOLVERS	23
+#define OPTION_DNS_SERVERS	23
 #define OPTION_DOMAIN_LIST      24
 
-// RFC3633: Prefix options (not supported yet)
+// RFC3633: Prefix options
 #define OPTION_IA_PD            25
 #define OPTION_IAPREFIX         26
 
@@ -154,15 +114,64 @@
 #define OPTION_NIS_DOMAIN_NAME  29
 #define OPTION_NISP_DOMAIN_NAME 30
 
-// draft-ietf-dhc-dhcpv6-opt-timeconfig-02.txt
-#define OPTION_NTP_SERVERS      40
-#define OPTION_TIME_ZONE        41
+// RFC4075: Simple Network Time Protocol (SNTP)
+#define OPTION_SNTP_SERVERS     31
 
-// draft-ietf-dhc-lifetime-02.txt
-#define OPTION_LIFETIME         42
+// RFC4242: Information Refresh Time Option
+#define OPTION_INFORMATION_REFRESH_TIME     32
 
-// draft-ietf-dhc-dhcpv6-fqdn-00.txt
-#define OPTION_FQDN             43
+// RFC4280: Broadcast and Multicast Control Servers
+#define OPTION_BCMCS_SERVER_D   33
+#define OPTION_BCMCS_SERVER_A   34
+
+// RFC4776: Option for Civic Addresses Configuration Information
+#define OPTION_GEOCONF_CIVIC    36
+
+// RFC4649: Relay Agent Remote-ID Option
+#define OPTION_REMOTE_ID        37
+
+// RFC4580: Relay Agent Subscriber-ID Option
+#define OPTION_SUBSCRIBER_ID    38
+
+// RFC4704: Client Fully Qualified Domain Name (FQDN) Option
+#define OPTION_FQDN             39
+
+// RFC-ietf-dhc-paa-option-05.txt
+#define OPTION_PANA_AGENT       40
+
+// RFC4833: Timezone options for DHCP 
+#define OPTION_NEW_POSIX_TIMEZONE 41
+#define OPTION_NEW_TZDB_TIMEZONE  42
+// old implementation define
+#define OPTION_TIME_ZONE          41
+
+// RFC-ietf-dhc-dhcpv6-ero-01.txt
+#define OPTION_ERO              43
+
+// RFC-ietf-dhc-dhcpv6-leasequery-01.txt
+#define OPTION_LQ_QUERY         44
+
+// RFC-ietf-dhc-dhcpv6-leasequery-01.txt
+#define OPTION_CLIENT_DATA      45
+
+// RFC-ietf-dhc-dhcpv6-leasequery-01.txt
+#define OPTION_CLT_TIME         46
+#define OPTION_LQ_RELAY_DATA    47
+#define OPTION_LQ_CLIENT_LINK   48
+
+// Experimental implementation for address prefix length information
+// See: http://klub.com.pl/dhcpv6/doc/draft-mrugalski-addropts-XX-2007-04-17.txt
+#define OPTION_ADDRPARAMS           251
+
+// RFC-ietf-dhc-dhcpv6-leasequery-01.txt
+// -- Query types --
+#define QUERY_BY_ADDRESS  1
+#define QUERY_BY_CLIENTID 2
+
+// --- Option lengths --
+// (value of the len field, so actual option length is +4 bytes)
+#define OPTION_ELAPSED_TIME_LEN     2
+#define OPTION_INFORMATION_REFRESH_TIME_LEN         4
 
 // --- Status Codes --- 
 #define STATUSCODE_SUCCESS       0
@@ -171,12 +180,18 @@
 #define STATUSCODE_NOBINDING     3
 #define STATUSCODE_NOTONLINK     4
 #define STATUSCODE_USEMULTICAST  5
+#define STATUSCODE_NOPREFIXAVAIL 6 
+// unimplemented codes
+#define STATUSCODE_UNKNOWNQUERYTYPE 7
+#define STATUSCODE_MALFORMEDQUERY 8
+#define STATUSCODE_NOTCONFIGURED 9
+#define STATUSCODE_NOTALLOWED    10
 
 // INFINITY + 1 is 0. Hih, cool
 #define DHCPV6_INFINITY (unsigned) 0xffffffff
 
 // how long should we wait before we assume that OS detected duplicated addresses (in secs)
-#define DADTIMEOUT (unsigned) 1
+#define DADTIMEOUT (unsigned) 3
 
 // addresses reported ad DECLINED are not used for 2 hours
 #define DECLINED_TIMEOUT (unsigned) 7200
@@ -190,38 +205,76 @@
 #define SERVER_DEFAULT_RAPIDCOMMIT false
 
 #define SERVER_DEFAULT_MIN_T1    5
-#define SERVER_DEFAULT_MAX_T1    ULONG_MAX
+#define SERVER_DEFAULT_MAX_T1    3600      /* 1 hour */
 #define SERVER_DEFAULT_MIN_T2    10
-#define SERVER_DEFAULT_MAX_T2    ULONG_MAX
-#define SERVER_DEFAULT_MIN_PREF  1800
+#define SERVER_DEFAULT_MAX_T2    5400      /* 1,5 hour */
+#define SERVER_DEFAULT_MIN_PREF  7200      /* 2 hours */
 #define SERVER_DEFAULT_MAX_PREF  86400     /* 1 day */
-#define SERVER_DEFAULT_MIN_VALID 3600
+#define SERVER_DEFAULT_MIN_VALID 10800     /* 3 hours */
 #define SERVER_DEFAULT_MAX_VALID 172800    /* 2 days */
 #define SERVER_DEFAULT_CLASS_SHARE 100
 #define SERVER_DEFAULT_CLASS_MAX_LEASE ULONG_MAX
-
+#define SERVER_DEFAULT_TA_PREF_LIFETIME 3600
+#define SERVER_DEFAULT_TA_VALID_LIFETIME 7200
+#define SERVER_DEFAULT_CACHE_SIZE 1048576   /* cache size, specified in bytes */
 #define CLIENT_DEFAULT_T1 ULONG_MAX
 #define CLIENT_DEFAULT_T2 ULONG_MAX
 
 #define CLIENT_DEFAULT_UNICAST false
 #define CLIENT_DEFAULT_RAPID_COMMIT false
 
+#define CLIENT_DEFAULT_PREFIX_LENGTH 64
+#define CLIENT_DEFAULT_DIGEST DIGEST_NONE
 
-enum ETentative {YES, NO, DONTKNOWYET};
-enum EState {NOTCONFIGURED, INPROCESS, CONFIGURED, FAILED, DISABLED, UNKNOWN, TENTATIVECHECK, TENTATIVE};
+enum ETentative {
+    TENTATIVE_UNKNOWN = -1,
+    TENTATIVE_NO      = 0,
+    TENTATIVE_YES     = 1
+};
+
+enum EState {
+    STATE_NOTCONFIGURED, 
+    STATE_INPROCESS, 
+    STATE_CONFIGURED, 
+    STATE_FAILED, 
+    STATE_DISABLED, 
+    STATE_TENTATIVECHECK, 
+    STATE_TENTATIVE};
 
 // FQDN option flags
 #define FQDN_N 0x4
 #define FQDN_O 0x2
 #define FQDN_S 0x1
+
+// --- Bitfield in ADDRPARAMS option ---
+#define ADDRPARAMS_MASK_PREFIX    0x01
+#define ADDRPARAMS_MASK_ANYCAST   0x02
+#define ADDRPARAMS_MASK_MULTICAST 0x04
+
     
-bool allowOptInOpt(int msgType, int optOut, int optIn);
-bool allowOptInMsg(int msgType, int optType);
+int allowOptInOpt(int msgType, int optOut, int optIn);
+int allowOptInMsg(int msgType, int optType);
 
 #ifdef WIN32
 #define uint16_t unsigned short int
 #define uint32_t unsigned int
 #endif
 
+enum DigestTypes {
+        DIGEST_NONE = 0,
+        DIGEST_PLAIN = 1,
+        DIGEST_HMAC_MD5 = 2,
+        DIGEST_HMAC_SHA1 = 3,
+        DIGEST_HMAC_SHA224 = 4,
+        DIGEST_HMAC_SHA256 = 5,
+        DIGEST_HMAC_SHA384 = 6,
+        DIGEST_HMAC_SHA512 = 7
+};
+
+unsigned getDigestSize(enum DigestTypes type);
+        
+// for debugging purposes:
+#define printhex(buf, len) { unsigned j; for (j = 0; j < len; j++) printf("%02x ", (unsigned char) *(buf+j)); printf("\n"); }
+
 #endif
 
