@@ -6,15 +6,7 @@
  *
  * released under GNU GPL v2 or later licence
  *
- * $Id: RelParsGlobalOpt.h,v 1.2 2005/07/16 14:46:32 thomson Exp $
- *
- * $Log: RelParsGlobalOpt.h,v $
- * Revision 1.2  2005/07/16 14:46:32  thomson
- * Compatilibility with gcc 2.95 improved (bug #119)
- * Fix was provided by Tomasz Torcz. Thanks.
- *
- * Revision 1.1  2005/01/11 22:53:35  thomson
- * Relay skeleton implemented.
+ * $Id: RelParsGlobalOpt.h,v 1.4 2007-05-01 12:03:13 thomson Exp $
  *
  */
 
@@ -23,6 +15,12 @@
 #include "RelParsIfaceOpt.h"
 #include <string>
 
+typedef enum {
+    REL_IFACE_ID_ORDER_BEFORE,
+    REL_IFACE_ID_ORDER_AFTER,
+    REL_IFACE_ID_ORDER_NONE
+} ERelIfaceIdOrder;
+
 class TRelParsGlobalOpt : public TRelParsIfaceOpt
 {
  public:
@@ -31,8 +29,17 @@ class TRelParsGlobalOpt : public TRelParsIfaceOpt
     
     string getWorkDir();
     void setWorkDir(string dir);
+
+    void setGuessMode(bool guess);
+    bool getGuessMode();
+
+    void setInterfaceIDOrder(ERelIfaceIdOrder order);
+    ERelIfaceIdOrder getInterfaceIDOrder();
     
 private:
     string WorkDir;
+    bool GuessMode;
+
+    ERelIfaceIdOrder InterfaceIDOrder;
 };
 #endif
