@@ -6,16 +6,39 @@
  *                                                                           
  * released under GNU GPL v2 or later licence                                
  *                                                                           
- * $Id: SrvCfgIface.cpp,v 1.20 2005/08/03 22:47:34 thomson Exp $
+ * $Id: SrvCfgIface.cpp,v 1.40 2007-05-03 23:16:29 thomson Exp $
  */
 
+#include <cstdlib>
 #include <sstream>
 #include "SrvCfgIface.h"
 #include "SrvCfgAddrClass.h"
+#include "SrvCfgPD.h"
+#include "DNSUpdate.h"
 #include "Logger.h"
 
 using namespace std;
 
+void TSrvCfgIface::addClientExceptionsLst(List(TSrvCfgOptions) exLst)
+{
+    Log(Debug) << exLst.count() << " per-client configurations (exceptions) added." << LogEnd;
+    ExceptionsLst = exLst;
+}
+
+SPtr<TSrvCfgOptions> TSrvCfgIface::getClientException(SPtr<TDUID> duid, bool quiet)
+{
+    SPtr<TSrvCfgOptions> x;
+    ExceptionsLst.first();
+    while (x=ExceptionsLst.get()) {
+	if (*(x->getDuid()) == *duid) {
+	    if (!quiet)
+		Log(Debug) << "Found per-client configuration (exception) for client with DUID=" << x->getDuid()->getPlain() << LogEnd;
+	    return x;
+	}
+    }
+    return 0;
+}
+
 void TSrvCfgIface::firstAddrClass() {
     this->SrvCfgAddrClassLst.first();
 }
@@ -74,6 +97,51 @@ bool TSrvCfgIface::getAllowedAddrClassID(SmartPtr<TDUID> duid, SmartPtr<TIPv6Add
     return true;
 }
 
+
+void TSrvCfgIface::firstPD() {
+    this->SrvCfgPDLst.first();
+}
+
+bool TSrvCfgIface::supportPrefixDelegation() {
+    return this->PrefixDelegationSupport;
+}
+
+void TSrvCfgIface::addTA(SmartPtr<TSrvCfgTA> ta) {
+    this->SrvCfgTALst.append(ta);
+}
+
+void TSrvCfgIface::firstTA() {
+    this->SrvCfgTALst.first();
+}
+SmartPtr<TSrvCfgTA> TSrvCfgIface::getTA() {
+    return this->SrvCfgTALst.get();
+}
+
+void TSrvCfgIface::addPD(SmartPtr<TSrvCfgPD> pd) {
+    this->PrefixDelegationSupport = true;
+    this->SrvCfgPDLst.append(pd);
+}
+
+SmartPtr<TSrvCfgTA> TSrvCfgIface::getTA(SmartPtr<TDUID> clntDuid, SmartPtr<TIPv6Addr> clntAddr) {
+    SmartPtr<TSrvCfgTA> ta;
+
+    // try to find preferred TA for this client
+    this->SrvCfgTALst.first();
+    while ( ta = this->getTA() ) {
+	if (ta->clntPrefered(clntDuid, clntAddr))
+	    return ta;
+    }
+
+    // prefered not found? Then find first allowed
+    this->SrvCfgTALst.first();
+    while ( ta = this->getTA() ) {
+	if (ta->clntSupported(clntDuid, clntAddr))
+	    return ta;
+    }
+
+    return 0;
+}
+
 SmartPtr<TSrvCfgAddrClass> TSrvCfgIface::getAddrClass() {
     return SrvCfgAddrClassLst.get();
 }
@@ -149,6 +217,93 @@ long TSrvCfgIface::countAddrClass() {
     return this->SrvCfgAddrClassLst.count();
 }
 
+
+
+/** Prefix delegation functions 
+
+*/
+
+SmartPtr<TSrvCfgPD> TSrvCfgIface::getPD() {
+    return SrvCfgPDLst.get();
+}
+
+SmartPtr<TSrvCfgPD> TSrvCfgIface::getPDByID(unsigned long id) {
+    this->firstPD();
+    SmartPtr<TSrvCfgPD> ptrPD;
+    while (ptrPD = this->getPD()) {
+	if (ptrPD->getID() == id)
+	    return ptrPD;
+    }
+    return 0;
+}
+
+bool TSrvCfgIface::addClntPrefix(SmartPtr<TIPv6Addr> ptrAddr) {
+    SmartPtr<TSrvCfgPD> ptrPD;
+    this->firstPD();
+    while (ptrPD = this->getPD() ) {
+	if (ptrPD->prefixInPool(ptrAddr)) {
+	    Log(Debug) << "PD: Prefix usage for class " << ptrPD->getID()
+		       << " increased by 1." << LogEnd;
+	    ptrPD->incrAssigned();
+	    return true;
+	}
+    }
+    Log(Warning) << "Unable to increase prefix usage: no prefix found for " 
+		 << *ptrAddr << LogEnd;
+    return false;
+}
+
+bool TSrvCfgIface::delClntPrefix(SmartPtr<TIPv6Addr> ptrAddr) {
+    SmartPtr<TSrvCfgPD> ptrPD;
+    this->firstPD();
+    while (ptrPD = this->getPD() ) {
+	if (ptrPD->prefixInPool(ptrAddr)) {
+	    ptrPD->decrAssigned();
+	    Log(Debug) << "PD: Prefix usage for class " << ptrPD->getID()
+		       << " decreased by 1." << LogEnd;
+	    return true;
+	}
+    }
+    Log(Warning) << "Unable to decrease address usage: no class found for " 
+		 << *ptrAddr << LogEnd;
+    return false;
+}
+
+#if 0
+SmartPtr<TSrvCfgPD> TSrvCfgIface::getRandomPrefix(SmartPtr<TDUID> clntDuid, 
+							SmartPtr<TIPv6Addr> clntAddr) {
+/* this method is not used currently 
+   
+      unsigned long classid;
+
+    // step 1: Is there a class reserved for this client?
+
+    // if there is class where client is on whitelist, it should be used rather than any other class 
+    // that would be also suitable
+    if(this->getPreferedAddrClassID(clntDuid, clntAddr, classid)) {
+      Log(Debug) << "Found prefered class for client (duid = " << *clntDuid << ", addr = "
+  	        << *clntAddr << ")" << LogEnd;
+      return this->getClassByID(classid);
+    } 
+
+    // Get one of the normal classes
+    if(this->getAllowedAddrClassID(clntDuid, clntAddr, classid)) {
+	Log(Debug) << "Prefered class for client not found, using classid=" << classid << "." << LogEnd; 
+	return this->getClassByID(classid);
+    } 
+
+    // This is some kind of problem...
+    Log(Error) << "No class is available for client (duid=" << clntDuid->getPlain() << ", addr="
+	       << clntAddr->getPlain() << ")." << LogEnd;*/
+    return 0;
+}
+#endif
+
+long TSrvCfgIface::countPD() {
+    return this->SrvCfgPDLst.count();
+}
+
+
 int TSrvCfgIface::getID() {
     return this->ID;
 }
@@ -181,20 +336,26 @@ void TSrvCfgIface::setOptions(SmartPtr<TSrvParsGlobalOpt> opt) {
     this->RapidCommit   = opt->getRapidCommit();
     this->Unicast       = opt->getUnicast();
     
-    // option: DNS-SERVERS
-    if (opt->supportDNSServer())  this->setDNSServerLst(opt->getDNSServerLst());
-    if (opt->supportDomain())     this->setDomainLst(opt->getDomainLst());
-    if (opt->supportNTPServer())  this->setNTPServerLst(opt->getNTPServerLst());
-    if (opt->supportTimezone())   this->setTimezone(opt->getTimezone());
-    if (opt->supportFQDN())       this->setFQDN(opt->getFQDN());
-    if (opt->supportSIPServer())  this->setSIPServerLst(opt->getSIPServerLst());
-    if (opt->supportSIPDomain())  this->setSIPDomainLst(opt->getSIPDomainLst());
-    if (opt->supportNISServer())  this->setNISServerLst(opt->getNISServerLst());
-    if (opt->supportNISDomain())  this->setNISDomain(opt->getNISDomain());
-    if (opt->supportNISPServer()) this->setNISPServerLst(opt->getNISPServerLst());
-    if (opt->supportNISPDomain()) this->setNISPDomain(opt->getNISPDomain());
-    if (opt->supportLifetime())   this->setLifetime(opt->getLifetime());
-
+    if (opt->supportFQDN()){
+	this->setFQDNLst(opt->getFQDNLst());
+	this->setFQDNMode(opt->getFQDNMode());
+	this->setRevDNSZoneRootLength(opt->getRevDNSZoneRootLength());
+	Log(Debug) <<"FQDN: Support is enabled on the " << this->getName()  << " interface." << LogEnd;
+	Log(Debug) <<"FQDN: Mode set to " << this->getFQDNMode() << ": ";
+	switch (this->getFQDNMode()) {
+	case DNSUPDATE_MODE_NONE:
+	    Log(Cont) << "server will not perform any updates." << LogEnd;
+	    break;
+	case DNSUPDATE_MODE_PTR:
+	    Log(Cont) << "server will perform reverse (PTR) update only." << LogEnd;
+	    break;
+	case DNSUPDATE_MODE_BOTH:
+	    Log(Cont) << "server will perform both (AAAA and PTR) updates." << LogEnd;
+	    break;
+	}
+    	Log(Debug) <<"FQDN: revDNS zoneroot lenght set to " << this->getRevDNSZoneRootLength()<< "." << LogEnd;
+    }
+  
     if (opt->isRelay()) {
 	this->Relay = true;
 	this->RelayName        = opt->getRelayName();
@@ -206,6 +367,8 @@ void TSrvCfgIface::setOptions(SmartPtr<TSrvParsGlobalOpt> opt) {
 	this->RelayID = 0;
 	this->RelayInterfaceID = 0;
     }
+
+    TSrvCfgOptions::setOptions(opt);
 }
 
 /*
@@ -232,19 +395,9 @@ void TSrvCfgIface::setDefaults() {
     this->ID = -1;
     this->NoConfig = false;
     this->preference = 0;
-    
-    this->DNSServerSupport  = false;
-    this->DomainSupport     = false;
-    this->NTPServerSupport  = false;
-    this->TimezoneSupport   = false;
-    this->FQDNSupport       = false;
-    this->SIPServerSupport  = false;
-    this->SIPDomainSupport  = false;
-    this->NISServerSupport  = false;
-    this->NISDomainSupport  = false;
-    this->NISPServerSupport = false;
-    this->NISPDomainSupport = false;
-    this->LifetimeSupport   = false;
+   
+    this->FQDNSupport             = false;
+    this->PrefixDelegationSupport = false;
 }
 
 void TSrvCfgIface::setNoConfig() {
@@ -304,154 +457,123 @@ void TSrvCfgIface::setRelayID(int id) {
     this->RelayID = id;
 }
 
-
-// --------------------------------------------------------------------
-// --- options --------------------------------------------------------
-// --------------------------------------------------------------------
-// --- option: DNS servers ---
-void TSrvCfgIface::setDNSServerLst(List(TIPv6Addr) *lst) {
-    this->DNSServerLst = *lst;
-    this->DNSServerSupport = true;
-}
-List(TIPv6Addr) * TSrvCfgIface::getDNSServerLst() {
-    return &this->DNSServerLst;
-}
-bool TSrvCfgIface::supportDNSServer(){
-    return this->DNSServerSupport;
-}
-
-// --- option: DOMAIN ---
-void TSrvCfgIface::setDomainLst(List(string) * lst) {
-    this->DomainLst = *lst;
-    this->DomainSupport = true;
-}
-List(string) * TSrvCfgIface::getDomainLst() {
-    return &this->DomainLst;
-}
-bool TSrvCfgIface::supportDomain(){
-    return this->DomainSupport;
-}
-
-// --- option: NTP-SERVERS ---
-void TSrvCfgIface::setNTPServerLst(List(TIPv6Addr) * lst) {
-    this->NTPServerLst = *lst;
-    this->NTPServerSupport = true;
-}
-List(TIPv6Addr) * TSrvCfgIface::getNTPServerLst() {
-    return &this->NTPServerLst;
-}
-bool TSrvCfgIface::supportNTPServer(){
-    return this->NTPServerSupport;
-}
-
-// --- option: TIMEZONE ---
-void TSrvCfgIface::setTimezone(string timezone) {
-    this->Timezone=timezone;
-    this->TimezoneSupport = true;
-}
-string TSrvCfgIface::getTimezone() {
-    return this->Timezone;
-}
-bool TSrvCfgIface::supportTimezone(){
-    return this->NTPServerSupport;
-}
-
-// --- option: SIP server ---
-void TSrvCfgIface::setSIPServerLst(TContainer<SmartPtr<TIPv6Addr> > *lst) {
-    this->SIPServerLst = *lst;
-    this->SIPServerSupport = true;
-}
-List(TIPv6Addr) * TSrvCfgIface::getSIPServerLst() {
-    return &this->SIPServerLst;
-}
-bool TSrvCfgIface::supportSIPServer(){
-    return this->SIPServerSupport;
-}
-
-// --- option: SIP domain ---
-List(string) * TSrvCfgIface::getSIPDomainLst() { 
-    return &this->SIPDomainLst;
-}
-void TSrvCfgIface::setSIPDomainLst(List(string) * domain) { 
-    this->SIPDomainLst = *domain;
-    this->SIPDomainSupport = true;
-}
-bool TSrvCfgIface::supportSIPDomain() {
-    return this->SIPDomainSupport;
-}
-
 // --- option: FQDN ---
-void TSrvCfgIface::setFQDN(string fqdn) { 
-    this->FQDN=fqdn;
+void TSrvCfgIface::setFQDNLst(List(TFQDN) *fqdn) {
+    this->FQDNLst = *fqdn;
     this->FQDNSupport = true;
 }
-string TSrvCfgIface::getFQDN() { 
-    return this->FQDN;
+
+/** 
+ * this method tries to find a name for a client. It check client's hint, and possible reservations
+ * by duid or by address
+ * 
+ * @param duid 
+ * @param addr 
+ * @param hint 
+ * 
+ * @return 
+ */
+SPtr<TFQDN> TSrvCfgIface::getFQDNName(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr> addr, string hint) {
+    FQDNLst.first();
+
+    SPtr<TFQDN> bestFound=0; // best FQDN found for that client
+
+    SPtr<TFQDN> foo;
+    while (foo=this->FQDNLst.get()) {
+	if (foo->isUsed())
+	    continue;
+	if (duid && (foo->Duid) && *(foo->Duid)== *duid) {
+	    Log(Debug) << "FQDN found: " << foo->Name << " using duid " << duid->getPlain() << LogEnd;
+	    return foo;
+	}
+	if (addr && (foo->Addr) && *(foo->Addr)==*addr) {
+	    Log(Debug) << "FQDN found: " << foo->Name << " using address " << addr->getPlain() << LogEnd;
+	    return foo;
+	}
+	
+	if (foo->Name == hint){
+	    // client asked for this name. Let's check if client is allowed to get this name.
+	    if ( (!foo->Duid) && (!foo->Addr) ) {
+		Log(Debug) << "Client's hint: " << hint << " found in fqdn list, setting fqdn to "<< foo->Name << LogEnd;
+		return foo;
+	    }
+	}
+	if (!foo->Addr && !foo->Duid) {
+	    if (!bestFound)
+		bestFound = foo;
+	    return foo;
+	}
+    }
+    
+    if (bestFound) {
+	Log(Debug) << "Not reserved FQDN found: " << foo->Name << LogEnd;
+	return bestFound;
+    }
+
+    // not found
+    return 0;
 }
+
+SmartPtr<TDUID> TSrvCfgIface::getFQDNDuid(string name) {
+    SmartPtr<TDUID> res = new TDUID();
+    return res;
+}
+
+List(TFQDN) *TSrvCfgIface::getFQDNLst() {
+    return &this->FQDNLst;
+}
+
 bool TSrvCfgIface::supportFQDN() {
     return this->FQDNSupport;
 }
 
-// --- option: NIS server ---
-void TSrvCfgIface::setNISServerLst(TContainer<SmartPtr<TIPv6Addr> > *lst) {
-    this->NISServerLst     = *lst;
-    this->NISServerSupport = true;
+void TSrvCfgIface::setFQDNMode(int FQDNMode) {
+    this->FQDNMode = FQDNMode;
 }
-List(TIPv6Addr) * TSrvCfgIface::getNISServerLst() {
-    return &this->NISServerLst;
+int TSrvCfgIface::getFQDNMode(){
+    return this->FQDNMode;
 }
-bool TSrvCfgIface::supportNISServer(){
-    return this->NISServerSupport;
+int TSrvCfgIface::getRevDNSZoneRootLength(){
+	return this->revDNSZoneRootLength;
+}
+void TSrvCfgIface::setRevDNSZoneRootLength(int revDNSZoneRootLength){
+	this->revDNSZoneRootLength=revDNSZoneRootLength;
 }
 
-// --- option: NIS domain ---
-void TSrvCfgIface::setNISDomain(string domain) { 
-    this->NISDomain=domain;
-    this->NISDomainSupport=true;
-}
-string TSrvCfgIface::getNISDomain() { 
-    return this->NISDomain;
-}
-bool TSrvCfgIface::supportNISDomain() {
-    return this->NISDomainSupport;
+string TSrvCfgIface::getFQDNModeString() {
+    switch (this->FQDNMode) {
+    case 0:  return "updates disabled";
+    case 1:  return "server will update PTR";
+    case 2:  return "server will update PTR and AAAA";
+    default: return "unknown";
+    }
 }
 
-// --- option: NIS+ server ---
-void TSrvCfgIface::setNISPServerLst(TContainer<SmartPtr<TIPv6Addr> > *lst) {
-    this->NISPServerLst = *lst;
-    this->NISPServerSupport = true;
-}
-List(TIPv6Addr) * TSrvCfgIface::getNISPServerLst() {
-    return &this->NISPServerLst;
-}
-bool TSrvCfgIface::supportNISPServer(){
-    return this->NISPServerSupport;
+
+void TSrvCfgIface::addTAAddr() {
+    SmartPtr<TSrvCfgTA> ta;
+    this->firstTA();
+    ta=this->getTA();
+    if (!ta) {
+	Log(Error) << "Unable to increase TA usage. TA (temporary addresses) is not found on the " 
+		   << this->getFullName() << " interface." << LogEnd;
+	return;
+    }
+    ta->incrAssigned();
 }
 
-// --- option: NIS+ domain ---
-void TSrvCfgIface::setNISPDomain(string domain) { 
-    this->NISPDomain=domain;
-    this->NISPDomainSupport=true;
-}
-string TSrvCfgIface::getNISPDomain() { 
-    return this->NISPDomain;
-}
-bool TSrvCfgIface::supportNISPDomain() {
-    return this->NISPDomainSupport;
+void TSrvCfgIface::delTAAddr() {
+    SmartPtr<TSrvCfgTA> ta;
+    this->firstTA();
+    ta = this->getTA();
+    if (!ta) {
+	Log(Error) << "Unable to decrease TA usage. TA (temporary addresses) is not found on the " 
+		   << this->getFullName() << " interface." << LogEnd;
+	return;
+    }
+    ta->decrAssigned();
 }
 
-// --- option: LIFETIME ---
-void TSrvCfgIface::setLifetime(unsigned int x) {
-    this->Lifetime = x;
-    this->LifetimeSupport = true;
-}
-unsigned int TSrvCfgIface::getLifetime() {
-    return this->Lifetime;
-}
-
-bool TSrvCfgIface::supportLifetime() {
-    return this->LifetimeSupport;
-}
 
 // --------------------------------------------------------------------
 // --- operators ------------------------------------------------------
@@ -488,24 +610,44 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
         out << "    <!-- <rapid-commit/> -->" << std::endl;
     }
 
-    SmartPtr<TSrvCfgAddrClass>	groupPtr;
+    out << endl;
+    // print IA objects
+    SmartPtr<TSrvCfgAddrClass>	ia;
     iface.SrvCfgAddrClassLst.first();
-    out << "    <!-- IPv6 addr class count: " << iface.SrvCfgAddrClassLst.count() << "-->" << endl;
-    while(groupPtr=iface.SrvCfgAddrClassLst.get())
+    out << "    <!-- IA: non-temporary addr class count: " << iface.SrvCfgAddrClassLst.count() << "-->" << endl;
+    while( ia=iface.SrvCfgAddrClassLst.get() ) {	
+	out << *ia;
+    }
+
+    out << endl;
+    // print PD objects
+    SmartPtr<TSrvCfgPD>	pd;
+    iface.SrvCfgPDLst.first();
+    out << "    <!-- PD: prefix delegation class count: " << iface.SrvCfgPDLst.count() << "-->" << endl;
+    while( pd=iface.SrvCfgPDLst.get() ) {	
+	out << *pd;
+    }
+
+    out << endl;
+    // print TA objects
+    SmartPtr<TSrvCfgTA> ta;
+    iface.firstTA();
+    out << "    <!-- TA: temporary IPv6 addr class count: " << iface.SrvCfgTALst.count() << "-->" << endl;
+    while( ta=iface.getTA() )
     {	
-	out << *groupPtr;
+	out << *ta;
     }
     
-    out << "    <!-- options -->" << endl;
+    out << endl << "    <!-- options -->" << endl;
 
-    // DNS-SERVERS
+    // option: DNS-SERVERS
     out << "    <!-- <dns-servers count=\"" << iface.DNSServerLst.count() << "\"> -->" << endl;
     iface.DNSServerLst.first();
     while (addr = iface.DNSServerLst.get()) {
         out << "    <dns-server>" << *addr << "</dns-server>" << endl;
     }
 
-    // DOMAINS
+    // option: DOMAINS
     out << "    <!-- <domains count=\"" << iface.DomainLst.count() << "\"> -->" << endl;
     iface.DomainLst.first();
     while (str = iface.DomainLst.get()) {
@@ -540,13 +682,6 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
         out << "    <sip-domain>" << *str << "</sip-domain>" << endl;
     }
 
-    // option: FQDN
-    if (iface.supportFQDN()) {
-        out << "    <fqdn>" << iface.FQDN << "</fqdn>" << endl;
-    } else {
-        out << "    <!-- <fqdn/> -->" << endl;
-    }
-
     // option: NIS-SERVERS
     out << "    <!-- <nis-servers count=\"" << iface.NISServerLst.count() << "\"> -->" << endl;
     iface.NISServerLst.first();
@@ -570,7 +705,7 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
 
     // option: NIS+-DOMAIN
     if (iface.supportNISPDomain()) {
-        out << "    <nisplus-domain>" << iface.FQDN << "</nisplus-domain>" << endl;
+        out << "    <nisplus-domain>" << iface.NISPDomain << "</nisplus-domain>" << endl;
     } else {
         out << "    <!-- <nisplus-domain/> -->" << endl;
     }
@@ -581,49 +716,43 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
     } else {
         out << "    <!-- <lifetime/> -->" << endl;
     }
+
+    // option: VENDOR-SPEC
+    if (iface.supportVendorSpec()) {
+	out << "    <vendorSpecList count=\"" << iface.VendorSpec.count() << "\">" << endl;
+	iface.VendorSpec.first();
+        SPtr<TSrvOptVendorSpec> v;
+	while (v = iface.VendorSpec.get()) {
+	    out << "      <vendorSpec vendor=\"" << v->getVendor() << "\" length=\"" << v->getVendorDataLen() 
+	    << "\">" << v->getVendorDataPlain() << "</vendorSpec>" << endl;
+	}
+	out << "    </vendorSpecList>" << endl;
+    } else {
+	out << "    <!-- <vendorSpec/> -->" << endl;
+    }
+
+    // option: FQDN
+    if (iface.supportFQDN()) {
+      SPtr<TFQDN> f;
+      List(TFQDN) * lst = iface.getFQDNLst();
+      out << "    <fqdnOptions count=\"" << lst->count() << "\" prefix=\"" << iface.getRevDNSZoneRootLength() << "\">" << endl;
+      lst->first();
+      while (f=lst->get()) {
+	    out << "       " << *f;
+      }
+      out << "    </fqdnOptions>" << endl;
+    } else { 
+      out << "    <!-- <fqdnOptions/> -->" << endl;
+    }
+
+
+    SPtr<TSrvCfgOptions> ex;
+    out << "    <!-- " << iface.ExceptionsLst.count() << " per-client parameters (exceptions) -->" << endl;
+    iface.ExceptionsLst.first();
+    while (ex = iface.ExceptionsLst.get()) {
+	out << *ex;
+    }
     
     out << "  </SrvCfgIface>" << endl;
     return out;
 }
-
-/*
- * $Log: SrvCfgIface.cpp,v $
- * Revision 1.20  2005/08/03 22:47:34  thomson
- * Support for 'share' parameter added,
- * Support for randomization between classes added.
- *
- * Revision 1.19  2005/08/02 23:43:27  thomson
- * *** empty log message ***
- *
- * Revision 1.18  2005/08/02 00:33:58  thomson
- * White-list bug fixed (bug #120),
- * Minor compilation warnings in gcc 4.0 removed.
- *
- * Revision 1.17  2005/05/02 21:48:42  thomson
- * getFullName() method implemented.
- *
- * Revision 1.16  2005/01/08 16:52:04  thomson
- * Relay support implemented.
- *
- * Revision 1.15  2005/01/03 21:57:08  thomson
- * Relay support added.
- *
- * Revision 1.14  2004/10/25 20:45:53  thomson
- * Option support, parsers rewritten. ClntIfaceMgr now handles options.
- *
- * Revision 1.13  2004/09/05 15:27:49  thomson
- * Data receive switched from recvfrom to recvmsg, unicast partially supported.
- *
- * Revision 1.12  2004/09/03 23:20:23  thomson
- * RAPID-COMMIT support fixed. (bugs #50, #51, #52)
- *
- * Revision 1.8  2004/06/28 21:34:18  thomson
- * DUID is now parsed properly and SrvCfgMgr dumps valid xml file.
- *
- * Revision 1.7  2004/06/17 23:53:54  thomson
- * Server Address Assignment rewritten.
- *
- * Revision 1.6  2004/06/06 22:12:29  thomson
- * Preference option has changed scope from class to interface
- *                                                                           
- */
