@@ -3,97 +3,12 @@
  *                                                                           
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>                           
  *          Marek Senderski <msend@o2.pl>                                    
+ * changes: Petr Pisar <petr.pisar(at)atlas(dot)cz>
  *                                                                           
  * released under GNU GPL v2 or later licence                                
  *                                                                           
- * $Id: SrvCfgMgr.cpp,v 1.41 2005/07/21 23:29:32 thomson Exp $
+ * $Id: SrvCfgMgr.cpp,v 1.52 2007-09-07 08:31:19 thomson Exp $
  *
- * $Log: SrvCfgMgr.cpp,v $
- * Revision 1.41  2005/07/21 23:29:32  thomson
- * Logging cleanup.
- *
- * Revision 1.40  2005/07/17 21:09:54  thomson
- * Minor improvements for 0.4.1 release.
- *
- * Revision 1.39  2005/05/02 21:49:23  thomson
- * Messages cleanup, relay interface checking relaxed.
- *
- * Revision 1.38  2005/03/08 00:43:48  thomson
- * 0.4.0-RC2 release.
- *
- * Revision 1.37  2005/03/07 23:36:14  thomson
- * Minor changes.
- *
- * Revision 1.36  2005/03/07 22:45:14  thomson
- * Fixed problem with non-existent interface name (bug #105)
- *
- * Revision 1.35  2005/02/07 20:51:56  thomson
- * Server stateless mode fixed (bug #103)
- *
- * Revision 1.34  2005/02/01 00:57:36  thomson
- * no message
- *
- * Revision 1.33  2005/01/13 22:45:55  thomson
- * Relays implemented.
- *
- * Revision 1.32  2005/01/08 16:52:04  thomson
- * Relay support implemented.
- *
- * Revision 1.31  2005/01/03 21:57:08  thomson
- * Relay support added.
- *
- * Revision 1.30  2004/12/27 20:48:22  thomson
- * Problem with absent link local addresses fixed (bugs #90, #91)
- *
- * Revision 1.29  2004/12/07 22:55:50  thomson
- * Typos corrected.
- *
- * Revision 1.28  2004/12/07 00:43:03  thomson
- * Server no longer support link local addresses (bug #38),
- * Server now supports stateless mode (bug #71)
- *
- * Revision 1.27  2004/12/02 00:51:05  thomson
- * Log files are now always created (bugs #34, #36)
- *
- * Revision 1.26  2004/10/25 20:45:53  thomson
- * Option support, parsers rewritten. ClntIfaceMgr now handles options.
- *
- * Revision 1.25  2004/09/05 15:27:49  thomson
- * Data receive switched from recvfrom to recvmsg, unicast partially supported.
- *
- * Revision 1.24  2004/07/11 14:04:54  thomson
- * Downed/invalid interface specified in cfg now results in server shutdown (bug #23)
- *
- * Revision 1.23  2004/07/05 00:53:03  thomson
- * Various changes.
- *
- * Revision 1.22  2004/07/05 00:12:30  thomson
- * Lots of minor changes.
- *
- * Revision 1.21  2004/07/01 18:12:46  thomson
- * Minor clean up.
- *
- * Revision 1.20  2004/06/29 22:03:36  thomson
- * *MaxLease option partialy implemented/fixed.
- *
- * Revision 1.19  2004/06/28 22:37:59  thomson
- * Minor changes.
- *
- * Revision 1.18  2004/06/20 21:00:45  thomson
- * Various fixes.
- *
- * Revision 1.17  2004/06/17 23:53:54  thomson
- * Server Address Assignment rewritten.
- *
- * Revision 1.16  2004/06/04 16:55:27  thomson
- * *** empty log message ***
- *
- * Revision 1.15  2004/05/24 21:16:37  thomson
- * Various fixes.
- *
- * Revision 1.14  2004/05/23 23:46:02  thomson
- * *** empty log message ***
- *                                                                           
  */
 
 #include <iostream>
@@ -137,8 +52,6 @@ TSrvCfgMgr::TSrvCfgMgr(SmartPtr<TSrvIfaceMgr> ifaceMgr, string cfgFile, string x
 		return;
     }
 
-    Log(Info) << "My duid is " << this->DUID->getPlain() << "." << LogEnd;
-
     this->dump();
 
     IsDone = false;
@@ -203,8 +116,12 @@ void TSrvCfgMgr::dump() {
 }
 
 bool TSrvCfgMgr::setupGlobalOpts(SmartPtr<TSrvParsGlobalOpt> opt) {
-    this->Workdir   = opt->getWorkDir();
-    this->Stateless = opt->getStateless();
+    this->Workdir          = opt->getWorkDir();
+    this->Stateless        = opt->getStateless();
+    this->CacheSize        = opt->getCacheSize();
+    this->DigestLst        = opt->getDigest();
+    this->InterfaceIDOrder = opt->getInterfaceIDOrder();
+    this->InactiveMode     = opt->getInactiveMode(); // should the client accept not ready interfaces?
     return true;
 }
 
@@ -249,14 +166,51 @@ bool TSrvCfgMgr::matchParsedSystemInterfaces(SrvParser *parser) {
 	    return false;
 	}
 
+	// Complete name and ID (one of them usually misses depending on
+	// identifier used in config file.
+	// FIXME: Client's class uses setIface{Name,ID}(). We should unite the
+	// method names.
+	cfgIface->setName(ifaceIface->getName());
+	cfgIface->setID(ifaceIface->getID());
+
+
+	// Check for link scope address presence
         if (!ifaceIface->countLLAddress()) {
+            if (this->inactiveMode()) {
+                Log(Notice) << "Interface " << ifaceIface->getFullName() 
+                            << " is not operational yet (does not have link scope address), skipping it for now." << LogEnd;
+                this->addIface(cfgIface);
+                this->makeInactiveIface(cfgIface->getID(), true); // move it to InactiveLst
+		continue;
+            }
+
 	    Log(Crit) << "Interface " << ifaceIface->getName() << "/" << ifaceIface->getID()
-		      << " is down or doesn't have any link-layer address." << LogEnd;
+		      << " is down or doesn't have any link scope address." << LogEnd;
 	    return false;
         }
 
-	cfgIface->setName(ifaceIface->getName());
-	cfgIface->setID(ifaceIface->getID());
+	// Check if the interface is during bring-up phase (i.e. DAD procedure for link-local addr is not complete yet)
+	char tmp[64];
+	ifaceIface->firstLLAddress();
+	inet_ntop6(ifaceIface->getLLAddress(), tmp); 
+	if (is_addr_tentative(ifaceIface->getName(), ifaceIface->getID(), tmp) == LOWLEVEL_TENTATIVE_YES) {
+	    Log(Notice) << "Interface " << ifaceIface->getFullName() << " has link-scope address " << tmp 
+			<< ", but it is currently tentative." << LogEnd;
+
+	    if (this->inactiveMode()) {
+		Log(Notice) << "Interface " << ifaceIface->getFullName() 
+			    << " is not operational yet (link-scope address is not ready), skipping it for now." << LogEnd;
+		addIface(cfgIface);
+		makeInactiveIface(cfgIface->getID(), true); // move it to InactiveLst
+		continue;
+	    }
+
+	    Log(Crit) << "Interface " << ifaceIface->getFullName()
+		      << " has tentative link-scope address (and inactive-mode is disabled)." << LogEnd;
+	    return false;
+
+	}
+
 	this->addIface(cfgIface);
 	Log(Info) << "Interface " << cfgIface->getName() << "/" << cfgIface->getID() 
 		  << " configuration has been loaded." << LogEnd;
@@ -272,6 +226,54 @@ void TSrvCfgMgr::addIface(SmartPtr<TSrvCfgIface> ptr) {
     SrvCfgIfaceLst.append(ptr);
 }
 
+/**
+ * Moves interface identified by ifindex between SrvCfgIfaceLst and InactiveLst.
+ *
+ * @param ifindex Index of the moving interface
+ * @param inactive true for makeing inactive, false for makeing active
+ */
+void TSrvCfgMgr::makeInactiveIface(int ifindex, bool inactive) {
+    SmartPtr<TSrvCfgIface> x;
+
+    if (inactive)
+    {
+	SrvCfgIfaceLst.first();
+	while (x= SrvCfgIfaceLst.get()) {
+	    if (x->getID() == ifindex) {
+		Log(Info) << "Switching " << x->getFullName() << " to inactive-mode." << LogEnd;
+		SrvCfgIfaceLst.del();
+		InactiveLst.append(x);
+		return;
+	    }
+	}
+	Log(Error) << "Unable to switch interface ifindex=" << ifindex << " to inactive-mode: interface not found." << LogEnd;
+	// something is wrong, VERY wrong
+    }
+
+    else 
+    {
+        InactiveLst.first();
+        while (x= InactiveLst.get()) {
+            if (x->getID() == ifindex) {
+                Log(Info) << "Switching " << x->getFullName() << " to normal mode." << LogEnd;
+                InactiveLst.del();
+                InactiveLst.first();
+                addIface(x);
+                return;
+            }
+        }
+
+        Log(Error) << "Unable to switch interface ifindex=" << ifindex << " from inactive-mode to normal operation: interface not found." << LogEnd;
+    }
+}
+
+/**
+ * Returns number of inactive interfaces
+ */
+int TSrvCfgMgr::inactiveIfacesCnt() {
+    return InactiveLst.count();
+}
+
 void TSrvCfgMgr::firstIface() {
     SrvCfgIfaceLst.first();
 }
@@ -280,10 +282,89 @@ long TSrvCfgMgr::countIface() {
     return SrvCfgIfaceLst.count();
 }
 
+/**
+ *
+ * Tryies to find interface in list of inactive interfaces which becames
+ * ready. In positive case such an interface is moved back into standard list.
+ * Only first matching interface is processed.
+ *
+ * @returns Interface that becomes ready and has been moved out of inactive list.
+ * If no such interface exists, 0 will be returned.
+ */
+SPtr<TSrvCfgIface> TSrvCfgMgr::checkInactiveIfaces() {
+    if (!InactiveLst.count())
+	return 0;
+
+    IfaceMgr->redetectIfaces();
+    SPtr<TSrvCfgIface> x;
+    SPtr<TIfaceIface> iface;
+    InactiveLst.first();
+    while (x = InactiveLst.get()) {
+	iface = IfaceMgr->getIfaceByID(x->getID());
+	if (!iface) {
+	    Log(Error) << "TSrvCfgMgr::checkInactiveIfaces(): " <<
+		"Unable to find interface with ifindex=" << x->getID() << LogEnd;
+	    continue;
+	}
+	iface->firstLLAddress();
+	if (iface->flagUp() && iface->flagRunning() && iface->getLLAddress()) {
+	    // check if its link-local address is not tentative
+	    char tmp[64];
+	    iface->firstLLAddress();
+	    inet_ntop6(iface->getLLAddress(), tmp);
+	    if (is_addr_tentative(iface->getName(), iface->getID(), tmp)==LOWLEVEL_TENTATIVE_YES) {
+		Log(Debug) << "Interface " << iface->getFullName() << " is up and running, but link-scope address " << tmp
+			   << " is currently tentative." << LogEnd;
+		continue;
+	    }
+
+
+	    makeInactiveIface(x->getID(), false); // move it to InactiveLst
+	    return x;
+	}
+    }
+
+    return 0;
+}
+
+
 TSrvCfgMgr::~TSrvCfgMgr() {
     Log(Debug) << "SrvCfgMgr cleanup." << LogEnd;
 }
 
+/**
+ * checks if this Addr is already configured in that TA belonging to that client 
+ * (used in CONFIRM message)
+ */
+bool TSrvCfgMgr::isTAAddrSupported(int iface, SmartPtr<TIPv6Addr> addr) {
+    SmartPtr<TSrvCfgIface> ptrIface = this->getIfaceByID(iface);
+    if (!ptrIface)
+	return false;
+    ptrIface->firstTA();
+    SmartPtr<TSrvCfgTA> ta = ptrIface->getTA();
+    if (!ta)
+	return false;
+    return ta->addrInPool(addr);
+}
+
+/**
+ * checks if this Addr is already configured in that IA belonging to that client 
+ * (used in CONFIRM message)
+ */
+bool TSrvCfgMgr::isIAAddrSupported(int iface, SmartPtr<TIPv6Addr> addr) {
+    SmartPtr<TSrvCfgIface> ptrIface = this->getIfaceByID(iface);
+    if (!ptrIface)
+	return false;
+    ptrIface->firstAddrClass();
+    SmartPtr<TSrvCfgAddrClass> addrClass;
+    while (addrClass = ptrIface->getAddrClass()) {
+	if (addrClass->addrInPool(addr))
+	    return true;
+    }
+    return false;
+}
+
+
 /*
  * returns how many addresses can be assigned to this client?
  * factors used: 
@@ -353,6 +434,36 @@ SmartPtr<TSrvCfgAddrClass> TSrvCfgMgr::getClassByAddr(int iface, SmartPtr<TIPv6A
     return 0; // NULL
 }
 
+/** 
+ * get a class, which prefix belongs to
+ * 
+ * @param iface 
+ * @param addr 
+ * 
+ * @return class (or 0 if no class is found)
+ */
+SmartPtr<TSrvCfgPD> TSrvCfgMgr::getClassByPrefix(int iface, SmartPtr<TIPv6Addr> addr)
+{
+    this->firstIface();
+    SmartPtr<TSrvCfgIface> ptrIface;
+    ptrIface = this->getIfaceByID(iface);
+
+    if (!ptrIface) {
+	Log(Error) << "Trying to find class on unknown (" << iface <<") interface." << LogEnd;
+	return 0;
+    }
+
+    SmartPtr<TSrvCfgPD> ptrClass;
+    ptrIface->firstPD();
+    while (ptrClass = ptrIface->getPD()) {
+	if (ptrClass->prefixInPool(addr))
+	    return ptrClass;
+    }
+
+    return 0;
+}
+
+
 //on basis of duid/address/iface assign addresss to client
 SmartPtr<TIPv6Addr> TSrvCfgMgr::getRandomAddr(SmartPtr<TDUID> clntDuid, 
 					      SmartPtr<TIPv6Addr> clntAddr,
@@ -388,13 +499,21 @@ bool TSrvCfgMgr::isClntSupported(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr> clntA
     if (this->stateless())
 	return true;
 
+    int classCnt = 0;
     if (ptrIface)
     {
         SmartPtr<TSrvCfgAddrClass> ptrClass;
         ptrIface->firstAddrClass();
-        while(ptrClass=ptrIface->getAddrClass())
+        while(ptrClass=ptrIface->getAddrClass()) {
             if (ptrClass->clntSupported(duid,clntAddr))
                 return true;
+	    classCnt++;
+	}
+    }
+    if (!classCnt) {
+	Log(Warning) << "There are no address class defined on the " << ptrIface->getFullName() 
+		     << ". Maybe you are trying to configure clients on cascade relay interface? "
+		     << "If that is so, please define separate class on this interface, too." << LogEnd;
     }
     return false;
 }
@@ -406,7 +525,7 @@ bool TSrvCfgMgr::isDone() {
 bool TSrvCfgMgr::validateConfig() {
     SmartPtr<TSrvCfgIface> ptrIface;
 
-    if (!this->countIface()) {
+    if (0 == (this->countIface() + this->inactiveIfacesCnt())) {
 	Log(Crit) << "Config problem: No interface defined." << LogEnd;
 	return false;
     }
@@ -437,10 +556,17 @@ bool TSrvCfgMgr::validateIface(SmartPtr<TSrvCfgIface> ptrIface)
 		      << ": No class definitions present, but stateless mode not set." << LogEnd;
 	    return false;
 	} else {
-	    Log(Warning) << "Interface " << ptrIface->getFullName() << " has no addrs defined, working as cascade relay interface." << LogEnd;
+	    Log(Warning) << "Interface " << ptrIface->getFullName() << " has no addrs defined, working as cascade relay interface." 
+			 << LogEnd;
 	}
     }
 
+    if (ptrIface->supportFQDN() && !ptrIface->supportDNSServer()) {
+	Log(Crit) << "FQDN defined on the " << ptrIface->getFullName() << ", but no DNS servers defined." 
+		  << " Please disable FQDN support or add DNS servers." << LogEnd;
+	return false;
+    }
+
     SmartPtr<TSrvCfgAddrClass> ptrClass;
     ptrIface->firstAddrClass();
     while(ptrClass=ptrIface->getAddrClass()) {
@@ -515,10 +641,38 @@ void TSrvCfgMgr::addClntAddr(int iface, SmartPtr<TIPv6Addr> addr) {
     ptrIface->addClntAddr(addr);
 }
 
+void TSrvCfgMgr::addTAAddr(int iface) {
+    SmartPtr<TSrvCfgIface> ptrIface;
+    ptrIface = this->getIfaceByID(iface);
+    if (!ptrIface) {
+	Log(Error) << "Unable to increase address usage: interface (id=" 
+		   << iface << ") not found." << LogEnd;
+	return;
+    }
+    ptrIface->addTAAddr();
+}
+
+void TSrvCfgMgr::delTAAddr(int iface) {
+    SmartPtr<TSrvCfgIface> ptrIface;
+    ptrIface = this->getIfaceByID(iface);
+    if (!ptrIface) {
+	Log(Error) << "Unable to decrease address usage: interface (id=" 
+		   << iface << ") not found." << LogEnd;
+	return;
+    }
+    ptrIface->delTAAddr();
+}
+
 bool TSrvCfgMgr::stateless() {
     return this->Stateless;
 }
 
+bool TSrvCfgMgr::inactiveMode()
+{
+    return InactiveMode;
+}
+
+
 bool TSrvCfgMgr::setupRelay(SmartPtr<TSrvCfgIface> cfgIface) {
     SmartPtr<TIfaceIface> iface;
     string name = cfgIface->getRelayName();
@@ -539,6 +693,53 @@ bool TSrvCfgMgr::setupRelay(SmartPtr<TSrvCfgIface> cfgIface) {
     return true;
 }
 
+/** 
+ * returns size (in bytes of the configured cache size 
+ * 
+ * 
+ * @return 
+ */
+int TSrvCfgMgr::getCacheSize() {
+    return this->CacheSize;
+}
+
+ESrvIfaceIdOrder TSrvCfgMgr::getInterfaceIDOrder()
+{
+    return InterfaceIDOrder;
+}
+
+/** 
+ * decreases prefix usage count (i.e. decreases prefix-pool usage by one)
+ * actual prefix is deleted in AddrMgr
+ * 
+ * @param iface 
+ * @param prefix 
+ * 
+ * @return 
+ */
+bool TSrvCfgMgr::decrPrefixCount(int ifindex, SPtr<TIPv6Addr> prefix)
+{
+    SmartPtr<TSrvCfgIface> iface;
+    iface = getIfaceByID(ifindex);
+    if (!iface) {
+	Log(Error) << "Unable to find interface with ifindex=" << ifindex << ", prefix deletion aborted." << LogEnd;
+	return false;
+    }
+    return iface->delClntPrefix(prefix);
+}
+
+bool TSrvCfgMgr::incrPrefixCount(int ifindex, SPtr<TIPv6Addr> prefix)
+{
+    SmartPtr<TSrvCfgIface> iface;
+    iface = getIfaceByID(ifindex);
+    if (!iface) {
+	Log(Error) << "Unable to find interface with ifindex=" << ifindex << ", prefix increase count aborted." << LogEnd;
+	return false;
+    }
+    return iface->addClntPrefix(prefix);
+}
+
+
 // --------------------------------------------------------------------
 // --- operators ------------------------------------------------------
 // --------------------------------------------------------------------
@@ -548,11 +749,48 @@ ostream & operator<<(ostream &out, TSrvCfgMgr &x) {
     out << "  <workDir>" << x.getWorkDir()  << "</workDir>" << endl;
     out << "  <LogName>" << x.getLogName()  << "</LogName>" << endl;
     out << "  <LogLevel>" << x.getLogLevel() << "</LogLevel>" << endl;
+    out << "  <InactiveMode>" << (x.InactiveMode?1:0) << "</InactiveMode>" << endl;
     if (x.DUID)
 	out << "  " << *x.DUID;
     else
 	out << "  <!-- DUID not set -->" << std::endl;
 
+    out << "  <InterfaceIDOrder>";
+    switch (x.InterfaceIDOrder) {
+    case SRV_IFACE_ID_ORDER_BEFORE:
+	out << "before";
+	break;
+    case SRV_IFACE_ID_ORDER_AFTER:
+	out << "after";
+	break;
+    case SRV_IFACE_ID_ORDER_NONE:
+	out << "none";
+	break;
+    default:
+	out << "unknown(" << x.InterfaceIDOrder << ")";
+	break;
+    }
+    out << "</InterfaceIDOrder>" << endl;
+
+    out << "  <auth count=\"" << x.DigestLst.count() << "\">";
+    x.DigestLst.first();
+    SPtr<DigestTypes> dig;
+    while (dig=x.DigestLst.get()) {
+	switch (*dig) {
+	case DIGEST_NONE:
+	    out << "digest-none ";
+	    break;
+	case DIGEST_HMAC_SHA1:
+	    out << "digest-hmac-sha1";
+	    break;
+	default:
+	    break;
+	}
+	out << "X";
+    }
+
+    out << "</auth>" << endl;
+
     if (x.stateless())
 	out << "  <stateless/>" << std::endl;
     else
