@@ -6,30 +6,8 @@
  *                                                                           
  * released under GNU GPL v2 or later licence                                
  *                                                                           
- * $Id: SrvCfgMgr.h,v 1.10 2006/01/12 00:23:35 thomson Exp $
+ * $Id: SrvCfgMgr.h,v 1.18 2007-09-07 08:31:19 thomson Exp $
  *
- * $Log: SrvCfgMgr.h,v $
- * Revision 1.10  2006/01/12 00:23:35  thomson
- * Cleanup changes. Now -pedantic option works.
- *
- * Revision 1.9  2005/02/01 00:57:36  thomson
- * no message
- *
- * Revision 1.8  2005/01/03 21:57:08  thomson
- * Relay support added.
- *
- * Revision 1.7  2004/12/07 00:43:03  thomson
- * Server no longer support link local addresses (bug #38),
- * Server now supports stateless mode (bug #71)
- *
- * Revision 1.6  2004/09/05 15:27:49  thomson
- * Data receive switched from recvfrom to recvmsg, unicast partially supported.
- *
- * Revision 1.5  2004/07/05 00:12:30  thomson
- * Lots of minor changes.
- *
- * Revision 1.4  2004/06/17 23:53:54  thomson
- * Server Address Assignment rewritten.
  */
 
 class TSrvCfgMgr;
@@ -60,6 +38,9 @@ public:
     SmartPtr<TSrvCfgIface> getIfaceByID(int iface);
     long countIface();
     void addIface(SmartPtr<TSrvCfgIface> iface);
+    void makeInactiveIface(int ifindex, bool inactive);
+    int inactiveIfacesCnt();
+    SPtr<TSrvCfgIface> checkInactiveIfaces();
 
     void dump();
 
@@ -68,13 +49,24 @@ public:
     //Address assignment connected methods
     long countAvailAddrs(SmartPtr<TDUID> clntDuid, SmartPtr<TIPv6Addr> clntAddr, int iface);
     SmartPtr<TSrvCfgAddrClass> getClassByAddr(int iface, SmartPtr<TIPv6Addr> addr);
+    SmartPtr<TSrvCfgPD> getClassByPrefix(int iface, SmartPtr<TIPv6Addr> prefix);
     SmartPtr<TIPv6Addr> getRandomAddr(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr> clntAddr, int iface);
     bool isClntSupported(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr> clntAddr, int iface);
 
+    // prefix-related
+    bool incrPrefixCount(int iface, SPtr<TIPv6Addr> prefix);
+    bool decrPrefixCount(int iface, SPtr<TIPv6Addr> prefix);
+
     // class' usage management
     void delClntAddr(int iface, SmartPtr<TIPv6Addr> addr);
     void addClntAddr(int iface, SmartPtr<TIPv6Addr> addr);
 
+    bool isIAAddrSupported(int iface, SmartPtr<TIPv6Addr> addr);
+    bool isTAAddrSupported(int iface, SmartPtr<TIPv6Addr> addr);
+
+    void addTAAddr(int iface);
+    void delTAAddr(int iface);
+
     bool isDone();
     virtual ~TSrvCfgMgr();
     bool setupGlobalOpts(SmartPtr<TSrvParsGlobalOpt> opt);
@@ -82,9 +74,12 @@ public:
     // configuration parameters
     string getWorkdir();
     bool stateless();
-    static int NextRelayID;
+    bool inactiveMode();
+    ESrvIfaceIdOrder getInterfaceIDOrder();
+    int getCacheSize();
 
 private:    
+    static int NextRelayID;
     string XmlFile;
 
     bool IsDone;
@@ -92,11 +87,16 @@ private:
     bool validateIface(SmartPtr<TSrvCfgIface> ptrIface);
     bool validateClass(SmartPtr<TSrvCfgIface> ptrIface, SmartPtr<TSrvCfgAddrClass> ptrClass);
     List(TSrvCfgIface) SrvCfgIfaceLst;
+    List(TSrvCfgIface) InactiveLst;
     SmartPtr<TSrvIfaceMgr> IfaceMgr;
     bool matchParsedSystemInterfaces(SrvParser *parser);
 
     // global options
     bool Stateless;
+    bool InactiveMode;
+    int  CacheSize;
+    List(DigestTypes) DigestLst;
+    ESrvIfaceIdOrder InterfaceIDOrder;
 };
 
 #endif /* SRVCONFMGR_H */
