@@ -24,19 +24,35 @@
 #include "SrvParsGlobalOpt.h"
 #include "SrvParsClassOpt.h"
 #include "SrvParsIfaceOpt.h"
+#include "SrvCfgTA.h"
+#include "SrvCfgPD.h"
 #include "SrvCfgAddrClass.h"
 #include "SrvCfgIface.h"
+#include "SrvCfgOptions.h"
 #include "DUID.h"
 #include "Logger.h"
-
+#include "FQDN.h"
+#include "SrvOptVendorSpec.h"
+#include "SrvOptAddrParams.h"
+#include "Portable.h"
 #define YY_USE_CLASS
 #define YY_SrvParser_MEMBERS  FlexLexer * lex;                                                     \
 List(TSrvParsGlobalOpt) ParserOptStack;    /* list of parsed interfaces/IAs/addrs */ \
 List(TSrvCfgIface) SrvCfgIfaceLst;         /* list of SrvCfg interfaces */           \
 List(TSrvCfgAddrClass) SrvCfgAddrClassLst; /* list of SrvCfg address classes */      \
+List(TSrvCfgTA) SrvCfgTALst;               /* list of SrvCfg TA objects */           \
+List(TSrvCfgPD) SrvCfgPDLst;		   /* list of SrvCfg PD objects */           \
 List(TIPv6Addr) PresentAddrLst;            /* address list (used for DNS,NTP,etc.)*/ \
 List(string) PresentStringLst;             /* string list */                         \
+List(TFQDN) PresentFQDNLst;                                                          \
+SmartPtr<TDUID> duidNew;                                                             \
+SmartPtr<TIPv6Addr> addr;                                                            \
 List(TStationRange) PresentRangeLst;                                                 \
+List(TStationRange) PDLst;                                                           \
+int VendorEnterpriseNumber;                                                          \
+List(TSrvOptVendorSpec) VendorSpec;			                             \
+List(TSrvCfgOptions) ClientLst;                                                      \
+int PDPrefix;                                                                        \
 /*method check whether interface with id=ifaceNr has been already declared */        \
 bool CheckIsIface(int ifaceNr);                                                      \
 /*method check whether interface with id=ifaceName has been already declared*/       \
@@ -45,8 +61,12 @@ void StartIfaceDeclaration();
 bool EndIfaceDeclaration();                                                          \
 void StartClassDeclaration();                                                        \
 bool EndClassDeclaration();                                                          \
-SmartPtr<TIPv6Addr> getRangeMin(char * addrPacked, int prefix);                     \
-SmartPtr<TIPv6Addr> getRangeMax(char * addrPacked, int prefix);                       \
+SmartPtr<TIPv6Addr> getRangeMin(char * addrPacked, int prefix);                      \
+SmartPtr<TIPv6Addr> getRangeMax(char * addrPacked, int prefix);                      \
+void StartTAClassDeclaration();                                                      \
+bool EndTAClassDeclaration();                                                        \
+void StartPDDeclaration();                                                           \
+bool EndPDDeclaration();                                                             \
 virtual ~SrvParser();
 #define YY_SrvParser_CONSTRUCTOR_PARAM  yyFlexLexer * lex
 #define YY_SrvParser_CONSTRUCTOR_CODE                                                           \
@@ -54,7 +74,7 @@ virtual ~SrvParser();
     ParserOptStack.getLast()->setUnicast(false);                                  \
     this->lex = lex;
 
-#line 53 "SrvParser.y"
+#line 73 "SrvParser.y"
 typedef union    
 {
     unsigned int ival;
@@ -129,6 +149,7 @@ typedef union
 #ifndef YY_SrvParser_DEBUG
 
  #line 71 "../bison++/bison.h"
+#define YY_SrvParser_DEBUG 1
 
 #line 71 "../bison++/bison.h"
 /* YY_SrvParser_DEBUG */
@@ -238,44 +259,58 @@ typedef
 #define	IFACE_	258
 #define	RELAY_	259
 #define	IFACE_ID_	260
-#define	CLASS_	261
-#define	LOGNAME_	262
-#define	LOGLEVEL_	263
-#define	LOGMODE_	264
-#define	WORKDIR_	265
-#define	OPTION_	266
-#define	DNS_SERVER_	267
-#define	DOMAIN_	268
-#define	NTP_SERVER_	269
-#define	TIME_ZONE_	270
-#define	SIP_SERVER_	271
-#define	SIP_DOMAIN_	272
-#define	NIS_SERVER_	273
-#define	NIS_DOMAIN_	274
-#define	NISP_SERVER_	275
-#define	NISP_DOMAIN_	276
-#define	FQDN_	277
-#define	LIFETIME_	278
-#define	ACCEPT_ONLY_	279
-#define	REJECT_CLIENTS_	280
-#define	POOL_	281
-#define	SHARE_	282
-#define	T1_	283
-#define	T2_	284
-#define	PREF_TIME_	285
-#define	VALID_TIME_	286
-#define	UNICAST_	287
-#define	PREFERENCE_	288
-#define	RAPID_COMMIT_	289
-#define	IFACE_MAX_LEASE_	290
-#define	CLASS_MAX_LEASE_	291
-#define	CLNT_MAX_LEASE_	292
-#define	STATELESS_	293
-#define	STRING_	294
-#define	HEXNUMBER_	295
-#define	INTNUMBER_	296
-#define	IPV6ADDR_	297
-#define	DUID_	298
+#define	IFACE_ID_ORDER_	261
+#define	CLASS_	262
+#define	TACLASS_	263
+#define	LOGNAME_	264
+#define	LOGLEVEL_	265
+#define	LOGMODE_	266
+#define	WORKDIR_	267
+#define	OPTION_	268
+#define	DNS_SERVER_	269
+#define	DOMAIN_	270
+#define	NTP_SERVER_	271
+#define	TIME_ZONE_	272
+#define	SIP_SERVER_	273
+#define	SIP_DOMAIN_	274
+#define	NIS_SERVER_	275
+#define	NIS_DOMAIN_	276
+#define	NISP_SERVER_	277
+#define	NISP_DOMAIN_	278
+#define	FQDN_	279
+#define	LIFETIME_	280
+#define	ACCEPT_ONLY_	281
+#define	REJECT_CLIENTS_	282
+#define	POOL_	283
+#define	SHARE_	284
+#define	T1_	285
+#define	T2_	286
+#define	PREF_TIME_	287
+#define	VALID_TIME_	288
+#define	UNICAST_	289
+#define	PREFERENCE_	290
+#define	RAPID_COMMIT_	291
+#define	IFACE_MAX_LEASE_	292
+#define	CLASS_MAX_LEASE_	293
+#define	CLNT_MAX_LEASE_	294
+#define	STATELESS_	295
+#define	CACHE_SIZE_	296
+#define	PDCLASS_	297
+#define	PD_LENGTH_	298
+#define	PD_POOL_	299
+#define	VENDOR_SPEC_	300
+#define	AUTH_	301
+#define	DIGEST_NONE_	302
+#define	DIGEST_HMAC_SHA1_	303
+#define	CLIENT_	304
+#define	INACTIVE_MODE_	305
+#define	EXPERIMENTAL_	306
+#define	ADDR_PARAMS_	307
+#define	STRING_	308
+#define	HEXNUMBER_	309
+#define	INTNUMBER_	310
+#define	IPV6ADDR_	311
+#define	DUID_	312
 
 
 #line 169 "../bison++/bison.h"
@@ -327,7 +362,9 @@ public:
 static const int IFACE_;
 static const int RELAY_;
 static const int IFACE_ID_;
+static const int IFACE_ID_ORDER_;
 static const int CLASS_;
+static const int TACLASS_;
 static const int LOGNAME_;
 static const int LOGLEVEL_;
 static const int LOGMODE_;
@@ -360,6 +397,18 @@ static const int IFACE_MAX_LEASE_;
 static const int CLASS_MAX_LEASE_;
 static const int CLNT_MAX_LEASE_;
 static const int STATELESS_;
+static const int CACHE_SIZE_;
+static const int PDCLASS_;
+static const int PD_LENGTH_;
+static const int PD_POOL_;
+static const int VENDOR_SPEC_;
+static const int AUTH_;
+static const int DIGEST_NONE_;
+static const int DIGEST_HMAC_SHA1_;
+static const int CLIENT_;
+static const int INACTIVE_MODE_;
+static const int EXPERIMENTAL_;
+static const int ADDR_PARAMS_;
 static const int STRING_;
 static const int HEXNUMBER_;
 static const int INTNUMBER_;
@@ -376,44 +425,58 @@ static const int DUID_;
 	,IFACE_=258
 	,RELAY_=259
 	,IFACE_ID_=260
-	,CLASS_=261
-	,LOGNAME_=262
-	,LOGLEVEL_=263
-	,LOGMODE_=264
-	,WORKDIR_=265
-	,OPTION_=266
-	,DNS_SERVER_=267
-	,DOMAIN_=268
-	,NTP_SERVER_=269
-	,TIME_ZONE_=270
-	,SIP_SERVER_=271
-	,SIP_DOMAIN_=272
-	,NIS_SERVER_=273
-	,NIS_DOMAIN_=274
-	,NISP_SERVER_=275
-	,NISP_DOMAIN_=276
-	,FQDN_=277
-	,LIFETIME_=278
-	,ACCEPT_ONLY_=279
-	,REJECT_CLIENTS_=280
-	,POOL_=281
-	,SHARE_=282
-	,T1_=283
-	,T2_=284
-	,PREF_TIME_=285
-	,VALID_TIME_=286
-	,UNICAST_=287
-	,PREFERENCE_=288
-	,RAPID_COMMIT_=289
-	,IFACE_MAX_LEASE_=290
-	,CLASS_MAX_LEASE_=291
-	,CLNT_MAX_LEASE_=292
-	,STATELESS_=293
-	,STRING_=294
-	,HEXNUMBER_=295
-	,INTNUMBER_=296
-	,IPV6ADDR_=297
-	,DUID_=298
+	,IFACE_ID_ORDER_=261
+	,CLASS_=262
+	,TACLASS_=263
+	,LOGNAME_=264
+	,LOGLEVEL_=265
+	,LOGMODE_=266
+	,WORKDIR_=267
+	,OPTION_=268
+	,DNS_SERVER_=269
+	,DOMAIN_=270
+	,NTP_SERVER_=271
+	,TIME_ZONE_=272
+	,SIP_SERVER_=273
+	,SIP_DOMAIN_=274
+	,NIS_SERVER_=275
+	,NIS_DOMAIN_=276
+	,NISP_SERVER_=277
+	,NISP_DOMAIN_=278
+	,FQDN_=279
+	,LIFETIME_=280
+	,ACCEPT_ONLY_=281
+	,REJECT_CLIENTS_=282
+	,POOL_=283
+	,SHARE_=284
+	,T1_=285
+	,T2_=286
+	,PREF_TIME_=287
+	,VALID_TIME_=288
+	,UNICAST_=289
+	,PREFERENCE_=290
+	,RAPID_COMMIT_=291
+	,IFACE_MAX_LEASE_=292
+	,CLASS_MAX_LEASE_=293
+	,CLNT_MAX_LEASE_=294
+	,STATELESS_=295
+	,CACHE_SIZE_=296
+	,PDCLASS_=297
+	,PD_LENGTH_=298
+	,PD_POOL_=299
+	,VENDOR_SPEC_=300
+	,AUTH_=301
+	,DIGEST_NONE_=302
+	,DIGEST_HMAC_SHA1_=303
+	,CLIENT_=304
+	,INACTIVE_MODE_=305
+	,EXPERIMENTAL_=306
+	,ADDR_PARAMS_=307
+	,STRING_=308
+	,HEXNUMBER_=309
+	,INTNUMBER_=310
+	,IPV6ADDR_=311
+	,DUID_=312
 
 
 #line 215 "../bison++/bison.h"
