@@ -5,42 +5,6 @@
  *          Marek Senderski <msend@o2.pl>
  *
  * released under GNU GPL v2 or later licence
- *
- * $Id: SrvMsgReply.cpp,v 1.18 2005/09/13 23:00:06 thomson Exp $
- *
- * $Log: SrvMsgReply.cpp,v $
- * Revision 1.18  2005/09/13 23:00:06  thomson
- * Compilation warning removed.
- *
- * Revision 1.17  2005/03/15 23:02:31  thomson
- * 0.4.0 release.
- *
- * Revision 1.16  2005/02/07 20:51:56  thomson
- * Server stateless mode fixed (bug #103)
- *
- * Revision 1.15  2005/01/08 16:52:04  thomson
- * Relay support implemented.
- *
- * Revision 1.14  2004/12/04 23:43:26  thomson
- * Server no longer crashes after receiving the same INF-REQUEST (bug #84)
- *
- * Revision 1.13  2004/12/02 00:51:06  thomson
- * Log files are now always created (bugs #34, #36)
- *
- * Revision 1.12  2004/10/25 20:45:54  thomson
- * Option support, parsers rewritten. ClntIfaceMgr now handles options.
- *
- * Revision 1.11  2004/09/07 22:02:33  thomson
- * pref/valid/IAID is not unsigned, RAPID-COMMIT now works ok.
- *
- * Revision 1.7  2004/06/20 19:29:23  thomson
- * New address assignment finally works.
- *
- * Revision 1.6  2004/06/20 17:25:07  thomson
- * getName() method implemented, clean up
- *
- * Revision 1.5  2004/06/17 23:53:54  thomson
- * Server Address Assignment rewritten.
  *                                                                           
  */
 
@@ -52,20 +16,30 @@
 #include "SrvOptStatusCode.h"
 #include "SrvOptIAAddress.h"
 #include "SrvOptIA_NA.h"
+#include "SrvOptIA_PD.h"
+#include "SrvOptTA.h"
 #include "SrvOptRapidCommit.h"
 #include "SrvOptServerIdentifier.h"
 #include "SrvOptTimeZone.h"
 #include "SrvOptDomainName.h"
 #include "SrvOptDNSServers.h"
 #include "SrvOptNTPServers.h"
+#include "SrvOptFQDN.h"
 #include "AddrClient.h"
 #include "AddrIA.h"
 #include "AddrAddr.h"
 #include "Logger.h"
-#include "OptStatusCode.h"
+#include "DNSUpdate.h"
 
-
-// used as CONFIRM reply
+/** 
+ * this constructor is used to create REPLY message as a response for CONFIRM message
+ * 
+ * @param ifaceMgr 
+ * @param transMgr 
+ * @param CfgMgr 
+ * @param AddrMgr 
+ * @param confirm 
+ */
 TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr, 
 			   SmartPtr<TSrvTransMgr> transMgr, 
 			   SmartPtr<TSrvCfgMgr> CfgMgr, 
@@ -84,56 +58,81 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
 	return;
     }
 
-    // copy client's DUID
-    SmartPtr<TOpt> ptrOpt;
-    SmartPtr<TSrvOptClientIdentifier> ptrClntDUID;
-    ptrOpt = confirm->getOption(OPTION_CLIENTID);
-    ptrClntDUID = (Ptr*) ptrOpt;
+    setOptionsReqOptClntDUID((Ptr*)confirm);
+    if (!duidOpt) {
+	Log(Warning) << "CONFIRM message without client-id option received, message dropped." << LogEnd;
+	IsDone = true;
+	return;
+    }
 
-    // include our DUID
+    // copy client's ID
+    SmartPtr<TOpt> ptrOpt;
     ptrOpt = confirm->getOption(OPTION_CLIENTID);
     Options.append(ptrOpt);
 
     confirm->firstOption();
     bool OnLink = true;
-    while (ptrOpt=confirm->getOption() && (OnLink) ) {
+    int checkCnt = 0;
+
+    while ( (ptrOpt = confirm->getOption()) && OnLink ) {
 	switch (ptrOpt->getOptType()) {
-	case OPTION_IA: 
-	{
-	    SmartPtr<TSrvOptIA_NA> ptrIA = (Ptr*) ptrOpt;
-	    SmartPtr<TOpt> ptrSubOpt;
-	    ptrIA->firstOption();
-	    while ( ptrSubOpt = ptrIA->getOption() && (OnLink) ) {
-		if (ptrSubOpt->getOptType() == OPTION_IAADDR) {
-		    SmartPtr< TSrvOptIAAddress > ptrAddr = (Ptr*) ptrSubOpt;
-		    // addr found. Now look for class it belongs to
-		    SmartPtr<TSrvCfgAddrClass> ptrClass;
-		    bool Found = false;
-		    // check each class on this interface
-		    ptrIface->firstAddrClass();
-		    while ( ptrClass = ptrIface->getAddrClass() && (OnLink) ) {
-			if (ptrClass->addrInPool(ptrAddr->getAddr() ) )
-			    Found = true;
-			
-		    }
-		    if (!Found)
-			OnLink = false;
+	case OPTION_IA_NA: {
+	    SmartPtr<TSrvOptIA_NA> ia = (Ptr*) ptrOpt;
+	    SmartPtr<TOpt> subOpt;
+	    ia->firstOption();
+	    while ( (subOpt = ia->getOption()) && (OnLink) ) {
+		if (subOpt->getOptType() != OPTION_IAADDR)
+		    continue;
+		SmartPtr<TSrvOptIAAddress> optAddr = (Ptr*) subOpt;
+		Log(Debug) << "CONFIRM message: checking if " << optAddr->getAddr()->getPlain() << " is supported:";
+		if (!CfgMgr->isIAAddrSupported(this->Iface, optAddr->getAddr())) {
+		    Log(Cont) << "no." << LogEnd;
+		    OnLink = false;
+		} else {
+		    // TODO check if it is bound or not. If it is, then check if it is bound to this client
+		    Log(Cont) << "yes." << LogEnd;
 		}
+		checkCnt++;
 	    }
 	    break;
-	}   
+	}
+	case OPTION_IA_TA: {
+	    SmartPtr<TSrvOptTA> ta = (Ptr*) ptrOpt;
+	    SmartPtr<TOpt> subOpt;
+	    ta->firstOption();
+	    while (subOpt = ta->getOption() && (OnLink)) {
+		if (subOpt->getOptType() != OPTION_IAADDR)
+		    continue;
+		SmartPtr<TSrvOptIAAddress> optAddr = (Ptr*) subOpt;
+		if (!CfgMgr->isTAAddrSupported(this->Iface, optAddr->getAddr())) {
+		    OnLink = false;
+		}
+		checkCnt++;
+	    }
+	    break;
+	}
 	default:
 	    // other options - ignore them
 	    continue;
 	}
     }
+    if (!checkCnt) {
+	// no check
+	SmartPtr <TSrvOptStatusCode> ptrCode = 
+	    new TSrvOptStatusCode(STATUSCODE_NOTONLINK,
+				  "No addresses checked. Did you send any?",
+				  this);
+	this->Options.append( (Ptr*) ptrCode );
+    } else
     if (!OnLink) {
+	// not-on-link
 	SmartPtr <TSrvOptStatusCode> ptrCode = 
 	    new TSrvOptStatusCode(STATUSCODE_NOTONLINK,
 				  "Sorry, those addresses are not valid for this link.",
 				  this);
 	this->Options.append( (Ptr*) ptrCode );
     } else {
+	// success
 	SmartPtr <TSrvOptStatusCode> ptrCode = 
 	    new TSrvOptStatusCode(STATUSCODE_SUCCESS,
 				  "Your addresses are valid! Yahoo!",
@@ -141,14 +140,27 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
 	this->Options.append( (Ptr*) ptrCode);
     }
 
+    // include our ServerID
+    SmartPtr<TSrvOptServerIdentifier> srvDUID=new TSrvOptServerIdentifier(CfgMgr->getDUID(),this);
+    this->Options.append((Ptr*)srvDUID);
+
+    appendRequestedOptions(duidOpt->getDUID(), confirm->getAddr(), confirm->getIface(), reqOpts);
+
     pkt = new char[this->getSize()];
     IsDone = false;
     this->MRT = 31;
     this->send();
 }
 	
-// used as DECLINE reply
-TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr, 
+/** 
+ * this constructor is used to create REPLY message as a response for DECLINE message
+ * 
+ * @param ifaceMgr 
+ * @param transMgr 
+ * @param CfgMgr 
+ * @param AddrMgr 
+ * @param decline 
+ */TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr, 
 			   SmartPtr<TSrvTransMgr> transMgr, 
 			   SmartPtr<TSrvCfgMgr> CfgMgr, 
 			   SmartPtr<TSrvAddrMgr> AddrMgr,
@@ -159,6 +171,8 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
     this->copyRelayInfo((Ptr*)decline);
 
     SmartPtr<TOpt> ptrOpt;
+
+    // FIXME: Implement DECLINE support for client-id and option request as in other messages
     // include our DUID
     ptrOpt = decline->getOption(OPTION_SERVERID);
     Options.append(ptrOpt);
@@ -166,12 +180,17 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
     // copy client's DUID
     SmartPtr<TSrvOptClientIdentifier> ptrClntDUID;
     ptrOpt = decline->getOption(OPTION_CLIENTID);
+    if (!ptrOpt) {
+	Log(Warning) << "Received DECLINE message without client-id option. Message ignored." << LogEnd;
+	IsDone = true;
+	return;
+    }
     ptrClntDUID = (Ptr*) ptrOpt;
     Options.append(ptrOpt);
     duidOpt=(Ptr*)ptrOpt;
     SmartPtr<TAddrClient> ptrClient = AddrMgr->getClient(duidOpt->getDUID());
     if (!ptrClient) {
-	Log(Warning) << "Received DECLINE from unknown client, DUID=" << *duidOpt->getDUID() << LogEnd;
+	Log(Warning) << "Received DECLINE from unknown client, DUID=" << *duidOpt->getDUID() << ". Ignored." << LogEnd;
 	IsDone = true;
 	return;
     }
@@ -188,7 +207,7 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
     {
         switch (ptrOpt->getOptType()) 
 	{
-        case OPTION_IA: 
+        case OPTION_IA_NA: 
 	{
 	    SmartPtr<TSrvOptIA_NA> ptrIA_NA = (Ptr*) ptrOpt;
 	    SmartPtr<TAddrIA> ptrIA = ptrClient->getIA(ptrIA_NA->getIAID());
@@ -241,6 +260,9 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
 		new TSrvOptStatusCode(STATUSCODE_SUCCESS,tmp+" addrs declined.",this);
 	    replyIA_NA->addOption( (Ptr*) optStatusCode );
 	};
+	case OPTION_IA_TA:
+	    Log(Info) << "TA address declined. Oh well. Since it's temporary, let's ignore it entirely." << LogEnd;
+	    break;
 	default:
 	    break;
 	}
@@ -252,8 +274,15 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
     this->send();
 }
 
-// used as REBIND reply
-TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr, 
+/** 
+ * this constructor is used to create REPLY message as a response for REBIND message
+ * 
+ * @param ifaceMgr 
+ * @param transMgr 
+ * @param CfgMgr 
+ * @param AddrMgr 
+ * @param rebind 
+ */TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr, 
 			   SmartPtr<TSrvTransMgr> transMgr, 
 			   SmartPtr<TSrvCfgMgr> CfgMgr, 
 			   SmartPtr<TSrvAddrMgr> AddrMgr,
@@ -267,29 +296,42 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
     SmartPtr<TOpt> ptrOpt;
 
     setOptionsReqOptClntDUID((Ptr*)rebind);
+    if (!duidOpt) {
+      Log(Warning) << "REBIND message without client-id option received, message dropped." << LogEnd;
+      IsDone = true;
+      return;
+    }
     
     rebind->firstOption();
     while (ptrOpt = rebind->getOption() ) 
     {
         switch (ptrOpt->getOptType()) 
         {
-        case OPTION_IA: 
-            {
-               SmartPtr<TSrvOptIA_NA> optIA_NA;
-                optIA_NA = new TSrvOptIA_NA(CfgMgr, AddrMgr, (Ptr*)ptrOpt, 
-                    rebind->getAddr(), duidOpt->getDUID(),
-                    rebind->getIface(), addrCount, RENEW_MSG,
-                    this);
-		if (optIA_NA->getStatusCode() != STATUSCODE_NOBINDING )
-		    this->Options.append((Ptr*)optIA_NA);
-		else {
-		    this->IsDone = true;
-		    Log(Notice) << "REBIND received with unknown addresses and " 
-			      << "was silently discarded." << LogEnd;
-		    return;
-		}
-                break;
+        case OPTION_IA_NA: 
+          {
+            SmartPtr<TSrvOptIA_NA> optIA_NA;
+            optIA_NA = new TSrvOptIA_NA(CfgMgr, AddrMgr, (Ptr*)ptrOpt, 
+                                        rebind->getAddr(), duidOpt->getDUID(),
+                                        rebind->getIface(), addrCount, REBIND_MSG,
+                                        this);
+            if (optIA_NA->getStatusCode() != STATUSCODE_NOBINDING )
+              this->Options.append((Ptr*)optIA_NA);
+            else {
+              this->IsDone = true;
+              Log(Notice) << "REBIND received with unknown addresses and " 
+                          << "was silently discarded." << LogEnd;
+              return;
             }
+            break;
+          }
+        case OPTION_IA_PD: {
+          SmartPtr<TSrvOptIA_PD> pd;
+          pd = new TSrvOptIA_PD(CfgMgr, AddrMgr, (Ptr*) ptrOpt, rebind->getAddr(), 
+                                duidOpt->getDUID(), rebind->getIface(), REQUEST_MSG, this);
+          this->Options.append((Ptr*)pd);
+          break;
+        }
+
         case OPTION_IAADDR:
         case OPTION_RAPID_COMMIT:
         case OPTION_STATUS_CODE:
@@ -316,8 +358,15 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
 
 }
 
-// used as RELEASE replay
-TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr, 
+/** 
+ * this constructor is used to create REPLY message as a response for RELEASE message
+ * 
+ * @param ifaceMgr 
+ * @param transMgr 
+ * @param CfgMgr 
+ * @param AddrMgr 
+ * @param release 
+ */TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr, 
 			   SmartPtr<TSrvTransMgr> transMgr, 
 			   SmartPtr<TSrvCfgMgr> CfgMgr, 
 			   SmartPtr<TSrvAddrMgr> AddrMgr,
@@ -335,9 +384,8 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
     //Identifier option containing the server's DUID, the Client Identifier
     //option from the client message, and no other options.
 
-    SmartPtr<TSrvOptIA_NA> clntIA;
     SmartPtr<TSrvOptClientIdentifier> clntID;
-    SmartPtr<TOpt> opt;
+    SmartPtr<TOpt> opt, subOpt;
 
     release->firstOption();
     opt = release->getOption(OPTION_SERVERID);
@@ -348,67 +396,142 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
     Options.append(opt);
     clntID=(Ptr*) opt;
 
+    SmartPtr<TAddrClient> client = AddrMgr->getClient(clntID->getDUID());
+    if (!client) {
+	Log(Warning) << "Received RELEASE from unknown client." << LogEnd;
+	IsDone = true;
+	return;
+    }
+
+    SmartPtr<TSrvCfgIface> ptrIface = SrvCfgMgr->getIfaceByID( this->Iface );
+    if (!ptrIface) {
+	Log(Crit) << "Msg received through not configured interface. "
+	    "Somebody call an exorcist!" << LogEnd;
+	IsDone = true;
+	return;
+    }
+
     release->firstOption();
     while(opt=release->getOption()) {
-        if (opt->getOptType()!=OPTION_IA)
-	    continue;
-
-	clntIA=(Ptr*) opt;
-	SmartPtr<TSrvOptIAAddress> addr;
-	clntIA->firstOption();
-	SmartPtr<TOpt> subOpt;
-	bool anyDeleted=false;
-	clntIA->firstOption();
-	while(subOpt=clntIA->getOption()) {
-	    if (subOpt->getOptType()!=OPTION_IAADDR)
-		continue;
+	switch (opt->getOptType()) {
+	case OPTION_IA_NA: {
+	    SmartPtr<TSrvOptIA_NA> clntIA = (Ptr*) opt;
+	    SmartPtr<TSrvOptIAAddress> addr;
+	    bool anyDeleted=false;
 	    
-	    addr=(Ptr*) subOpt;                
-	    SmartPtr<TAddrClient> ptrClient = AddrMgr->getClient(clntID->getDUID());
-	    if (!ptrClient) {
-		Log(Warning) << "Received RELEASE from unknown client." << LogEnd;
-		IsDone = true;
-		return;
-	    }
-	    SmartPtr<TAddrIA> ptrIA = ptrClient->getIA(clntIA->getIAID() );
+	    // does this client has IA? (iaid check)
+	    SmartPtr<TAddrIA> ptrIA = client->getIA(clntIA->getIAID() );
 	    if (!ptrIA) {
-		Log(Warning) << "No such IA(" << clntIA->getIAID() 
-			  << ") found for client:" << *clntID->getDUID() << LogEnd;
-		Options.append( new TSrvOptIA_NA(clntIA->getIAID(), 0, 0, STATUSCODE_NOBINDING,
-						 "No such IA is bound.",this) );
-	    } else {
-		CfgMgr->delClntAddr(this->Iface,addr->getAddr());
-		if (AddrMgr->delClntAddr(clntID->getDUID(),
-					 clntIA->getIAID(),
-					 addr->getAddr(),
-					 false) )
+		Log(Warning) << "No such IA (iaid=" << clntIA->getIAID() << ") found for client:" << *clntID->getDUID() << LogEnd;
+		Options.append( new TSrvOptIA_NA(clntIA->getIAID(), 0, 0, STATUSCODE_NOBINDING,"No such IA is bound.",this) );
+		continue;
+	    }
+
+	    // if there was DNS Update performed, execute deleting Update
+	    SPtr<TFQDN> fqdn = ptrIA->getFQDN();
+	    if (fqdn) {
+		this->fqdnRelease(ptrIface, ptrIA, fqdn);
+	    }
+
+	    // let's verify each address
+	    clntIA->firstOption();
+	    while(subOpt=clntIA->getOption()) {
+		if (subOpt->getOptType()!=OPTION_IAADDR)
+		    continue;
+		addr = (Ptr*) subOpt;
+		if (AddrMgr->delClntAddr(clntID->getDUID(), clntIA->getIAID(), addr->getAddr(), false) ) {
+		    CfgMgr->delClntAddr(this->Iface,addr->getAddr());
 		    anyDeleted=true;                    
-		else {
-		    Log(Warning) << "No such binding found: "
-				 << "IAID:" <<clntIA->getIAID() 
-				 << "Address:"<< addr->getAddr()->getPlain()
-				 << "DUID:"<< clntID->getDUID()->getPlain()
-				 << LogEnd;
+		} else {
+		    Log(Warning) << "No such binding found: client=" << clntID->getDUID()->getPlain() << ", IA (iaid=" 
+				 << clntIA->getIAID() << "), addr="<< addr->getAddr()->getPlain() << LogEnd;
 		};
 	    };
-	};
-	//After all the addresses have been processed, the server generates a
-	//Reply message and includes a Status Code option with value Success,a
-	//Server Identifier option with the server's DUID, and a Client
-	//Identifier option with the client's DUID.  For each IA in the Release
-	//message for which the server has no binding information, the server
-	//adds an IA option using the IAID from the Release message, and
-	//includes a Status Code option with the value NoBinding in the IA
-	//option.  No other options are included in the IA option.
-	if (!anyDeleted)
-	{
-	    SmartPtr<TSrvOptIA_NA> ansIA(new TSrvOptIA_NA(clntIA->getIAID(),
-							  clntIA->getT1(),clntIA->getT2(),this));
-	    Options.append((Ptr*)ansIA);
-	    ansIA->addOption(new TSrvOptStatusCode(STATUSCODE_NOBINDING,
-						   "Not every address had binding.",this));
-	};
-    };
+
+	    // send result to the client
+	    if (!anyDeleted)
+	    {
+		SmartPtr<TSrvOptIA_NA> ansIA(new TSrvOptIA_NA(clntIA->getIAID(), clntIA->getT1(),clntIA->getT2(),this));
+		Options.append((Ptr*)ansIA);
+		ansIA->addOption(new TSrvOptStatusCode(STATUSCODE_NOBINDING, "Not every address had binding.",this));
+	    };
+	    break;
+	}
+	case OPTION_IA_TA: {
+	    SmartPtr<TSrvOptTA> ta = (Ptr*) opt;
+	    bool anyDeleted = false;
+	    SmartPtr<TAddrIA> ptrIA = client->getTA(ta->getIAID() );
+	    if (!ptrIA) {
+		Log(Warning) << "No such TA (iaid=" << ta->getIAID() << ") found for client:" << *clntID->getDUID() << LogEnd;
+		Options.append( new TSrvOptTA(ta->getIAID(), STATUSCODE_NOBINDING, "No such IA is bound.", this) );
+		continue;
+	    }
+	    
+	    // let's verify each address
+	    ta->firstOption();
+	    while ( subOpt = (Ptr*) ta->getOption() ) {
+		if (subOpt->getOptType()!=OPTION_IAADDR)
+		    continue;
+		SmartPtr<TSrvOptIAAddress> addr = (Ptr*) subOpt;
+		if (AddrMgr->delTAAddr(clntID->getDUID(), ta->getIAID(), addr->getAddr()) ) {
+		    CfgMgr->delTAAddr(this->Iface);
+		    anyDeleted=true;                    
+		} else {
+		    Log(Warning) << "No such binding found: client=" << clntID->getDUID()->getPlain() << ", TA (iaid=" 
+			     << ta->getIAID() << "), addr="<< addr->getAddr()->getPlain() << LogEnd;
+		};
+	    }
+	    
+	    // send results to the client
+	    if (!anyDeleted)
+	    {
+		SmartPtr<TSrvOptTA> answerTA = new TSrvOptTA(ta->getIAID(), STATUSCODE_NOBINDING, "Not every address had binding.", this);
+		Options.append((Ptr*)answerTA);
+	    };
+	    break;
+	}
+
+	case OPTION_IA_PD: {
+	    SmartPtr<TSrvOptIA_PD> pd = (Ptr*) opt;
+	    SmartPtr<TSrvOptIAPrefix> prefix;
+	    bool anyDeleted=false;
+	    
+	    // does this client has PD? (iaid check)
+	    SmartPtr<TAddrIA> ptrPD = client->getPD( pd->getIAID() );
+	    if (!ptrPD) {
+		Log(Warning) << "No such PD (iaid=" << pd->getIAID() << ") found for client:" << *clntID->getDUID() << LogEnd;
+		Options.append( new TSrvOptIA_PD(pd->getIAID(), 0, 0, STATUSCODE_NOBINDING,"No such PD is bound.",this) );
+		continue;
+	    }
+
+	    // let's verify each address
+	    pd->firstOption();
+	    while(subOpt=pd->getOption()) {
+		if (subOpt->getOptType()!=OPTION_IAPREFIX)
+		    continue;
+		prefix = (Ptr*) subOpt;
+		if (AddrMgr->delPrefix(clntID->getDUID(), pd->getIAID(), prefix->getPrefix(), false) ) {
+		    CfgMgr->decrPrefixCount(Iface, prefix->getPrefix());
+		    anyDeleted=true;                    
+		} else {
+		    Log(Warning) << "PD: No such binding found: client=" << clntID->getDUID()->getPlain() << ", PD (iaid=" 
+				 << pd->getIAID() << "), addr="<< prefix->getPrefix()->getPlain() << LogEnd;
+		};
+	    };
+
+	    // send result to the client
+	    if (!anyDeleted)
+	    {
+		SmartPtr<TSrvOptIA_PD> ansPD(new TSrvOptIA_PD(pd->getIAID(), 0, 0, this));
+		Options.append((Ptr*)ansPD);
+		ansPD->addOption(new TSrvOptStatusCode(STATUSCODE_NOBINDING, "Not every address had binding.",this));
+	    };
+	    break;
+	}
+	default:
+	    break;
+	}; // switch(...)
+    } // while 
     
     Options.append(new TSrvOptStatusCode(STATUSCODE_SUCCESS,
 					 "All IAs in RELEASE message were processed.",this));
@@ -438,6 +561,11 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
     unsigned long addrCount=0;
     SmartPtr<TOpt> ptrOpt;
     setOptionsReqOptClntDUID((Ptr*)renew);
+    if (!duidOpt) {
+	Log(Warning) << "RENEW message without client-id option received, message dropped." << LogEnd;
+	IsDone = true;
+	return;
+    }
     
     renew->firstOption();
     while (ptrOpt = renew->getOption() ) 
@@ -447,15 +575,25 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
         case OPTION_SERVERID:
             this->Options.append(ptrOpt);
             break;
-        case OPTION_IA: 
-            {
-                SmartPtr<TSrvOptIA_NA> optIA_NA;
-                optIA_NA = new TSrvOptIA_NA(CfgMgr, AddrMgr, (Ptr*)ptrOpt, 
-                    renew->getAddr(), duidOpt->getDUID(),
-                    renew->getIface(), addrCount, RENEW_MSG, this);
-                this->Options.append((Ptr*)optIA_NA);
-            }
+        case OPTION_IA_NA: {
+	    SmartPtr<TSrvOptIA_NA> optIA_NA;
+	    optIA_NA = new TSrvOptIA_NA(CfgMgr, AddrMgr, (Ptr*)ptrOpt, 
+					renew->getAddr(), duidOpt->getDUID(),
+					renew->getIface(), addrCount, RENEW_MSG, this);
+	    this->Options.append((Ptr*)optIA_NA);
             break;
+	}
+	case OPTION_IA_PD: {
+	    SPtr<TSrvOptIA_PD> optPD;
+	    optPD = new TSrvOptIA_PD(CfgMgr, AddrMgr, (Ptr*)ptrOpt,
+				     renew->getAddr(), duidOpt->getDUID(),
+				     renew->getIface(), RENEW_MSG, this);
+	    Options.append( (Ptr*) optPD);
+	    break;
+	}
+	case OPTION_IA_TA:
+	    Log(Warning) << "TA option present. Temporary addreses cannot be renewed." << LogEnd;
+	    break;
         //Invalid options in RENEW message
         case OPTION_RELAY_MSG :
         case OPTION_INTERFACE_ID :
@@ -479,7 +617,15 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
     this->send();
 }
 
-// used as REQUEST reply
+/** 
+ * this constructor is used to construct REPLY for REQUEST message
+ * 
+ * @param ifaceMgr 
+ * @param transMgr 
+ * @param CfgMgr 
+ * @param AddrMgr 
+ * @param request 
+ */
 TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr, 
 			   SmartPtr<TSrvTransMgr> transMgr, 
 			   SmartPtr<TSrvCfgMgr> CfgMgr, 
@@ -503,6 +649,12 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
     clntIface = request->getIface();
 
     setOptionsReqOptClntDUID((Ptr*)request);
+    if (!duidOpt) {
+	Log(Warning) << "REQUEST message without client-id option received, message dropped." << LogEnd;
+	IsDone = true;
+	return;
+    }
+
 
     // is this client supported?
     if (!CfgMgr->isClntSupported(clntDuid, clntAddr, clntIface)) {
@@ -513,26 +665,37 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
         return;
     }
 
-    //So if we can do something for this client at least set configuration
-    //parameters - let's do  option by option - try to answer to it
+    // try to provide answer for all options
     request->firstOption();
-    while ( opt = request->getOption() ) 
-    {
-        switch (opt->getOptType()) 
-        {
-	case OPTION_SERVERID    :
+    while ( opt = request->getOption() ) {
+	switch (opt->getOptType()) {
+	case OPTION_SERVERID    : {
 	    this->Options.append(opt);
 	    break;
-	case OPTION_IA          : 
-	{
-	    SmartPtr<TSrvOptIA_NA> optIA_NA;
-	    optIA_NA=new TSrvOptIA_NA(AddrMgr, CfgMgr, (Ptr*) opt, 
-				      clntDuid, clntAddr,
-				      clntIface, REQUEST_MSG,this);
-	    this->Options.append((Ptr*)optIA_NA);
+	}
+	case OPTION_IA_NA: {
+	    SmartPtr<TSrvOptIA_NA> optIA;
+	    optIA = new TSrvOptIA_NA(AddrMgr, CfgMgr, (Ptr*) opt, clntDuid, 
+				     clntAddr, clntIface, REQUEST_MSG, this);
+	    this->Options.append((Ptr*)optIA);
 	    break;
 	}
-	case OPTION_STATUS_CODE : 
+	case OPTION_IA_TA: {
+	    SmartPtr<TSrvOptTA> ta;
+	    ta = new TSrvOptTA(AddrMgr, CfgMgr, (Ptr*) opt, clntDuid, clntAddr,
+			       clntIface, REQUEST_MSG, this);
+	    this->Options.append((Ptr*)ta);
+	    break;
+	}
+
+	case OPTION_IA_PD: {
+	    SmartPtr<TSrvOptIA_PD> pd;
+	    pd = new TSrvOptIA_PD(CfgMgr, AddrMgr, (Ptr*) opt, clntAddr, clntDuid, 
+			       clntIface, REQUEST_MSG, this);
+	    this->Options.append((Ptr*)pd);
+	    break;
+	}
+	case OPTION_STATUS_CODE :
 	case OPTION_RELAY_MSG   :
 	case OPTION_PREFERENCE  :
 	case OPTION_RECONF_MSG  :
@@ -542,11 +705,36 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
 	case OPTION_RAPID_COMMIT:
 	    Log(Warning) << "Invalid option " << opt->getOptType()<<" received." << LogEnd;
 	    break;
+	case OPTION_FQDN : {
+	    SmartPtr<TSrvOptFQDN> requestFQDN = (Ptr*) opt;
+	    SmartPtr<TOptFQDN> anotherFQDN = (Ptr*) opt;
+	    SmartPtr<TSrvOptFQDN> optFQDN;
+
+	    string hint = anotherFQDN->getFQDN();
+		
+	    SPtr<TIPv6Addr> clntAssignedAddr = SrvAddrMgr->getFirstAddr(clntDuid);
+	    if (clntAssignedAddr)
+		optFQDN = this->prepareFQDN(requestFQDN, clntDuid, clntAssignedAddr, hint, true);
+	    else
+		optFQDN = this->prepareFQDN(requestFQDN, clntDuid, clntAddr, hint, true);
+	    if (optFQDN) {
+		this->Options.append((Ptr*) optFQDN);
+	    }
+	    break;
+	}
+	case OPTION_VENDOR_OPTS:
+	{
+	    SPtr<TSrvOptVendorSpec> v = (Ptr*) opt;
+	    appendVendorSpec(clntDuid, clntIface, v->getVendor(), reqOpts);
+	    break;
+	}
+
 	default:
 	    appendDefaultOption(opt);
 	    break;
-        }
+	}
     }
+
     appendRequestedOptions(clntDuid, clntAddr, clntIface, reqOpts);
     pkt = new char[this->getSize()];
     IsDone = false;
@@ -579,6 +767,11 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
     clntIface = solicit->getIface();
 
     setOptionsReqOptClntDUID((Ptr*)solicit);
+    if (!duidOpt) {
+	Log(Warning) << "SOLICIT message without client-id option received, message dropped." << LogEnd;
+	IsDone = true;
+	return;
+    }
 
     // include our DUID 
     SmartPtr<TSrvOptServerIdentifier> srvDUID=new TSrvOptServerIdentifier(CfgMgr->getDUID(),this);
@@ -608,15 +801,20 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
 	    Log(Warning) << "Solicit message contains ServerID option. Rejecting message. " << LogEnd;
 	    IsDone = true;
 	    return;
-	case OPTION_IA          : 
-	{
+	case OPTION_IA_NA       : {
 	    SmartPtr<TSrvOptIA_NA> optIA_NA;
-	    optIA_NA=new TSrvOptIA_NA(AddrMgr, CfgMgr, (Ptr*) opt, 
-				      clntDuid, clntAddr, 
-				      clntIface, REQUEST_MSG,this);
+	    optIA_NA = new TSrvOptIA_NA(AddrMgr, CfgMgr, (Ptr*) opt, clntDuid, clntAddr, 
+					clntIface, REQUEST_MSG,this);
 	    this->Options.append((Ptr*)optIA_NA);
 	    break;
 	}
+	case OPTION_IA_TA: {
+	    SmartPtr<TSrvOptTA> ta;
+	    ta = new TSrvOptTA(AddrMgr, CfgMgr, (Ptr*) opt, clntDuid, clntAddr,
+			       clntIface, REQUEST_MSG, this);
+	    this->Options.append((Ptr*)ta);
+	    break;
+	}
 	case OPTION_STATUS_CODE : 
 	case OPTION_RELAY_MSG   :
 	case OPTION_PREFERENCE  :
@@ -674,8 +872,9 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
             case OPTION_PREFERENCE  :
             case OPTION_UNICAST     :
             case OPTION_RECONF_MSG  :
-            case OPTION_IA          : 
+            case OPTION_IA_NA       : 
             case OPTION_IA_TA       :
+            case OPTION_AUTH        :
                 Log(Warning) << "Invalid option " << ptrOpt->getOptType() <<" received." << LogEnd;
                 break;
             default:
@@ -683,7 +882,13 @@ TSrvMsgReply::TSrvMsgReply(SmartPtr<TSrvIfaceMgr> ifaceMgr,
             break;
         }
     }
-    if (appendRequestedOptions(duidOpt->getDUID(),question->getAddr(),question->getIface(),reqOpts)) {
+    SPtr<TDUID> duid;
+    if (duidOpt)
+	duid = duidOpt->getDUID();
+    else
+	duid = new TDUID();
+    
+    if (appendRequestedOptions(duid, question->getAddr(),question->getIface(),reqOpts)) {
         // include our DUID
         SmartPtr<TSrvOptServerIdentifier> srvDUID=new TSrvOptServerIdentifier(CfgMgr->getDUID(),this);
         this->Options.append((Ptr*)srvDUID);
@@ -710,6 +915,7 @@ unsigned long TSrvMsgReply::getTimeout() {
 }
 
 bool TSrvMsgReply::check() {
+    /* not used on the server side */
     return false;
 }
 
@@ -734,7 +940,11 @@ void TSrvMsgReply::appendDefaultOption(SmartPtr<TOpt> ptrOpt) {
     }
 }
 
-// finds CLIENT_DUID and OPTIONREQUEST in options
+/** 
+ * finds CLIENT_DUID and OPTIONREQUEST in options
+ * 
+ * @param msg - parent message
+ */
 void TSrvMsgReply::setOptionsReqOptClntDUID(SmartPtr<TMsg> msg)
 {
     reqOpts= SmartPtr<TSrvOptOptionRequest>();
@@ -758,3 +968,4 @@ TSrvMsgReply::~TSrvMsgReply()
 string TSrvMsgReply::getName() {
     return "REPLY";
 }
+
