@@ -6,47 +6,7 @@
  *                                                                           
  * released under GNU GPL v2 or later licence                                
  *                                                                           
- * $Id: SrvOptIA_NA.cpp,v 1.13 2004/11/15 20:46:11 thomson Exp $
- *
- * $Log: SrvOptIA_NA.cpp,v $
- * Revision 1.13  2004/11/15 20:46:11  thomson
- * Server no longer crashes when receives IA without IAADDR option (bug #67)
- *
- * Revision 1.12  2004/10/27 22:07:56  thomson
- * Signed/unsigned issues fixed, Lifetime option implemented, INFORMATION-REQUEST
- * message is now sent properly. Valid lifetime granted by server fixed.
- *
- * Revision 1.11  2004/10/25 20:45:54  thomson
- * Option support, parsers rewritten. ClntIfaceMgr now handles options.
- *
- * Revision 1.10  2004/09/08 21:22:46  thomson
- * Parser improvements, signed/unsigned issues addressed.
- *
- * Revision 1.9  2004/07/05 23:04:08  thomson
- * *** empty log message ***
- *
- * Revision 1.8  2004/07/05 00:12:30  thomson
- * Lots of minor changes.
- *
- * Revision 1.7  2004/06/28 22:37:59  thomson
- * Minor changes.
- *
- * Revision 1.6  2004/06/21 23:08:49  thomson
- * Minor fixes.
- *
- * Revision 1.5  2004/06/20 21:00:45  thomson
- * Various fixes.
- *
- * Revision 1.4  2004/06/20 19:29:23  thomson
- * New address assignment finally works.
- *
- * Revision 1.3  2004/06/17 23:53:55  thomson
- * Server Address Assignment rewritten.
- *
- * Revision 1.2  2004/06/06 22:31:44  thomson
- * *** empty log message ***
- *
- *                                                                           
+ * $Id: SrvOptIA_NA.cpp,v 1.24 2007-09-06 23:10:39 thomson Exp $
  */
 
 #ifdef WIN32
@@ -88,7 +48,7 @@ TSrvOptIA_NA::TSrvOptIA_NA( char * buf, int bufsize, TMsg* parent)
         pos+=2;
         if ((code>0)&&(code<=24))
         {                
-            if(allowOptInOpt(parent->getType(),OPTION_IA,code)) {
+            if(allowOptInOpt(parent->getType(),OPTION_IA_NA,code)) {
                 SmartPtr<TOpt> opt;
 		opt = SmartPtr<TOpt>(); /* NULL */
                 switch (code)
@@ -123,11 +83,17 @@ TSrvOptIA_NA::TSrvOptIA_NA( char * buf, int bufsize, TMsg* parent)
     }
 }
 
-/*
- * Constructor used in aswers to:
- * - SOLICIT 
- * - SOLICIT (with RAPID_COMMIT)
- * - REQUEST
+/** 
+ * This constructor is used to create IA option as an aswer to a SOLICIT, SOLICIT (RAPID_COMMIT) or REQUEST
+ * 
+ * @param addrMgr 
+ * @param cfgMgr 
+ * @param queryOpt 
+ * @param clntDuid 
+ * @param clntAddr 
+ * @param iface 
+ * @param msgType 
+ * @param parent 
  */
 TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr> cfgMgr,
 			   SmartPtr<TSrvOptIA_NA> queryOpt,
@@ -144,22 +110,31 @@ TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr>
     // FIXME: SOLICIT without RAPID COMMIT should set this to true
     bool quiet = false;
 
+
+    // --- check if client already has binding
+    if (renew(queryOpt, false)) {
+      Log(Info) << "Previous binding for client " << ClntDuid->getPlain() << ", IA(iaid=" 
+                << queryOpt->getIAID() << ") found and renewed." << LogEnd;
+      return;
+    }
+
+
     // --- Is this IA without IAADDR options? ---
     if (!queryOpt->countAddrs()) {
-	Log(Warning) << "IA option (with IAADDR suboptions missing) received. Assigning one address."
-		     << LogEnd;
-	
-	SmartPtr<TIPv6Addr> anyaddr = new TIPv6Addr();
-	assignAddr(anyaddr, DHCPV6_INFINITY, DHCPV6_INFINITY, quiet);
-	
-       	// include status code
-        SmartPtr<TSrvOptStatusCode> ptrStatus;
-        ptrStatus = new TSrvOptStatusCode(STATUSCODE_SUCCESS,
-					  "1 addr granted. Next time include "
-					  "IAADDR in IA option, please.",this->Parent);
-        this->SubOptions.append((Ptr*)ptrStatus);
-	
-	return;
+      Log(Notice) << "IA option (with IAADDR suboptions missing) received. Assigning one address."
+                  << LogEnd;
+      
+      SmartPtr<TIPv6Addr> anyaddr = new TIPv6Addr();
+      this->assignAddr(anyaddr, DHCPV6_INFINITY, DHCPV6_INFINITY, quiet);
+      
+      // include status code
+      SmartPtr<TSrvOptStatusCode> ptrStatus;
+      ptrStatus = new TSrvOptStatusCode(STATUSCODE_SUCCESS,
+                                        "1 addr granted. Next time you might consider including "
+                                        "IAADDR in IA option.",this->Parent);
+      this->SubOptions.append((Ptr*)ptrStatus);
+      
+      return;
     }
 
     // --- check address counts, how many we've got, how many assigned etc. ---
@@ -177,21 +152,21 @@ TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr>
     willAssign = addrsRequested;
 
     if (willAssign > addrsMax - addrsAssigned) {
-	Log(Notice) << "Client got " << addrsAssigned << " and requested " 
-		    << addrsRequested << " more, but limit for a client is "
-		    << addrsMax << LogEnd;
-	willAssign = addrsMax - addrsAssigned;
+      Log(Notice) << "Client got " << addrsAssigned << " and requested " 
+                  << addrsRequested << " more, but limit for a client is "
+                  << addrsMax << LogEnd;
+      willAssign = addrsMax - addrsAssigned;
     }
 
     if (willAssign > addrsAvail) {
-	Log(Notice) << willAssign << " addrs would be assigned, but only" << addrsAssigned
-		    << " is available." << LogEnd;
-	willAssign = addrsAvail;
+      Log(Notice) << willAssign << " addrs would be assigned, but only" << addrsAssigned
+                  << " is available." << LogEnd;
+      willAssign = addrsAvail;
     }
 
     Log(Info) << "Client has " << addrsAssigned << " addrs, asks for " 
-	      << addrsRequested << ", " << addrsAvail << " is available, limit for client is "
-	      << addrsMax << ", " << willAssign << " will be assigned." << LogEnd;
+              << addrsRequested << ", " << addrsAvail << " is available, limit for client is "
+              << addrsMax << ", " << willAssign << " will be assigned." << LogEnd;
 
     // --- ok, let's assign those damn addresses ---
     SmartPtr<TOpt> opt;
@@ -203,11 +178,11 @@ TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr>
     queryOpt->firstOption();
     while ( opt = queryOpt->getOption() ) {
 	switch ( opt->getOptType() ) {
-	case OPTION_IAADDR: {
-	    optAddr = (Ptr*) opt;
-	    hint    = optAddr->getAddr();
-	    
-	    if (willAssign) {
+      case OPTION_IAADDR: {
+      optAddr = (Ptr*) opt;
+      hint    = optAddr->getAddr();
+	  
+      if (willAssign) {
 		// we've got free addrs left, assign one of them
 		// always register this address as used by this client
 		// (if this is solicit, this addr will be released later)
@@ -216,40 +191,46 @@ TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr>
 		this->assignAddr(hint, pref, valid, quiet);
 		willAssign--;
 		addrsAssigned++;
-
-	    } else {
+        
+      } else {
 		ok = false;
-	    }
-	    break;
-	}
-	case OPTION_STATUS_CODE: {
+      }
+      break;
+      }
+      case OPTION_STATUS_CODE: {
 	    SmartPtr<TOptStatusCode> ptrStatus = (Ptr*) opt;
 	    Log(Notice) << "Receviced STATUS_CODE code=" 
-			<<  ptrStatus->getCode() << ", message=(" << ptrStatus->getText()
-			<< ")" << LogEnd;
+                    <<  ptrStatus->getCode() << ", message=(" << ptrStatus->getText()
+                    << ")" << LogEnd;
 	    break;
-	}
-	default: {
+      }
+      default: {
 	    Log(Warning) << "Invalid suboption (" << opt->getOptType() 
-			 << ") in an OPTION_IA option received. Option ignored." << LogEnd;
+                     << ") in an OPTION_IA_NA option received. Option ignored." << LogEnd;
 	    break;
-	}
+      }
 	}
     }
-
+    
     // --- now include STATUS CODE ---
     SmartPtr<TSrvOptStatusCode> ptrStatus;
     if (ok) {
-	ptrStatus = new TSrvOptStatusCode(STATUSCODE_SUCCESS,
-					  "All addresses were assigned.",this->Parent);
-	// FIXME: if this is solicit, place "all addrs would be assigned."
+      ptrStatus = new TSrvOptStatusCode(STATUSCODE_SUCCESS,
+                                        "All addresses were assigned.",this->Parent);
+      // FIXME: if this is solicit, place "all addrs would be assigned."
     } else {
-	string tmp = addrsRequested+" addrs requested, but assigned only "+addrsAssigned;
-	ptrStatus = new TSrvOptStatusCode(STATUSCODE_NOADDRSAVAIL,tmp.c_str(), this->Parent);
+	char buf[60];
+	sprintf(buf, "%lu addr(s) requested, but assigned only %lu.",addrsRequested, addrsAssigned);
+	if (addrsAssigned) {
+	    ptrStatus = new TSrvOptStatusCode(STATUSCODE_SUCCESS,buf, this->Parent);
+	} else {
+	    ptrStatus = new TSrvOptStatusCode(STATUSCODE_NOADDRSAVAIL,buf, this->Parent);
+	}
+
     }
     SubOptions.append((Ptr*)ptrStatus);
-
-    // --- release addresses if this reply for SOLICIT) ---
+    
+    // if this is a ADVERTISE message, release those addresses in TSrvMsgAdvertise::answer() method
 }
 
 void TSrvOptIA_NA::releaseAllAddrs(bool quiet) {
@@ -282,6 +263,10 @@ SmartPtr<TSrvOptIAAddress> TSrvOptIA_NA::assignAddr(SmartPtr<TIPv6Addr> hint, un
     pref = ptrClass->getPref(pref);
     valid= ptrClass->getValid(valid);
     optAddr = new TSrvOptIAAddress(addr, pref, valid, this->Parent);
+    if (ptrClass->getAddrParams()) {
+	Log(Debug) << "Experimental: addr-params subotion added." << LogEnd;
+	optAddr->addOption((Ptr*)ptrClass->getAddrParams());
+    }
     SubOptions.append((Ptr*)optAddr);
 
     Log(Info) << "Client requested " << *hint <<", got " << *addr 
@@ -325,7 +310,7 @@ TSrvOptIA_NA::TSrvOptIA_NA( SmartPtr<TSrvCfgMgr> cfgMgr,
         break;
     
     case RENEW_MSG:
-        this->renew(queryOpt, addrCount);
+        this->renew(queryOpt, true);
         break;
     case REBIND_MSG:
         this->rebind(queryOpt, addrCount);
@@ -341,7 +326,7 @@ TSrvOptIA_NA::TSrvOptIA_NA( SmartPtr<TSrvCfgMgr> cfgMgr,
         break;
     default: {
 	Log(Warning) << "Unknown message type (" << msgType 
-		     << "). Cannot generate OPTION_IA."<< LogEnd;
+		     << "). Cannot generate OPTION_IA_NA."<< LogEnd;
         SubOptions.append(new TSrvOptStatusCode(STATUSCODE_UNSPECFAIL,
 						"Unknown message type.",this->Parent));
         break;
@@ -349,32 +334,48 @@ TSrvOptIA_NA::TSrvOptIA_NA( SmartPtr<TSrvCfgMgr> cfgMgr,
     }
 }
 
-// generate OPTION_IA based on OPTION_IA received in RENEW message
-void TSrvOptIA_NA::renew(SmartPtr<TSrvOptIA_NA> queryOpt, unsigned long &addrCount)
+// 
+/** 
+ * generate OPTION_IA_NA based on OPTION_IA_NA received in RENEW message
+ * 
+ * @param queryOpt - IA_NA option in the RENEW message
+ * @param addrCount 
+ *
+ * @return true - if binding was renewed, false - if not found or invalid
+ */
+bool TSrvOptIA_NA::renew(SmartPtr<TSrvOptIA_NA> queryOpt, bool complainIfMissing)
 {
     // find that client in addrdb
     SmartPtr <TAddrClient> ptrClient;
     ptrClient = this->AddrMgr->getClient(this->ClntDuid);
     if (!ptrClient) {
+      if (complainIfMissing) {
         SubOptions.append(new TSrvOptStatusCode(STATUSCODE_NOBINDING,"Who are you? Do I know you?",
-						this->Parent));
-        return;
+                                                this->Parent));
+        Log(Info) << "Unable to RENEW binding for IA(iaid=" << queryOpt->getIAID() << ", client="
+                  << ClntDuid->getPlain() << ": No such client." << LogEnd;
+      }
+      return false;
     }
 
     // find that IA
     SmartPtr <TAddrIA> ptrIA;
     ptrIA = ptrClient->getIA(this->IAID);
     if (!ptrIA) {
+      if (complainIfMissing) {
         SubOptions.append(new TSrvOptStatusCode(STATUSCODE_NOBINDING,"I see this IAID first time.",
-						this->Parent ));
-        return;
+                                                this->Parent ));
+        Log(Info) << "Unable to RENEW binding for IA(iaid=" << queryOpt->getIAID() << ", client="
+                  << ClntDuid->getPlain() << ": No such IA." << LogEnd;
+      }
+      return false;
     }
 
     // everything seems ok, update data in addrdb
     ptrIA->setTimestamp();
     this->T1 = ptrIA->getT1();
     this->T2 = ptrIA->getT2();
-
+    
     // send addr info to client
     SmartPtr<TAddrAddr> ptrAddr;
     ptrIA->firstAddr();
@@ -388,8 +389,10 @@ void TSrvOptIA_NA::renew(SmartPtr<TSrvOptIA_NA> queryOpt, unsigned long &addrCou
 
     // finally send greetings and happy OK status code
     SmartPtr<TSrvOptStatusCode> ptrStatus;
-    ptrStatus = new TSrvOptStatusCode(STATUSCODE_SUCCESS,"Greetings from planet Earth",this->Parent);
+    ptrStatus = new TSrvOptStatusCode(STATUSCODE_SUCCESS,"Address(es) renewed. Greetings from planet Earth",this->Parent);
     SubOptions.append( (Ptr*)ptrStatus );
+
+    return true;
 }
 
 void TSrvOptIA_NA::rebind(SmartPtr<TSrvOptIA_NA> queryOpt,
@@ -408,7 +411,7 @@ void TSrvOptIA_NA::rebind(SmartPtr<TSrvOptIA_NA> queryOpt,
     // find that IA
     SmartPtr <TAddrIA> ptrIA;
     ptrIA = ptrClient->getIA(this->IAID);
-    if (!ptrClient) {
+    if (!ptrIA) {
         SubOptions.append(new TSrvOptStatusCode(STATUSCODE_NOBINDING,
 						"I see this IAID first time.",this->Parent ));
         return;
@@ -472,9 +475,9 @@ SmartPtr<TIPv6Addr> TSrvOptIA_NA::getFreeAddr(SmartPtr<TIPv6Addr> hint) {
 		   << this->Iface << LogEnd;
 	return 0; // NULL
     }
-
+    
     // check if this address is ok
-
+    
     // is it anyaddress (::)?
     SmartPtr<TIPv6Addr> anyaddr = new TIPv6Addr();
     if (*anyaddr==*hint) {
@@ -482,7 +485,7 @@ SmartPtr<TIPv6Addr> TSrvOptIA_NA::getFreeAddr(SmartPtr<TIPv6Addr> hint) {
 		   << ") address. Hint ignored." << LogEnd;
 	invalidAddr = true;
     }
-
+    
     // is it multicast address (ff...)?
     if ((*(hint->getAddr()))==0xff) {
 	Log(Debug) << "Client requested multicast (" << *hint 
@@ -522,11 +525,24 @@ SmartPtr<TIPv6Addr> TSrvOptIA_NA::getFreeAddr(SmartPtr<TIPv6Addr> hint) {
 		do {
 		    addr = ptrClass->getRandomAddr();
 		} while (!AddrMgr->addrIsFree(addr));
-	    return addr;
+		return addr;
 	    }
 	}
     }
 
+    // do we have a cached address for that client?
+    if (addr = AddrMgr->getCachedAddr(this->ClntDuid)) {
+	if (this->CfgMgr->getClassByAddr(this->Iface, addr)) {
+	    Log(Info) << "Cache: Cached address " << *addr << " found. Welcome back." << LogEnd;
+	    if (AddrMgr->addrIsFree(addr))
+		return addr;
+	    Log(Info) << "Unfortunately, " << addr->getPlain() << " is used." << LogEnd;
+	} else {
+	    Log(Warning) << "Cache: Cached address " << *addr << " found, but it is no longer valid." << LogEnd;
+	    AddrMgr->delCachedAddr(addr);
+	}
+    }
+    
     // worst case: address does not belong to supported class
     // or specified hint is invalid
     SmartPtr<TSrvCfgAddrClass> ptrClass;
@@ -535,7 +551,8 @@ SmartPtr<TIPv6Addr> TSrvOptIA_NA::getFreeAddr(SmartPtr<TIPv6Addr> hint) {
 	ptrIface->firstAddrClass();
 	ptrClass = ptrIface->getAddrClass();
     }
-    
+
+    // FIXME: Check if there are any addresses available at all?
     do {
 	addr = ptrClass->getRandomAddr();
     } while (!AddrMgr->addrIsFree(hint));
