@@ -6,9 +6,18 @@
  *
  * released under GNU GPL v2 or later licence
  *
- * $Id: SrvOptIA_NA.h,v 1.7 2004/07/05 00:12:30 thomson Exp $
+ * $Id: SrvOptIA_NA.h,v 1.9 2007-05-04 17:47:25 thomson Exp $
  *
  * $Log: SrvOptIA_NA.h,v $
+ * Revision 1.9  2007-05-04 17:47:25  thomson
+ * *** empty log message ***
+ *
+ * Revision 1.8  2006-03-05 21:32:28  thomson
+ * Temp. addresses support merged into main branch.
+ *
+ * Revision 1.7.2.1  2006/02/05 23:38:08  thomson
+ * Devel branch with Temporary addresses support added.
+ *
  * Revision 1.7  2004/07/05 00:12:30  thomson
  * Lots of minor changes.
  *
@@ -29,7 +38,6 @@ class TSrvOptIA_NA;
 #define SRVOPTIA_NA_H
 
 #include "OptIA_NA.h"
-#include "SrvOptIA_NA.h"
 #include "SrvOptIAAddress.h"
 #include "SmartPtr.h"
 #include "DUID.h"
@@ -65,7 +73,7 @@ class TSrvOptIA_NA : public TOptIA_NA
 
     void solicit(SmartPtr<TSrvOptIA_NA> queryOpt, unsigned long &addrCount);
     void request(SmartPtr<TSrvOptIA_NA> queryOpt, unsigned long &addrCount);
-    void renew(SmartPtr<TSrvOptIA_NA> queryOpt,   unsigned long &addrCount);
+    bool renew(SmartPtr<TSrvOptIA_NA> queryOpt, bool complainIfMissing);
     void rebind(SmartPtr<TSrvOptIA_NA> queryOpt,  unsigned long &addrCount);
     void release(SmartPtr<TSrvOptIA_NA> queryOpt, unsigned long &addrCount);
     void confirm(SmartPtr<TSrvOptIA_NA> queryOpt, unsigned long &addrCount);
@@ -78,15 +86,10 @@ class TSrvOptIA_NA : public TOptIA_NA
     SmartPtr<TDUID>       ClntDuid;
     int                   Iface;
     
-    SmartPtr<TSrvOptIAAddress> assignAddr(SmartPtr<TIPv6Addr> hint, 
-					  unsigned long pref,
-					  unsigned long valid,
-					  bool quiet);
+    SmartPtr<TSrvOptIAAddress> assignAddr(SmartPtr<TIPv6Addr> hint, unsigned long pref,
+					  unsigned long valid, bool quiet);
     SmartPtr<TIPv6Addr> getFreeAddr(SmartPtr<TIPv6Addr> hint);
 
 };
 
-
-
-
 #endif
